/**
 * Фабрика операций и построитель батчей
 */
import { generateId } from '../utils/helpers';
import { BatchOperationType, BatchPriority } from '../types/batch-interfaces';
/**
 * Реализация фабрики операций
 */
export class DefaultBatchOperationFactory {
    /**
     * Создать операцию создания элемента
     */
    createElementOperation(elementData) {
        return {
            id: generateId(),
            type: BatchOperationType.CREATE,
            priority: BatchPriority.NORMAL,
            params: { elementData },
            execute: async (context) => {
                // Здесь должна быть логика создания элемента
                // В реальной реализации это будет вызов CSElement API
                const element = {
                    id: generateId(),
                    ...elementData,
                    createdAt: new Date(),
                    updatedAt: new Date()
                };
                context.sharedData.set(`created_${element.id}`, element);
                return element;
            },
            validate: async (context) => {
                const errors = [];
                const warnings = [];
                if (!context.operation.params?.elementData) {
                    errors.push('Отсутствуют данные для создания элемента');
                }
                if (context.operation.params?.elementData && !context.operation.params.elementData.type) {
                    warnings.push('Не указан тип элемента, будет использован тип по умолчанию');
                }
                return {
                    valid: errors.length === 0,
                    errors,
                    warnings
                };
            },
            metadata: {
                operationType: 'createElement',
                description: 'Создание нового элемента'
            }
        };
    }
    /**
     * Создать операцию обновления элемента
     */
    updateElementOperation(element, updates) {
        return {
            id: generateId(),
            type: BatchOperationType.UPDATE,
            priority: BatchPriority.NORMAL,
            target: element,
            params: { updates },
            execute: async (context) => {
                const target = context.operation.target;
                if (!target) {
                    throw new Error('Целевой элемент не найден');
                }
                // Применяем обновления
                const updatedElement = {
                    ...target,
                    ...context.operation.params?.updates,
                    updatedAt: new Date()
                };
                context.sharedData.set(`updated_${target.id}`, updatedElement);
                return updatedElement;
            },
            validate: async (context) => {
                const errors = [];
                const warnings = [];
                if (!context.operation.target) {
                    errors.push('Не указан целевой элемент для обновления');
                }
                if (!context.operation.params?.updates) {
                    errors.push('Отсутствуют данные для обновления');
                }
                return {
                    valid: errors.length === 0,
                    errors,
                    warnings
                };
            },
            metadata: {
                operationType: 'updateElement',
                description: `Обновление элемента ${element.id}`,
                elementId: element.id
            }
        };
    }
    /**
     * Создать операцию удаления элемента
     */
    deleteElementOperation(element) {
        return {
            id: generateId(),
            type: BatchOperationType.DELETE,
            priority: BatchPriority.HIGH,
            target: element,
            execute: async (context) => {
                const target = context.operation.target;
                if (!target) {
                    throw new Error('Целевой элемент не найден');
                }
                // Помечаем элемент как удаленный
                const deletedElement = {
                    ...target,
                    deleted: true,
                    deletedAt: new Date()
                };
                context.sharedData.set(`deleted_${target.id}`, deletedElement);
                return { deleted: true, elementId: target.id };
            },
            rollback: async (context) => {
                const target = context.operation.target;
                if (target) {
                    // Восстанавливаем элемент
                    context.sharedData.set(`restored_${target.id}`, target);
                }
            },
            validate: async (context) => {
                const errors = [];
                const warnings = [];
                if (!context.operation.target) {
                    errors.push('Не указан элемент для удаления');
                }
                return {
                    valid: errors.length === 0,
                    errors,
                    warnings
                };
            },
            metadata: {
                operationType: 'deleteElement',
                description: `Удаление элемента ${element.id}`,
                elementId: element.id
            }
        };
    }
    /**
     * Создать операцию перемещения элемента
     */
    moveElementOperation(element, newParent) {
        return {
            id: generateId(),
            type: BatchOperationType.MOVE,
            priority: BatchPriority.NORMAL,
            target: element,
            params: { newParent },
            execute: async (context) => {
                const target = context.operation.target;
                const newParent = context.operation.params?.newParent;
                if (!target || !newParent) {
                    throw new Error('Не указан элемент или новый родитель');
                }
                const movedElement = {
                    ...target,
                    parentId: newParent.id,
                    updatedAt: new Date()
                };
                context.sharedData.set(`moved_${target.id}`, movedElement);
                return movedElement;
            },
            validate: async (context) => {
                const errors = [];
                const warnings = [];
                if (!context.operation.target) {
                    errors.push('Не указан элемент для перемещения');
                }
                if (!context.operation.params?.newParent) {
                    errors.push('Не указан новый родительский элемент');
                }
                return {
                    valid: errors.length === 0,
                    errors,
                    warnings
                };
            },
            metadata: {
                operationType: 'moveElement',
                description: `Перемещение элемента ${element.id} к ${newParent.id}`,
                elementId: element.id,
                newParentId: newParent.id
            }
        };
    }
    /**
     * Создать операцию копирования элемента
     */
    copyElementOperation(element, target) {
        return {
            id: generateId(),
            type: BatchOperationType.COPY,
            priority: BatchPriority.NORMAL,
            target: element,
            params: { target },
            execute: async (context) => {
                const source = context.operation.target;
                const target = context.operation.params?.target;
                if (!source || !target) {
                    throw new Error('Не указан исходный элемент или цель копирования');
                }
                const copiedElement = {
                    ...source,
                    id: generateId(),
                    parentId: target.id,
                    createdAt: new Date(),
                    updatedAt: new Date()
                };
                context.sharedData.set(`copied_${copiedElement.id}`, copiedElement);
                return copiedElement;
            },
            validate: async (context) => {
                const errors = [];
                const warnings = [];
                if (!context.operation.target) {
                    errors.push('Не указан элемент для копирования');
                }
                if (!context.operation.params?.target) {
                    errors.push('Не указана цель копирования');
                }
                return {
                    valid: errors.length === 0,
                    errors,
                    warnings
                };
            },
            metadata: {
                operationType: 'copyElement',
                description: `Копирование элемента ${element.id} в ${target.id}`,
                sourceElementId: element.id,
                targetElementId: target.id
            }
        };
    }
    /**
     * Создать операцию валидации
     */
    validateElementOperation(element) {
        return {
            id: generateId(),
            type: BatchOperationType.VALIDATE,
            priority: BatchPriority.LOW,
            target: element,
            execute: async (context) => {
                const target = context.operation.target;
                if (!target) {
                    throw new Error('Целевой элемент не найден');
                }
                // Выполняем валидацию элемента
                const validationResult = {
                    elementId: target.id,
                    valid: true,
                    errors: [],
                    warnings: []
                };
                // Базовая валидация
                if (!target.id) {
                    validationResult.valid = false;
                    validationResult.errors.push('Отсутствует ID элемента');
                }
                if (!target.name) {
                    validationResult.warnings.push('Не указано имя элемента');
                }
                context.sharedData.set(`validation_${target.id}`, validationResult);
                return validationResult;
            },
            metadata: {
                operationType: 'validateElement',
                description: `Валидация элемента ${element.id}`,
                elementId: element.id
            }
        };
    }
    /**
     * Создать пользовательскую операцию
     */
    customOperation(id, executor) {
        return {
            id,
            type: BatchOperationType.CUSTOM,
            priority: BatchPriority.NORMAL,
            execute: executor,
            metadata: {
                operationType: 'custom',
                description: `Пользовательская операция ${id}`
            }
        };
    }
}
/**
 * Построитель батчей
 */
export class DefaultBatchBuilder {
    constructor(batchManager, operationFactory) {
        this.config = {};
        this.batchManager = batchManager;
        this.operationFactory = operationFactory || new DefaultBatchOperationFactory();
        this.batchId = this.batchManager.createBatch();
    }
    /**
     * Установить конфигурацию
     */
    withConfig(config) {
        this.config = { ...this.config, ...config };
        // Создаем новый батч с обновленной конфигурацией
        this.batchManager.removeBatch(this.batchId);
        this.batchId = this.batchManager.createBatch(this.config);
        return this;
    }
    /**
     * Добавить операцию
     */
    addOperation(operation) {
        this.batchManager.addOperation(this.batchId, operation);
        return this;
    }
    /**
     * Добавить операции
     */
    addOperations(operations) {
        this.batchManager.addOperations(this.batchId, operations);
        return this;
    }
    /**
     * Добавить операцию создания
     */
    createElement(elementData) {
        const operation = this.operationFactory.createElementOperation(elementData);
        return this.addOperation(operation);
    }
    /**
     * Добавить операцию обновления
     */
    updateElement(element, updates) {
        const operation = this.operationFactory.updateElementOperation(element, updates);
        return this.addOperation(operation);
    }
    /**
     * Добавить операцию удаления
     */
    deleteElement(element) {
        const operation = this.operationFactory.deleteElementOperation(element);
        return this.addOperation(operation);
    }
    /**
     * Добавить условную операцию
     */
    when(condition, operation) {
        // Оборачиваем операцию в условную логику
        const conditionalOperation = {
            ...operation,
            id: `conditional_${operation.id}`,
            execute: async (context) => {
                if (condition(context)) {
                    return operation.execute(context);
                }
                else {
                    return { skipped: true, reason: 'Условие не выполнено' };
                }
            },
            metadata: {
                ...operation.metadata,
                conditional: true,
                originalOperationId: operation.id
            }
        };
        return this.addOperation(conditionalOperation);
    }
    /**
     * Построить и выполнить батч
     */
    async execute() {
        return this.batchManager.executeBatch(this.batchId);
    }
    /**
     * Построить батч без выполнения
     */
    build() {
        return this.batchId;
    }
}
/**
 * Утилитарные функции для создания операций
 */
export const BatchOperations = {
    /**
     * Создать операцию задержки
     */
    delay(ms) {
        return {
            id: generateId(),
            type: BatchOperationType.CUSTOM,
            priority: BatchPriority.LOW,
            params: { delay: ms },
            execute: async () => {
                await new Promise(resolve => setTimeout(resolve, ms));
                return { delayed: ms };
            },
            metadata: {
                operationType: 'delay',
                description: `Задержка на ${ms}мс`
            }
        };
    },
    /**
     * Создать операцию логирования
     */
    log(message, level = 'info') {
        return {
            id: generateId(),
            type: BatchOperationType.CUSTOM,
            priority: BatchPriority.LOW,
            params: { message, level },
            execute: async (context) => {
                const logEntry = {
                    timestamp: new Date(),
                    level,
                    message,
                    batchId: context.batchId,
                    operationId: context.operation.id
                };
                console[level](message);
                context.sharedData.set(`log_${context.operation.id}`, logEntry);
                return logEntry;
            },
            metadata: {
                operationType: 'log',
                description: `Логирование: ${message}`
            }
        };
    },
    /**
     * Создать операцию проверки условия
     */
    assert(condition, message) {
        return {
            id: generateId(),
            type: BatchOperationType.VALIDATE,
            priority: BatchPriority.HIGH,
            params: { condition, message },
            execute: async (context) => {
                const result = context.operation.params?.condition(context);
                if (!result) {
                    throw new Error(context.operation.params?.message || 'Условие не выполнено');
                }
                return { assertion: true, message: context.operation.params?.message };
            },
            metadata: {
                operationType: 'assert',
                description: `Проверка условия: ${message}`
            }
        };
    }
};
//# sourceMappingURL=BatchOperationFactory.js.map