import { EventEmitter } from 'events';
/**
 * Менеджер шаблонов для управления коллекцией Blueprint
 */
export class BlueprintManager extends EventEmitter {
    constructor(factory, config = {}) {
        super();
        this.blueprints = new Map();
        this.cache = new Map();
        this.usageStats = new Map();
        this.factory = factory;
        this.config = {
            autoRegisterBuiltins: true,
            enableCaching: true,
            maxCacheSize: 100,
            defaultValidation: true,
            enableLogging: false,
            ...config
        };
        // Автоматическая регистрация встроенных шаблонов
        if (this.config.autoRegisterBuiltins) {
            this.registerBuiltinBlueprints();
        }
    }
    /**
     * Регистрация встроенных шаблонов
     */
    async registerBuiltinBlueprints() {
        try {
            const builtinTypes = this.factory.getAvailableTypes();
            for (const type of builtinTypes) {
                try {
                    const blueprint = this.factory.createBlueprint(type);
                    await this.registerBlueprint(blueprint);
                }
                catch (error) {
                    if (this.config.enableLogging) {
                        console.warn(`Не удалось зарегистрировать встроенный шаблон ${type}:`, error);
                    }
                }
            }
        }
        catch (error) {
            if (this.config.enableLogging) {
                console.error('Ошибка регистрации встроенных шаблонов:', error);
            }
        }
    }
    /**
     * Регистрация шаблона
     */
    async registerBlueprint(blueprint) {
        try {
            // Проверяем что шаблон не зарегистрирован
            if (this.blueprints.has(blueprint.id)) {
                throw new Error(`Шаблон с ID '${blueprint.id}' уже зарегистрирован`);
            }
            // Регистрируем шаблон
            this.blueprints.set(blueprint.id, blueprint);
            // Инициализируем статистику
            if (!this.usageStats.has(blueprint.id)) {
                this.usageStats.set(blueprint.id, {
                    blueprintId: blueprint.id,
                    usageCount: 0,
                    lastUsed: new Date(),
                    popularParameterCombinations: []
                });
            }
            // Эмитируем событие
            this.emit('blueprint-registered', { blueprint });
            if (this.config.enableLogging) {
                console.log(`Зарегистрирован шаблон: ${blueprint.name} (${blueprint.id})`);
            }
        }
        catch (error) {
            this.emit('blueprint-error', {
                blueprintId: blueprint.id,
                error: error instanceof Error ? error : new Error(String(error))
            });
            throw error;
        }
    }
    /**
     * Получение шаблона по ID
     */
    async getBlueprint(id) {
        return this.blueprints.get(id) || null;
    }
    /**
     * Получение всех зарегистрированных шаблонов
     */
    async getAllBlueprints() {
        return Array.from(this.blueprints.values());
    }
    /**
     * Поиск шаблонов
     */
    async searchBlueprints(criteria) {
        const startTime = Date.now();
        let blueprints = Array.from(this.blueprints.values());
        // Фильтрация по тексту
        if (criteria.query) {
            const query = criteria.query.toLowerCase();
            blueprints = blueprints.filter(bp => bp.name.toLowerCase().includes(query) ||
                bp.description.toLowerCase().includes(query) ||
                bp.metadata.tags?.some(tag => tag.toLowerCase().includes(query)));
        }
        // Фильтрация по типу
        if (criteria.type && criteria.type.length > 0) {
            blueprints = blueprints.filter(bp => criteria.type.includes(bp.type));
        }
        // Фильтрация по категории
        if (criteria.category && criteria.category.length > 0) {
            blueprints = blueprints.filter(bp => criteria.category.includes(bp.category));
        }
        // Фильтрация по тегам
        if (criteria.tags && criteria.tags.length > 0) {
            blueprints = blueprints.filter(bp => criteria.tags.some(tag => bp.metadata.tags?.includes(tag)));
        }
        // Фильтрация по автору
        if (criteria.author) {
            blueprints = blueprints.filter(bp => bp.metadata.author?.toLowerCase().includes(criteria.author.toLowerCase()));
        }
        // Сортировка
        if (criteria.sortBy) {
            blueprints.sort((a, b) => {
                let aValue, bValue;
                switch (criteria.sortBy) {
                    case 'name':
                        aValue = a.name;
                        bValue = b.name;
                        break;
                    case 'created':
                        aValue = a.metadata.created || new Date(0);
                        bValue = b.metadata.created || new Date(0);
                        break;
                    case 'updated':
                        aValue = a.metadata.updated || new Date(0);
                        bValue = b.metadata.updated || new Date(0);
                        break;
                    case 'popularity':
                        aValue = this.usageStats.get(a.id)?.usageCount || 0;
                        bValue = this.usageStats.get(b.id)?.usageCount || 0;
                        break;
                    default:
                        return 0;
                }
                if (criteria.sortOrder === 'desc') {
                    return aValue > bValue ? -1 : aValue < bValue ? 1 : 0;
                }
                else {
                    return aValue < bValue ? -1 : aValue > bValue ? 1 : 0;
                }
            });
        }
        // Пагинация
        const total = blueprints.length;
        const offset = criteria.offset || 0;
        const limit = criteria.limit || total;
        blueprints = blueprints.slice(offset, offset + limit);
        const searchTime = Date.now() - startTime;
        return {
            blueprints,
            total,
            searchTime,
            criteria
        };
    }
    /**
     * Удаление шаблона
     */
    async unregisterBlueprint(id) {
        const blueprint = this.blueprints.get(id);
        if (!blueprint) {
            return false;
        }
        this.blueprints.delete(id);
        this.usageStats.delete(id);
        // Очищаем кэш связанный с этим шаблоном
        for (const [key] of this.cache.entries()) {
            if (key.startsWith(id + ':')) {
                this.cache.delete(key);
            }
        }
        this.emit('blueprint-unregistered', { blueprintId: id });
        if (this.config.enableLogging) {
            console.log(`Удален шаблон: ${blueprint.name} (${id})`);
        }
        return true;
    }
    /**
     * Генерация структуры из шаблона
     */
    async generateFromBlueprint(blueprintId, parameters, options) {
        const blueprint = this.blueprints.get(blueprintId);
        if (!blueprint) {
            throw new Error(`Шаблон с ID '${blueprintId}' не найден`);
        }
        try {
            // Генерируем ключ кэша
            const cacheKey = this.config.enableCaching ?
                `${blueprintId}:${JSON.stringify(parameters)}:${JSON.stringify(options || {})}` :
                null;
            // Проверяем кэш
            if (cacheKey && this.cache.has(cacheKey)) {
                const cachedResult = this.cache.get(cacheKey);
                if (this.config.enableLogging) {
                    console.log(`Использован кэшированный результат для шаблона ${blueprintId}`);
                }
                return cachedResult;
            }
            // Генерируем результат
            const result = await blueprint.generate({
                parameters,
                parentElement: options?.parentElement,
                idPrefix: options?.idPrefix,
                options: {
                    validateParameters: this.config.defaultValidation,
                    generateMetadata: true,
                    preserveExisting: false,
                    dryRun: options?.dryRun || false
                }
            });
            // Кэшируем результат
            if (cacheKey && !options?.dryRun) {
                this.addToCache(cacheKey, result);
            }
            // Обновляем статистику
            if (!options?.dryRun) {
                this.updateUsageStatistics(blueprintId, parameters);
            }
            this.emit('blueprint-generated', { blueprintId, result });
            if (this.config.enableLogging) {
                console.log(`Сгенерирована структура из шаблона ${blueprint.name}: ${result.elements.length} элементов`);
            }
            return result;
        }
        catch (error) {
            this.emit('blueprint-error', {
                blueprintId,
                error: error instanceof Error ? error : new Error(String(error))
            });
            throw error;
        }
    }
    /**
     * Валидация параметров шаблона
     */
    async validateBlueprintParameters(blueprintId, parameters) {
        const blueprint = this.blueprints.get(blueprintId);
        if (!blueprint) {
            throw new Error(`Шаблон с ID '${blueprintId}' не найден`);
        }
        return blueprint.validateParameters(parameters);
    }
    /**
     * Получение статистики использования
     */
    async getUsageStatistics(blueprintId) {
        if (blueprintId) {
            const stats = this.usageStats.get(blueprintId);
            return stats ? [stats] : [];
        }
        return Array.from(this.usageStats.values());
    }
    /**
     * Экспорт шаблона
     */
    async exportBlueprint(blueprintId) {
        const blueprint = this.blueprints.get(blueprintId);
        if (!blueprint) {
            throw new Error(`Шаблон с ID '${blueprintId}' не найден`);
        }
        const exportData = {
            id: blueprint.id,
            name: blueprint.name,
            description: blueprint.description,
            type: blueprint.type,
            category: blueprint.category,
            version: blueprint.version,
            metadata: blueprint.metadata,
            parameterGroups: blueprint.parameterGroups,
            exportedAt: new Date().toISOString(),
            exportedBy: 'CSElement BlueprintManager'
        };
        return JSON.stringify(exportData, null, 2);
    }
    /**
     * Импорт шаблона
     */
    async importBlueprint(blueprintData) {
        try {
            const data = JSON.parse(blueprintData);
            // Валидация импортируемых данных
            if (!data.id || !data.name || !data.type || !data.category) {
                throw new Error('Неполные данные шаблона для импорта');
            }
            // Создаем шаблон через фабрику (если это встроенный тип) или как пользовательский
            let blueprint;
            if (this.factory.getAvailableTypes().includes(data.type)) {
                blueprint = this.factory.createBlueprint(data.type, data);
            }
            else {
                // Создаем пользовательский шаблон
                blueprint = this.factory.createCustomBlueprint({
                    id: data.id,
                    name: data.name,
                    description: data.description,
                    generator: async () => ({ elements: [], warnings: [] }), // Заглушка
                    parameters: data.parameterGroups?.flatMap((g) => g.parameters) || [],
                    metadata: data.metadata
                });
            }
            // Регистрируем импортированный шаблон
            await this.registerBlueprint(blueprint);
            return blueprint;
        }
        catch (error) {
            throw new Error(`Ошибка импорта шаблона: ${error instanceof Error ? error.message : String(error)}`);
        }
    }
    /**
     * Очистка кэша
     */
    async clearCache() {
        this.cache.clear();
        this.emit('cache-cleared', {});
        if (this.config.enableLogging) {
            console.log('Кэш шаблонов очищен');
        }
    }
    /**
     * Получение рекомендаций шаблонов
     */
    async getRecommendations(context) {
        const allBlueprints = Array.from(this.blueprints.values());
        // Простая логика рекомендаций на основе популярности и истории использования
        const recommendations = allBlueprints
            .map(bp => {
            let score = 0;
            // Популярность
            const stats = this.usageStats.get(bp.id);
            if (stats) {
                score += stats.usageCount * 0.1;
            }
            // История использования
            if (context?.usageHistory?.includes(bp.id)) {
                score += 5;
            }
            // Категория и теги (простая эвристика)
            if (bp.category === 'business')
                score += 1;
            if (bp.metadata.tags?.includes('popular'))
                score += 2;
            return { blueprint: bp, score };
        })
            .sort((a, b) => b.score - a.score)
            .slice(0, 5)
            .map(item => item.blueprint);
        return recommendations;
    }
    /**
     * Добавление в кэш с учетом лимитов
     */
    addToCache(key, value) {
        if (!this.config.enableCaching)
            return;
        // Проверяем лимит кэша
        if (this.cache.size >= (this.config.maxCacheSize || 100)) {
            // Удаляем самый старый элемент (простая стратегия FIFO)
            const firstKey = this.cache.keys().next().value;
            if (firstKey) {
                this.cache.delete(firstKey);
            }
        }
        this.cache.set(key, value);
    }
    /**
     * Обновление статистики использования
     */
    updateUsageStatistics(blueprintId, parameters) {
        let stats = this.usageStats.get(blueprintId);
        if (!stats) {
            stats = {
                blueprintId,
                usageCount: 0,
                lastUsed: new Date(),
                popularParameterCombinations: []
            };
        }
        stats.usageCount++;
        stats.lastUsed = new Date();
        // Обновляем популярные комбинации параметров
        if (!stats.popularParameterCombinations) {
            stats.popularParameterCombinations = [];
        }
        const existingCombo = stats.popularParameterCombinations.find(combo => JSON.stringify(combo.parameters) === JSON.stringify(parameters));
        if (existingCombo) {
            existingCombo.count++;
        }
        else {
            stats.popularParameterCombinations.push({
                parameters: { ...parameters },
                count: 1
            });
        }
        // Ограничиваем количество сохраняемых комбинаций
        stats.popularParameterCombinations = stats.popularParameterCombinations
            .sort((a, b) => b.count - a.count)
            .slice(0, 10);
        this.usageStats.set(blueprintId, stats);
        this.emit('statistics-updated', { statistics: [stats] });
    }
}
//# sourceMappingURL=BlueprintManager.js.map