import { EventEmitter } from 'eventemitter3';
import { ElementEventType } from '../types/interfaces';
import { generateId } from '../utils/helpers';
import { AsyncLock } from '../utils/AsyncLock';
import { defaultValidator } from '../utils/Validator';
import { QueryEngine, CommonSelectors } from '../utils/QueryEngine';
import { AdvancedQueryEngine } from '../utils/AdvancedQueryEngine';
import { PluginManager } from './PluginManager';
import { PersistenceManagerImpl } from '../persistence/PersistenceManagerImpl';
import { HistoryManagerImpl } from '../history/HistoryManagerImpl';
import { ReactivityManagerImpl } from '../reactivity/ReactivityManagerImpl';
import { LiveQueryManagerImpl, LiveQueryBuilderImpl } from '../live-queries/LiveQueryManagerImpl';
import { GraphAlgorithmsManagerImpl } from '../graph/GraphAlgorithmsManagerImpl';
/**
 * Основная реализация CSElement
 * Потокобезопасный класс для работы с графовыми структурами
 */
export class CSElement extends EventEmitter {
    /**
     * Получить количество всех элементов в системе
     */
    static get totalElementsCount() {
        return CSElement._globalRegistry.size;
    }
    /**
     * Получить все элементы из глобального реестра
     */
    static getAllElements() {
        return Array.from(CSElement._globalRegistry.values());
    }
    constructor(name = '', options = {}) {
        super();
        this._id = generateId();
        this._name = name;
        this._index = options.index ?? 0;
        this._data = options.data || new Map();
        this._elements = new Map();
        this._elementsByIndex = [];
        this._owners = new Map();
        this._mainOwner = null;
        this._lock = new AsyncLock();
        this._lastElement = null;
        // Регистрируем элемент в глобальном реестре
        CSElement._globalRegistry.set(this._id, this);
        // Выполняем хук afterCreate синхронно
        CSElement._pluginManager?.executeHooks('afterCreate', this);
    }
    // Геттеры для readonly свойств
    get id() {
        return this._id;
    }
    get name() {
        return this._name;
    }
    get index() {
        return this._index;
    }
    get data() {
        return new Map(this._data);
    }
    get mainOwner() {
        return this._mainOwner;
    }
    get lastElement() {
        return this._lastElement;
    }
    /**
     * Получить владельца по имени или индексу
     */
    getOwner(nameOrIndex) {
        if (typeof nameOrIndex === 'string') {
            return this._owners.get(nameOrIndex) || null;
        }
        else {
            const owners = Array.from(this._owners.values());
            return owners[nameOrIndex] || null;
        }
    }
    /**
     * Асинхронно получить владельца
     */
    async getOwnerAsync(nameOrIndex) {
        return this._lock.withLock(async () => {
            return this.getOwner(nameOrIndex);
        });
    }
    /**
     * Получить элемент по имени или индексу
     */
    getElement(nameOrIndex) {
        if (typeof nameOrIndex === 'string') {
            return this._elements.get(nameOrIndex) || null;
        }
        else {
            return this._elementsByIndex[nameOrIndex] || null;
        }
    }
    /**
     * Асинхронно получить элемент
     */
    async getElementAsync(nameOrIndex) {
        return this._lock.withLock(async () => {
            return this.getElement(nameOrIndex);
        });
    }
    /**
     * Получить относительный индекс для владельца
     */
    getRelativeIndex(owner) {
        if (!(owner instanceof CSElement)) {
            return -1;
        }
        const ownerElement = owner;
        const elements = Array.from(ownerElement._elements.values());
        return elements.findIndex(el => el._id === this._id);
    }
    /**
     * Количество дочерних элементов
     */
    elementsCount() {
        return this._elementsByIndex.length;
    }
    /**
     * Количество владельцев
     */
    ownersCount() {
        return this._owners.size;
    }
    /**
     * Получить все дочерние элементы
     */
    getAllElements() {
        return Array.from(this._elementsByIndex);
    }
    /**
     * Получить всех владельцев
     */
    getAllOwners() {
        return Array.from(this._owners.values());
    }
    /**
     * Проверяет наличие дочернего элемента по имени или индексу
     */
    hasElement(nameOrIndex) {
        if (typeof nameOrIndex === 'string') {
            return this._elements.has(nameOrIndex);
        }
        else {
            return nameOrIndex >= 0 && nameOrIndex < this._elementsByIndex.length;
        }
    }
    /**
     * Добавить новый элемент
     */
    async addElement(value, options = {}) {
        const context = {
            element: this,
            operation: 'addElement',
            args: [value, options],
            metadata: {},
            startTime: Date.now(),
            operationId: generateId(),
            middlewareStack: [],
            flags: {
                aborted: false,
                modified: false,
                inTransaction: false
            }
        };
        const operation = async () => {
            return this._lock.withLock(async () => {
                const element = value instanceof CSElement ? value : new CSElement(value, options);
                // Выполняем хук beforeAddElement
                await CSElement._pluginManager?.executeHooks('beforeAddElement', this, element);
                // Устанавливаем имя если не задано
                if (!element._name && options.name) {
                    element._name = options.name;
                }
                // Добавляем в коллекции
                if (element._name) {
                    this._elements.set(element._name, element);
                }
                // Устанавливаем индекс
                element._index = options.index ?? this._elementsByIndex.length;
                // Добавляем в массив по индексу
                if (element._index >= this._elementsByIndex.length) {
                    this._elementsByIndex.push(element);
                }
                else {
                    this._elementsByIndex.splice(element._index, 0, element);
                    // Обновляем индексы последующих элементов
                    for (let i = element._index + 1; i < this._elementsByIndex.length; i++) {
                        this._elementsByIndex[i]._index = i;
                    }
                }
                // Устанавливаем владельца
                element._owners.set(this._id, this);
                if (!element._mainOwner) {
                    element._mainOwner = this;
                }
                // Обновляем последний элемент
                this._lastElement = element;
                // Записываем операцию в историю
                this.recordOperation('create', `Add element ${element.name || element.id}`, null, {
                    id: element.id,
                    name: element.name,
                    index: element.index
                });
                // Генерируем событие
                this.emit(ElementEventType.ElementAdded, element);
                // Уведомляем реактивность об изменении дочерних элементов
                this.notifyChildrenChange('add', element);
                // Выполняем хук afterAddElement
                await CSElement._pluginManager?.executeHooks('afterAddElement', this, element);
                // Уведомляем Live queries о создании элемента
                element.notifyElementChange('create');
                return element;
            });
        };
        // Выполняем через middleware если есть менеджер плагинов
        if (CSElement._pluginManager) {
            return CSElement._pluginManager.executeMiddleware(context, operation);
        }
        else {
            return operation();
        }
    }
    /**
     * Удалить элемент
     */
    async removeElement(nameOrIndex) {
        const context = {
            element: this,
            operation: 'removeElement',
            args: [nameOrIndex],
            metadata: {},
            startTime: Date.now(),
            operationId: generateId(),
            middlewareStack: [],
            flags: {
                aborted: false,
                modified: false,
                inTransaction: false
            }
        };
        const operation = async () => {
            return this._lock.withLock(async () => {
                let element = null;
                if (nameOrIndex instanceof CSElement) {
                    element = nameOrIndex;
                }
                else if (typeof nameOrIndex === 'string' || typeof nameOrIndex === 'number') {
                    element = this.getElement(nameOrIndex);
                }
                else {
                    // nameOrIndex is ICSElement but not CSElement
                    element = nameOrIndex;
                }
                if (!element) {
                    return false;
                }
                // Выполняем хук beforeRemoveElement
                await CSElement._pluginManager?.executeHooks('beforeRemoveElement', this, element);
                // Удаляем из коллекций
                if (element._name) {
                    this._elements.delete(element._name);
                }
                // Удаляем из массива по индексу
                const index = this._elementsByIndex.indexOf(element);
                if (index !== -1) {
                    this._elementsByIndex.splice(index, 1);
                    // Обновляем индексы последующих элементов
                    for (let i = index; i < this._elementsByIndex.length; i++) {
                        this._elementsByIndex[i]._index = i;
                    }
                }
                // Удаляем владельца у элемента
                element._owners.delete(this._id);
                if (element._mainOwner === this) {
                    // Если это был главный владелец, назначаем нового
                    const newOwner = element._owners.values().next().value;
                    element._mainOwner = newOwner || null;
                }
                // Обновляем последний элемент
                if (this._lastElement === element) {
                    this._lastElement = this._elementsByIndex[this._elementsByIndex.length - 1] || null;
                }
                // Записываем операцию в историю
                this.recordOperation('delete', `Remove element ${element.name || element.id}`, {
                    id: element.id,
                    name: element.name,
                    index: element.index
                }, null);
                // Генерируем событие
                this.emit(ElementEventType.ElementRemoved, element);
                // Уведомляем реактивность об изменении дочерних элементов
                this.notifyChildrenChange('remove', element);
                // Выполняем хук afterRemoveElement
                await CSElement._pluginManager?.executeHooks('afterRemoveElement', this, element);
                // Уведомляем Live queries об удалении элемента
                element.notifyElementChange('delete');
                return true;
            });
        };
        // Выполняем через middleware если есть менеджер плагинов
        if (CSElement._pluginManager) {
            return CSElement._pluginManager.executeMiddleware(context, operation);
        }
        else {
            return operation();
        }
    }
    /**
     * Установить данные
     */
    async setData(key, value) {
        const context = {
            element: this,
            operation: 'setData',
            args: [key, value],
            metadata: {},
            startTime: Date.now(),
            operationId: generateId(),
            middlewareStack: [],
            flags: {
                aborted: false,
                modified: false,
                inTransaction: false
            }
        };
        const operation = async () => {
            return this._lock.withLock(async () => {
                // Используем аргументы из контекста (могут быть модифицированы middleware)
                const [finalKey, finalValue] = context.args;
                // Выполняем хук beforeSetData
                await CSElement._pluginManager?.executeHooks('beforeSetData', this, finalKey, finalValue);
                const oldValue = this._data.get(finalKey);
                this._data.set(finalKey, finalValue);
                // Записываем операцию в историю
                this.recordOperation('update', `Set data ${finalKey}`, oldValue, finalValue, [finalKey]);
                // Уведомляем систему реактивности
                this.notifyDataChange(finalKey, finalValue, oldValue);
                this.emit(ElementEventType.DataChanged, { key: finalKey, oldValue, newValue: finalValue });
                // Выполняем хук afterSetData
                await CSElement._pluginManager?.executeHooks('afterSetData', this, finalKey, finalValue);
            });
        };
        // Выполняем через middleware если есть менеджер плагинов
        if (CSElement._pluginManager) {
            return CSElement._pluginManager.executeMiddleware(context, operation);
        }
        else {
            return operation();
        }
    }
    /**
     * Получить данные
     */
    getData(key) {
        return this._data.get(key);
    }
    /**
     * Удалить данные
     */
    async deleteData(key) {
        const context = {
            element: this,
            operation: 'deleteData',
            args: [key],
            metadata: {},
            startTime: Date.now(),
            operationId: generateId(),
            middlewareStack: [],
            flags: {
                aborted: false,
                modified: false,
                inTransaction: false
            }
        };
        const operation = async () => {
            return this._lock.withLock(async () => {
                // Выполняем хук beforeDeleteData
                await CSElement._pluginManager?.executeHooks('beforeDeleteData', this, key);
                const hadKey = this._data.has(key);
                if (hadKey) {
                    const oldValue = this._data.get(key);
                    this._data.delete(key);
                    // Записываем операцию в историю
                    this.recordOperation('delete', `Delete data ${key}`, oldValue, undefined, [key]);
                    // Уведомляем систему реактивности
                    this.notifyDataChange(key, undefined, oldValue);
                    this.emit(ElementEventType.DataChanged, { key, oldValue, newValue: undefined });
                }
                // Выполняем хук afterDeleteData
                await CSElement._pluginManager?.executeHooks('afterDeleteData', this, key);
                return hadKey;
            });
        };
        // Выполняем через middleware если есть менеджер плагинов
        if (CSElement._pluginManager) {
            return CSElement._pluginManager.executeMiddleware(context, operation);
        }
        else {
            return operation();
        }
    }
    /**
     * Войти в блокировку
     */
    async enterLock() {
        await this._lock.acquire();
        this.emit(ElementEventType.Locked);
    }
    /**
     * Выйти из блокировки
     */
    async leaveLock() {
        this._lock.release();
        this.emit(ElementEventType.Unlocked);
    }
    /**
     * Проверить заблокирован ли элемент
     */
    isLocked() {
        return this._lock.isLocked();
    }
    /**
     * Добавить себя как дочерний элемент к другому элементу
     */
    async addTo(owner, options = {}) {
        return owner.addElement(this, options);
    }
    /**
     * Удалить себя из владельца
     */
    async removeFrom(owner) {
        return owner.removeElement(this);
    }
    // ===== Builder API =====
    /**
     * Установить имя элемента (fluent API)
     */
    setName(name) {
        this._name = name;
        return this;
    }
    /**
     * Установить индекс элемента (fluent API)
     */
    setIndex(index) {
        this._index = index;
        return this;
    }
    /**
     * Добавить данные (fluent API)
     */
    withData(key, value) {
        this._data.set(key, value);
        return this;
    }
    /**
     * Добавить несколько данных сразу (fluent API)
     */
    withDataObject(data) {
        Object.entries(data).forEach(([key, value]) => {
            this._data.set(key, value);
        });
        return this;
    }
    /**
     * Добавить дочерний элемент (fluent API)
     */
    withChild(nameOrElement, options) {
        // Используем синхронную версию для fluent API
        const element = nameOrElement instanceof CSElement ? nameOrElement : new CSElement(nameOrElement, options);
        // Устанавливаем имя если не задано
        if (!element._name && options?.name) {
            element._name = options.name;
        }
        // Добавляем в коллекции
        if (element._name) {
            this._elements.set(element._name, element);
        }
        // Устанавливаем индекс
        element._index = options?.index ?? this._elementsByIndex.length;
        // Добавляем в массив по индексу
        if (element._index >= this._elementsByIndex.length) {
            this._elementsByIndex.push(element);
        }
        else {
            this._elementsByIndex.splice(element._index, 0, element);
            // Обновляем индексы последующих элементов
            for (let i = element._index + 1; i < this._elementsByIndex.length; i++) {
                this._elementsByIndex[i]._index = i;
            }
        }
        // Устанавливаем владельца
        element._owners.set(this._id, this);
        if (!element._mainOwner) {
            element._mainOwner = this;
        }
        // Обновляем последний элемент
        this._lastElement = element;
        // Генерируем событие
        this.emit(ElementEventType.ElementAdded, element);
        return this;
    }
    /**
     * Добавить несколько дочерних элементов (fluent API)
     */
    withChildren(...names) {
        names.forEach(name => {
            this.addElement(name).catch(err => {
                console.error('Failed to add child:', err);
            });
        });
        return this;
    }
    // ===== Статические методы для создания =====
    /**
     * Создать новый элемент с fluent API
     */
    static create(name, options) {
        return new CSElement(name || '', options);
    }
    // ===== Глобальный поиск и навигация =====
    /**
     * Найти элемент по ID глобально
     */
    static getElementById(id) {
        return CSElement._globalRegistry.get(id) || null;
    }
    /**
     * Найти все элементы по имени глобально
     */
    static getElementsByName(name) {
        const result = [];
        CSElement._globalRegistry.forEach(element => {
            if (element.name === name) {
                result.push(element);
            }
        });
        return result;
    }
    // ===== Система плагинов =====
    /**
     * Установить плагин
     */
    static use(plugin, options) {
        if (!CSElement._pluginManager) {
            CSElement._pluginManager = new PluginManager();
        }
        CSElement._pluginManager.setCSElementClass(CSElement);
        CSElement._pluginManager.use(plugin, options || undefined);
    }
    /**
     * Удалить плагин
     */
    static uninstall(pluginName) {
        return CSElement._pluginManager?.uninstall(pluginName) || false;
    }
    /**
     * Получить менеджер плагинов
     */
    static get plugins() {
        if (!CSElement._pluginManager) {
            CSElement._pluginManager = new PluginManager();
            CSElement._pluginManager.setCSElementClass(CSElement);
        }
        return CSElement._pluginManager;
    }
    /**
     * Проверить, установлен ли плагин
     */
    static hasPlugin(name) {
        return CSElement._pluginManager?.hasPlugin(name) || false;
    }
    /**
     * Получить информацию о плагинах
     */
    static getPluginInfo() {
        return CSElement._pluginManager?.getPluginInfo() || [];
    }
    // ===== Система персистентности =====
    /**
     * Получить менеджер персистентности
     */
    static get persistence() {
        return CSElement._persistenceManager;
    }
    /**
     * Сохранить элемент в хранилище
     */
    async save(adapterName, options) {
        return CSElement._persistenceManager.save(this.id, adapterName, options);
    }
    /**
     * Загрузить элемент из хранилища и восстановить его состояние
     */
    static async load(elementId, adapterName, options) {
        const result = await CSElement._persistenceManager.load(elementId, adapterName, options);
        if (result.success && result.data) {
            // Десериализуем элемент
            const element = CSElement.deserialize(result.data);
            return element;
        }
        return null;
    }
    /**
     * Удалить элемент из хранилища
     */
    async deleteFromStorage(adapterName, options) {
        return CSElement._persistenceManager.delete(this.id, adapterName, options);
    }
    /**
     * Найти все элементы по условию
     */
    findElements(predicate, includeThis = true) {
        const result = [];
        const visited = new Set();
        const traverse = (element) => {
            if (visited.has(element.id))
                return;
            visited.add(element.id);
            if (predicate(element) && (includeThis || element !== this)) {
                result.push(element);
            }
            element.getAllElements().forEach(child => {
                if (child instanceof CSElement) {
                    traverse(child);
                }
            });
        };
        traverse(this);
        return result;
    }
    /**
     * Найти первый элемент по условию
     */
    findElement(predicate, includeThis = true) {
        const visited = new Set();
        const traverse = (element) => {
            if (visited.has(element.id))
                return null;
            visited.add(element.id);
            if (predicate(element) && (includeThis || element !== this)) {
                return element;
            }
            for (const child of element.getAllElements()) {
                if (child instanceof CSElement) {
                    const found = traverse(child);
                    if (found)
                        return found;
                }
            }
            return null;
        };
        return traverse(this);
    }
    /**
     * Обход всех элементов в глубину
     */
    async traverseDepthFirst(callback, includeThis = true) {
        const visited = new Set();
        const traverse = async (element) => {
            if (visited.has(element.id))
                return;
            visited.add(element.id);
            if (includeThis || element !== this) {
                await callback(element);
            }
            for (const child of element.getAllElements()) {
                if (child instanceof CSElement) {
                    await traverse(child);
                }
            }
        };
        await traverse(this);
    }
    /**
     * Обход всех элементов в ширину
     */
    async traverseBreadthFirst(callback, includeThis = true) {
        const visited = new Set();
        const queue = [this];
        while (queue.length > 0) {
            const element = queue.shift();
            if (visited.has(element.id))
                continue;
            visited.add(element.id);
            if (includeThis || element !== this) {
                await callback(element);
            }
            element.getAllElements().forEach(child => {
                if (child instanceof CSElement && !visited.has(child.id)) {
                    queue.push(child);
                }
            });
        }
    }
    /**
     * Получить путь до элемента
     */
    getPath() {
        const path = [];
        let current = this;
        while (current) {
            path.unshift(current.name);
            current = current.mainOwner;
        }
        return path;
    }
    /**
     * Получить глубину элемента
     */
    getDepth() {
        let depth = 0;
        let current = this.mainOwner;
        while (current) {
            depth++;
            current = current.mainOwner;
        }
        return depth;
    }
    /**
     * Экспорт структуры в JSON
     */
    toJSON(includeData = true) {
        const result = {
            id: this.id,
            name: this.name,
            index: this.index
        };
        if (includeData && this._data.size > 0) {
            result.data = Object.fromEntries(this._data);
        }
        if (this._elements.size > 0) {
            result.elements = this.getAllElements().map(child => {
                if (child instanceof CSElement) {
                    return child.toJSON(includeData);
                }
                return null;
            }).filter(Boolean);
        }
        return result;
    }
    /**
     * Клонировать элемент со всей структурой
     */
    async clone(deep = true) {
        const cloned = new CSElement(this.name, {
            data: new Map(this._data),
            index: this.index
        });
        if (deep) {
            for (const child of this.getAllElements()) {
                if (child instanceof CSElement) {
                    const clonedChild = await child.clone(deep);
                    await cloned.addElement(clonedChild);
                }
            }
        }
        return cloned;
    }
    /**
     * Удалить элемент из глобального реестра (вызывается при уничтожении)
     */
    destroy() {
        // Выполняем хук beforeDestroy синхронно
        CSElement._pluginManager?.executeHooks('beforeDestroy', this);
        // Удаляем из всех владельцев
        this.getAllOwners().forEach(owner => {
            if (owner instanceof CSElement) {
                owner.removeElement(this);
            }
        });
        // Рекурсивно удаляем все дочерние элементы
        this.getAllElements().forEach(child => {
            if (child instanceof CSElement) {
                child.destroy();
            }
        });
        // Удаляем из глобального реестра
        CSElement._globalRegistry.delete(this.id);
        // Удаляем все слушатели событий
        this.removeAllListeners();
        // Выполняем хук afterDestroy синхронно
        CSElement._pluginManager?.executeHooks('afterDestroy', this);
    }
    /**
     * Получить статистику элемента
     */
    getStats() {
        let totalElements = 0;
        let maxDepth = 0;
        let totalData = 0;
        const visited = new Set();
        const traverse = (element, currentDepth = 0) => {
            if (visited.has(element.id))
                return;
            visited.add(element.id);
            totalElements++;
            maxDepth = Math.max(maxDepth, currentDepth);
            totalData += element.data.size;
            element.getAllElements().forEach(child => {
                if (child instanceof CSElement) {
                    traverse(child, currentDepth + 1);
                }
            });
        };
        traverse(this, 0);
        return { totalElements, maxDepth, totalData };
    }
    /**
     * Проверить является ли элемент предком
     */
    isAncestorOf(element) {
        let current = element;
        while (current) {
            if (current === this)
                return true;
            current = current.mainOwner;
        }
        return false;
    }
    /**
     * Проверить является ли элемент потомком
     */
    isDescendantOf(element) {
        return element.isAncestorOf(this);
    }
    // ===== Расширенная сериализация =====
    /**
     * Улучшенная сериализация с опциями
     */
    serialize(options = {}) {
        const { includeChildren = true, includeData = true, includeOwners = false, includeMetadata = true, maxDepth = Infinity, excludeFields = [] } = options;
        const result = {
            id: this.id
        };
        // Добавляем поля если они не исключены
        if (!excludeFields.includes('name')) {
            result.name = this.name;
        }
        if (!excludeFields.includes('index')) {
            result.index = this.index;
        }
        // Добавляем данные
        if (includeData && !excludeFields.includes('data') && this._data.size > 0) {
            result.data = Object.fromEntries(this._data);
        }
        // Добавляем владельцев
        if (includeOwners && !excludeFields.includes('owners')) {
            result.owners = Array.from(this._owners.keys());
        }
        // Добавляем метаданные
        if (includeMetadata && !excludeFields.includes('metadata')) {
            result.metadata = {
                createdAt: Date.now(), // В реальном проекте это должно сохраняться при создании
                updatedAt: Date.now(),
                depth: this.getDepth(),
                path: this.getPath().join('/')
            };
        }
        // Добавляем дочерние элементы
        if (includeChildren && !excludeFields.includes('children') && maxDepth > 0) {
            const children = [];
            for (const child of this.getAllElements()) {
                if (child instanceof CSElement) {
                    children.push(child.serialize({
                        ...options,
                        maxDepth: maxDepth - 1
                    }));
                }
            }
            if (children.length > 0) {
                result.children = children;
            }
        }
        return result;
    }
    /**
     * Десериализация из объекта
     */
    static deserialize(data) {
        const element = new CSElement(data.name || '', {
            index: data.index
        });
        // Восстанавливаем оригинальный ID если он есть
        if (data.id) {
            // Удаляем из глобального реестра текущий ID
            CSElement._globalRegistry.delete(element._id);
            // Устанавливаем новый ID
            element._id = data.id;
            // Регистрируем с новым ID
            CSElement._globalRegistry.set(data.id, element);
        }
        // Восстанавливаем данные
        if (data.data) {
            Object.entries(data.data).forEach(([key, value]) => {
                element._data.set(key, value);
            });
        }
        // Рекурсивно восстанавливаем дочерние элементы
        if (data.children) {
            data.children.forEach(childData => {
                const child = CSElement.deserialize(childData);
                element.withChild(child);
            });
        }
        return element;
    }
    // ===== Валидация =====
    /**
     * Валидирует элемент используя глобальный валидатор
     */
    validate(elementType = 'default') {
        return defaultValidator.validate(this, elementType);
    }
    /**
     * Валидирует элемент используя кастомный валидатор
     */
    validateWith(validator, elementType = 'default') {
        return validator.validate(this, elementType);
    }
    // ===== Система запросов =====
    /**
     * Поиск элементов по CSS-like селектору
     */
    query(selector) {
        return QueryEngine.query(this, selector);
    }
    /**
     * Поиск первого элемента по селектору
     */
    queryOne(selector) {
        return QueryEngine.queryOne(this, selector);
    }
    /**
     * Проверяет, соответствует ли элемент селектору
     */
    matches(selector) {
        const results = QueryEngine.query(this, selector);
        return results.includes(this);
    }
    // ===== Расширенная система запросов =====
    /**
     * Поиск элементов с помощью расширенного движка запросов (CSS селекторы, XPath)
     */
    queryAdvanced(selector) {
        return AdvancedQueryEngine.query(this, selector);
    }
    /**
     * Поиск первого элемента с помощью расширенного движка запросов
     */
    queryAdvancedOne(selector) {
        const results = this.queryAdvanced(selector);
        return results.length > 0 ? results[0] : null;
    }
    /**
     * CSS селектор запрос
     */
    queryCSS(selector) {
        return AdvancedQueryEngine.queryCSS(this, selector);
    }
    /**
     * XPath запрос
     */
    queryXPath(selector) {
        return AdvancedQueryEngine.queryXPath(this, selector);
    }
    /**
     * Получить статистику запросов
     */
    static getQueryStats() {
        return AdvancedQueryEngine.getStats();
    }
    /**
     * Очистить кэш запросов
     */
    static clearQueryCache() {
        AdvancedQueryEngine.clearCache();
    }
    /**
     * Включить/выключить индексирование для оптимизации запросов
     */
    static setIndexingEnabled(enabled) {
        AdvancedQueryEngine.setIndexingEnabled(enabled);
    }
    // ===== Статические методы для работы с селекторами =====
    /**
     * Создает селектор для поиска
     */
    static createSelector() {
        return QueryEngine.createSelector();
    }
    /**
     * Получает предопределенные селекторы
     */
    static get selectors() {
        return CommonSelectors;
    }
    // ===== Утилиты для работы с данными =====
    /**
     * Массовое обновление данных с валидацией
     */
    async updateData(updates, validate = false) {
        return this._lock.withLock(async () => {
            // Сначала валидируем если требуется
            if (validate) {
                // Создаем временную копию для валидации
                const tempElement = await this.clone(false);
                Object.entries(updates).forEach(([key, value]) => {
                    tempElement._data.set(key, value);
                });
                const validationResult = tempElement.validate();
                if (!validationResult.isValid) {
                    return validationResult;
                }
            }
            // Применяем изменения
            const oldValues = {};
            Object.entries(updates).forEach(([key, value]) => {
                oldValues[key] = this._data.get(key);
                this._data.set(key, value);
            });
            // Генерируем событие
            this.emit(ElementEventType.DataChanged, {
                updates,
                oldValues,
                newValues: updates
            });
            return null; // Валидация прошла успешно
        });
    }
    /**
     * Получает все данные как обычный объект
     */
    getDataAsObject() {
        return Object.fromEntries(this._data);
    }
    /**
     * Проверяет наличие данных по ключу
     */
    hasData(key) {
        return this._data.has(key);
    }
    /**
     * Получает размер данных
     */
    getDataSize() {
        return this._data.size;
    }
    /**
     * Очищает все данные
     */
    async clearData() {
        return this._lock.withLock(async () => {
            const oldData = this.getDataAsObject();
            this._data.clear();
            this.emit(ElementEventType.DataChanged, {
                cleared: true,
                oldData
            });
        });
    }
    // === МЕТОДЫ РАБОТЫ С ИСТОРИЕЙ ===
    /**
     * Получить менеджер истории
     */
    static get history() {
        return CSElement._historyManager;
    }
    /**
     * Настроить систему истории
     */
    static configureHistory(config) {
        CSElement._historyManager = new HistoryManagerImpl(config);
    }
    /**
     * Отменить последнюю операцию
     */
    static async undo() {
        return CSElement._historyManager.undo();
    }
    /**
     * Повторить отмененную операцию
     */
    static async redo() {
        return CSElement._historyManager.redo();
    }
    /**
     * Отменить до определенной операции
     */
    static async undoTo(operationId) {
        return CSElement._historyManager.undoTo(operationId);
    }
    /**
     * Повторить до определенной операции
     */
    static async redoTo(operationId) {
        return CSElement._historyManager.redoTo(operationId);
    }
    /**
     * Получить состояние истории
     */
    static getHistoryState() {
        return CSElement._historyManager.getState();
    }
    /**
     * Получить операции истории
     */
    static getHistoryOperations(limit) {
        return CSElement._historyManager.getOperations(limit);
    }
    /**
     * Получить снимки истории
     */
    static getHistorySnapshots(limit) {
        return CSElement._historyManager.getSnapshots(limit);
    }
    /**
     * Создать снимок текущего состояния элемента
     */
    createSnapshot(description = 'Manual snapshot') {
        const data = {
            id: this._id,
            name: this._name,
            index: this._index,
            data: Object.fromEntries(this._data),
            elements: this._elementsByIndex.map(el => el.id),
            owners: Array.from(this._owners.keys())
        };
        return CSElement._historyManager.createSnapshot(data, description);
    }
    /**
     * Очистить историю
     */
    static clearHistory() {
        CSElement._historyManager.clear();
    }
    /**
     * Экспорт истории
     */
    static exportHistory() {
        return CSElement._historyManager.export();
    }
    /**
     * Импорт истории
     */
    static importHistory(data) {
        CSElement._historyManager.import(data);
    }
    /**
     * Подписка на события истории
     */
    static onHistoryEvent(event, callback) {
        CSElement._historyManager.on(event, callback);
    }
    /**
     * Отписка от событий истории
     */
    static offHistoryEvent(event, callback) {
        CSElement._historyManager.off(event, callback);
    }
    // Приватные методы для записи операций в историю
    recordOperation(type, description, before, after, path) {
        try {
            CSElement._historyManager.addOperation({
                type: type,
                before,
                after,
                path,
                description,
                canUndo: true,
                canRedo: true,
                metadata: {
                    elementId: this._id,
                    elementName: this._name,
                    timestamp: Date.now()
                }
            });
        }
        catch (error) {
            console.warn('Не удалось записать операцию в историю:', error);
        }
    }
    // ==================== СИСТЕМА РЕАКТИВНОСТИ ====================
    /**
     * Получить глобальный менеджер реактивности
     */
    static get reactivity() {
        return CSElement._reactivityManager;
    }
    /**
     * Настроить систему реактивности
     */
    static configureReactivity(config) {
        CSElement._reactivityManager = new ReactivityManagerImpl(config);
    }
    /**
     * Создать computed свойство
     */
    static computed(compute, config) {
        return CSElement._reactivityManager.computed(compute, config);
    }
    /**
     * Создать наблюдатель за изменениями
     */
    static watch(path, callback, config) {
        return CSElement._reactivityManager.watch(path, callback, config);
    }
    /**
     * Удалить наблюдатель
     */
    static unwatch(watcherId) {
        return CSElement._reactivityManager.unwatch(watcherId);
    }
    /**
     * Получить значение computed свойства
     */
    static getComputedValue(computedId) {
        return CSElement._reactivityManager.getComputedValue(computedId);
    }
    /**
     * Создать реактивную ссылку
     */
    static ref(value) {
        return CSElement._reactivityManager.ref(value);
    }
    /**
     * Создать реактивный объект
     */
    static reactive(obj) {
        return CSElement._reactivityManager.reactive(obj);
    }
    /**
     * Получить граф зависимостей
     */
    static getDependencyGraph() {
        return CSElement._reactivityManager.getDependencyGraph();
    }
    /**
     * Получить статистику реактивности
     */
    static getReactivityStats() {
        return CSElement._reactivityManager.getStats();
    }
    /**
     * Очистить все реактивные данные
     */
    static clearReactivity() {
        CSElement._reactivityManager.clear();
    }
    /**
     * Подписаться на события реактивности
     */
    static onReactivityEvent(event, callback) {
        CSElement._reactivityManager.onReactivityEvent(event, callback);
    }
    /**
     * Отписаться от событий реактивности
     */
    static offReactivityEvent(event, callback) {
        CSElement._reactivityManager.offReactivityEvent(event, callback);
    }
    // ==================== РЕАКТИВНЫЕ МЕТОДЫ ЭКЗЕМПЛЯРА ====================
    /**
     * Создать computed свойство для этого элемента
     */
    computed(compute, config) {
        return CSElement._reactivityManager.computed(compute, config);
    }
    /**
     * Создать наблюдатель за свойством этого элемента
     */
    watch(property, callback, config) {
        const path = [this._id, 'data', property];
        return CSElement._reactivityManager.watch(path, callback, config);
    }
    /**
     * Наблюдать за дочерними элементами
     */
    watchChildren(callback, config) {
        const path = [this._id, 'elements'];
        return CSElement._reactivityManager.watch(path, callback, config);
    }
    /**
     * Создать реактивную версию данных элемента
     */
    makeReactive() {
        // Создаем реактивную версию данных
        const reactiveData = CSElement._reactivityManager.reactive(Object.fromEntries(this._data));
        // Заменяем Map на реактивный объект
        this._data.clear();
        for (const [key, value] of Object.entries(reactiveData)) {
            this._data.set(key, value);
        }
        return this;
    }
    /**
     * Уведомить о изменении данных (для интеграции с реактивностью)
     */
    notifyDataChange(key, newValue, oldValue) {
        const path = [this._id, 'data', key];
        CSElement._reactivityManager.notify(path, newValue, oldValue);
        // Уведомляем Live queries об изменении данных
        if (CSElement._liveQueryManager) {
            CSElement._liveQueryManager.notifyDataChange(this, key, newValue, oldValue);
        }
    }
    /**
     * Уведомить о изменении дочерних элементов (для интеграции с реактивностью)
     */
    notifyChildrenChange(_action, _element) {
        const path = [this._id, 'elements'];
        const currentElements = Array.from(this._elements.values());
        CSElement._reactivityManager.notify(path, currentElements, currentElements);
    }
    // ==================== Live Queries API ====================
    /**
     * Получить менеджер Live queries
     */
    static get liveQueries() {
        if (!CSElement._liveQueryManager) {
            CSElement._liveQueryManager = new LiveQueryManagerImpl();
        }
        return CSElement._liveQueryManager;
    }
    /**
     * Получить менеджер алгоритмов работы с графами
     */
    static get graphAlgorithms() {
        if (!CSElement._graphAlgorithmsManager) {
            CSElement._graphAlgorithmsManager = new GraphAlgorithmsManagerImpl();
        }
        return CSElement._graphAlgorithmsManager;
    }
    /**
     * Создать Live query
     */
    static createLiveQuery(selector, options, config) {
        return CSElement.liveQueries.createLiveQuery(selector, options || {}, config);
    }
    /**
     * Создать builder для Live query
     */
    static queryBuilder() {
        return new LiveQueryBuilderImpl(CSElement.liveQueries);
    }
    /**
     * Подписаться на результаты Live query
     */
    static subscribeLiveQuery(queryId, callback) {
        return CSElement.liveQueries.subscribe(queryId, callback);
    }
    /**
     * Отписаться от Live query
     */
    static unsubscribeLiveQuery(queryId, subscriptionId) {
        return CSElement.liveQueries.unsubscribe(queryId, subscriptionId);
    }
    /**
     * Запустить Live query
     */
    static startLiveQuery(queryId) {
        CSElement.liveQueries.start(queryId);
    }
    /**
     * Остановить Live query
     */
    static stopLiveQuery(queryId) {
        CSElement.liveQueries.stop(queryId);
    }
    /**
     * Получить Live query по ID
     */
    static getLiveQuery(queryId) {
        return CSElement.liveQueries.getLiveQuery(queryId);
    }
    /**
     * Получить все Live queries
     */
    static getAllLiveQueries() {
        return CSElement.liveQueries.getAllLiveQueries();
    }
    /**
     * Удалить Live query
     */
    static removeLiveQuery(queryId) {
        return CSElement.liveQueries.removeLiveQuery(queryId);
    }
    /**
     * Получить статистику Live queries
     */
    static getLiveQueryStats() {
        return CSElement.liveQueries.getStats();
    }
    /**
     * Очистить все Live queries
     */
    static clearLiveQueries() {
        CSElement.liveQueries.clear();
    }
    /**
     * Обновить все активные Live queries
     */
    static updateAllLiveQueries() {
        CSElement.liveQueries.updateAllQueries();
    }
    /**
     * Подписаться на события Live queries
     */
    static onLiveQueryEvent(callback) {
        CSElement.liveQueries.on('live-query-event', callback);
    }
    /**
     * Отписаться от событий Live queries
     */
    static offLiveQueryEvent(callback) {
        CSElement.liveQueries.off('live-query-event', callback);
    }
    // Методы экземпляра для Live queries
    /**
     * Создать Live query для дочерних элементов
     */
    createChildrenLiveQuery(selector = '*', options, config) {
        // Модифицируем селектор для поиска только среди дочерних элементов
        const childSelector = `#${this._id} > ${selector}`;
        return CSElement.liveQueries.createLiveQuery(childSelector, options || {}, config);
    }
    /**
     * Создать Live query для всех потомков
     */
    createDescendantsLiveQuery(selector = '*', options, config) {
        // Модифицируем селектор для поиска среди всех потомков
        const descendantSelector = `#${this._id} ${selector}`;
        return CSElement.liveQueries.createLiveQuery(descendantSelector, options || {}, config);
    }
    /**
     * Подписаться на изменения дочерних элементов (Live query)
     */
    watchChildrenLive(callback, config) {
        const query = this.createChildrenLiveQuery('*', {}, config);
        query.onResults = callback;
        return query;
    }
    /**
     * Подписаться на изменения элементов с определенными данными
     */
    watchElementsWithData(dataKey, callback, config) {
        const selector = `[${dataKey}]`;
        const query = this.createDescendantsLiveQuery(selector, {}, config);
        query.onResults = callback;
        return query;
    }
    /**
     * Уведомить Live queries об изменении элемента
     */
    notifyElementChange(changeType) {
        if (CSElement._liveQueryManager) {
            CSElement._liveQueryManager.notifyElementChange(this, changeType);
        }
    }
    // ==================== Алгоритмы работы с графами ====================
    /**
     * Найти путь между двумя элементами
     */
    static async findPath(from, to, options) {
        return CSElement.graphAlgorithms.findPath(from, to, options);
    }
    /**
     * Найти кратчайший путь между двумя элементами
     */
    static async findShortestPath(from, to, options) {
        return CSElement.graphAlgorithms.findShortestPath(from, to, options);
    }
    /**
     * Найти все пути между двумя элементами
     */
    static async findAllPaths(from, to, options) {
        return CSElement.graphAlgorithms.findAllPaths(from, to, options);
    }
    /**
     * Обнаружить циклы в графе
     */
    static async detectCycles(root, options) {
        return CSElement.graphAlgorithms.detectCycles(root, options);
    }
    /**
     * Проверить наличие циклов в графе
     */
    static async hasCycle(root, options) {
        return CSElement.graphAlgorithms.hasCycle(root, options);
    }
    /**
     * Найти один цикл в графе
     */
    static async findCycle(root, options) {
        return CSElement.graphAlgorithms.findCycle(root, options);
    }
    /**
     * Найти компоненты связности
     */
    static async findConnectedComponents(root, options) {
        return CSElement.graphAlgorithms.findConnectedComponents(root, options);
    }
    /**
     * Найти сильно связанные компоненты
     */
    static async findStronglyConnectedComponents(root, options) {
        return CSElement.graphAlgorithms.findStronglyConnectedComponents(root, options);
    }
    /**
     * Проверить связность графа
     */
    static async isConnected(root, options) {
        return CSElement.graphAlgorithms.isConnected(root, options);
    }
    /**
     * Построить представление графа
     */
    static async buildGraph(root, options) {
        return CSElement.graphAlgorithms.buildGraph(root, options);
    }
    /**
     * Провести полный анализ графа
     */
    static async analyzeGraph(root, options) {
        return CSElement.graphAlgorithms.analyzeGraph(root, options);
    }
    /**
     * Получить статистику графа
     */
    static async getGraphStatistics(root, options) {
        return CSElement.graphAlgorithms.getGraphStatistics(root, options);
    }
    /**
     * Получить расстояние между элементами
     */
    static async getDistance(from, to, options) {
        return CSElement.graphAlgorithms.getDistance(from, to, options);
    }
    /**
     * Получить соседей элемента
     */
    static getNeighbors(node, options) {
        return CSElement.graphAlgorithms.getNeighbors(node, options);
    }
    /**
     * Получить степень элемента
     */
    static getDegree(node, options) {
        return CSElement.graphAlgorithms.getDegree(node, options);
    }
    /**
     * Топологическая сортировка
     */
    static async topologicalSort(root, options) {
        return CSElement.graphAlgorithms.topologicalSort(root, options);
    }
    /**
     * Найти минимальное остовное дерево
     */
    static async minimumSpanningTree(root, options) {
        return CSElement.graphAlgorithms.minimumSpanningTree(root, options);
    }
    /**
     * Вычислить центральность узлов
     */
    static async calculateCentrality(root, options) {
        return CSElement.graphAlgorithms.calculateCentrality(root, options);
    }
    /**
     * Получить метрики алгоритмов
     */
    static getGraphAlgorithmMetrics() {
        return CSElement.graphAlgorithms.getMetrics();
    }
    /**
     * Очистить кэш алгоритмов
     */
    static clearGraphAlgorithmCache() {
        CSElement.graphAlgorithms.clearCache();
    }
    /**
     * Настроить алгоритмы работы с графами
     */
    static configureGraphAlgorithms(config) {
        CSElement.graphAlgorithms.configure(config);
    }
    /**
     * Подписаться на события алгоритмов
     */
    static onGraphAlgorithmEvent(callback) {
        CSElement.graphAlgorithms.on('algorithm-event', callback);
    }
    /**
     * Отписаться от событий алгоритмов
     */
    static offGraphAlgorithmEvent(callback) {
        CSElement.graphAlgorithms.off('algorithm-event', callback);
    }
    // ==================== Методы экземпляра для алгоритмов ====================
    /**
     * Найти путь от этого элемента к другому
     */
    async findPathTo(target, options) {
        return CSElement.findPath(this, target, options);
    }
    /**
     * Найти кратчайший путь от этого элемента к другому
     */
    async findShortestPathTo(target, options) {
        return CSElement.findShortestPath(this, target, options);
    }
    /**
     * Найти все пути от этого элемента к другому
     */
    async findAllPathsTo(target, options) {
        return CSElement.findAllPaths(this, target, options);
    }
    /**
     * Обнаружить циклы начиная с этого элемента
     */
    async detectCyclesFromHere(options) {
        return CSElement.detectCycles(this, options);
    }
    /**
     * Проверить наличие циклов начиная с этого элемента
     */
    async hasCycleFromHere(options) {
        return CSElement.hasCycle(this, options);
    }
    /**
     * Найти компоненты связности начиная с этого элемента
     */
    async findConnectedComponentsFromHere(options) {
        return CSElement.findConnectedComponents(this, options);
    }
    /**
     * Проверить связность графа начиная с этого элемента
     */
    async isConnectedFromHere(options) {
        return CSElement.isConnected(this, options);
    }
    /**
     * Построить граф начиная с этого элемента
     */
    async buildGraphFromHere(options) {
        return CSElement.buildGraph(this, options);
    }
    /**
     * Провести анализ графа начиная с этого элемента
     */
    async analyzeGraphFromHere(options) {
        return CSElement.analyzeGraph(this, options);
    }
    /**
     * Получить статистику графа начиная с этого элемента
     */
    async getGraphStatisticsFromHere(options) {
        return CSElement.getGraphStatistics(this, options);
    }
    /**
     * Получить расстояние до другого элемента
     */
    async getDistanceTo(target, options) {
        return CSElement.getDistance(this, target, options);
    }
    /**
     * Получить соседей этого элемента
     */
    getNeighborsFromHere(options) {
        return CSElement.getNeighbors(this, options);
    }
    /**
     * Получить степень этого элемента
     */
    getDegreeFromHere(options) {
        return CSElement.getDegree(this, options);
    }
    /**
     * Топологическая сортировка начиная с этого элемента
     */
    async topologicalSortFromHere(options) {
        return CSElement.topologicalSort(this, options);
    }
    /**
     * Найти минимальное остовное дерево начиная с этого элемента
     */
    async minimumSpanningTreeFromHere(options) {
        return CSElement.minimumSpanningTree(this, options);
    }
    /**
     * Вычислить центральность узлов начиная с этого элемента
     */
    async calculateCentralityFromHere(options) {
        return CSElement.calculateCentrality(this, options);
    }
}
// Статическое хранилище всех элементов для поиска
CSElement._globalRegistry = new Map();
// Глобальный менеджер плагинов
CSElement._pluginManager = new PluginManager();
// Глобальный менеджер персистентности
CSElement._persistenceManager = new PersistenceManagerImpl();
// Глобальный менеджер истории
CSElement._historyManager = new HistoryManagerImpl();
// Глобальный менеджер реактивности
CSElement._reactivityManager = new ReactivityManagerImpl();
//# sourceMappingURL=CSElement.js.map