/**
 * Менеджер плагинов для CSElement
 * Обеспечивает загрузку, управление и выполнение плагинов
 */
import { EventEmitter } from 'eventemitter3';
import { AdvancedMiddlewareManagerImpl } from './AdvancedMiddlewareManager';
export class PluginManager extends EventEmitter {
    constructor() {
        super();
        this.plugins = new Map();
        this.pluginOptions = new Map();
        this.hooks = new Map();
        this.middleware = new Map();
        this.advancedMiddleware = new AdvancedMiddlewareManagerImpl();
        this.CSElementClass = null;
        this.initializeHooks();
        this.initializeMiddleware();
    }
    /**
     * Инициализация хуков
     */
    initializeHooks() {
        const hookNames = [
            'beforeCreate', 'afterCreate',
            'beforeAddElement', 'afterAddElement',
            'beforeRemoveElement', 'afterRemoveElement',
            'beforeSetData', 'afterSetData',
            'beforeDeleteData', 'afterDeleteData',
            'beforeDestroy', 'afterDestroy'
        ];
        hookNames.forEach(hookName => {
            this.hooks.set(hookName, []);
        });
    }
    /**
     * Инициализация middleware
     */
    initializeMiddleware() {
        const operations = ['setData', 'deleteData', 'addElement', 'removeElement'];
        operations.forEach(operation => {
            this.middleware.set(operation, []);
        });
    }
    /**
     * Установить ссылку на класс CSElement
     */
    setCSElementClass(CSElementClass) {
        this.CSElementClass = CSElementClass;
    }
    /**
     * Установить плагин
     */
    use(plugin, options) {
        try {
            // Проверка зависимостей
            if (plugin.dependencies) {
                for (const dependency of plugin.dependencies) {
                    if (!this.hasPlugin(dependency)) {
                        throw new Error(`Плагин "${plugin.name}" требует зависимость "${dependency}", которая не установлена`);
                    }
                }
            }
            // Проверка на дублирование
            if (this.hasPlugin(plugin.name)) {
                throw new Error(`Плагин "${plugin.name}" уже установлен`);
            }
            // Установка плагина
            if (!this.CSElementClass) {
                throw new Error('CSElement класс не установлен в менеджере плагинов');
            }
            plugin.install(this.CSElementClass, options);
            // Сохранение плагина
            this.plugins.set(plugin.name, plugin);
            this.pluginOptions.set(plugin.name, options || {});
            // Событие установки
            const event = {
                type: 'install',
                plugin: plugin.name,
                data: { version: plugin.version, options },
                timestamp: Date.now()
            };
            this.emit('plugin:installed', event);
            console.log(`✅ Плагин "${plugin.name}" v${plugin.version} успешно установлен`);
        }
        catch (error) {
            const errorMessage = error instanceof Error ? error.message : String(error);
            const event = {
                type: 'error',
                plugin: plugin.name,
                data: { error: errorMessage },
                timestamp: Date.now()
            };
            this.emit('plugin:error', event);
            throw error;
        }
    }
    /**
     * Удалить плагин
     */
    uninstall(pluginName) {
        try {
            const plugin = this.plugins.get(pluginName);
            if (!plugin) {
                return false;
            }
            // Проверка зависимостей других плагинов
            for (const [name, installedPlugin] of this.plugins) {
                if (name !== pluginName && installedPlugin.dependencies?.includes(pluginName)) {
                    throw new Error(`Нельзя удалить плагин "${pluginName}": от него зависит плагин "${name}"`);
                }
            }
            // Удаление плагина
            if (plugin.uninstall) {
                plugin.uninstall();
            }
            this.plugins.delete(pluginName);
            this.pluginOptions.delete(pluginName);
            // Событие удаления
            const event = {
                type: 'uninstall',
                plugin: pluginName,
                timestamp: Date.now()
            };
            this.emit('plugin:uninstalled', event);
            console.log(`✅ Плагин "${pluginName}" успешно удален`);
            return true;
        }
        catch (error) {
            const errorMessage = error instanceof Error ? error.message : String(error);
            const event = {
                type: 'error',
                plugin: pluginName,
                data: { error: errorMessage },
                timestamp: Date.now()
            };
            this.emit('plugin:error', event);
            throw error;
        }
    }
    /**
     * Получить установленный плагин
     */
    getPlugin(name) {
        return this.plugins.get(name) || null;
    }
    /**
     * Получить все установленные плагины
     */
    getPlugins() {
        return Array.from(this.plugins.values());
    }
    /**
     * Проверить, установлен ли плагин
     */
    hasPlugin(name) {
        return this.plugins.has(name);
    }
    /**
     * Получить информацию о всех плагинах
     */
    getPluginInfo() {
        return Array.from(this.plugins.entries()).map(([name, plugin]) => ({
            name: plugin.name,
            version: plugin.version,
            description: plugin.description,
            dependencies: plugin.dependencies || [],
            installed: true,
            options: this.pluginOptions.get(name)
        }));
    }
    /**
     * Добавить хук жизненного цикла
     */
    addHook(hookName, callback) {
        const hooks = this.hooks.get(hookName) || [];
        hooks.push(callback);
        this.hooks.set(hookName, hooks);
    }
    /**
     * Удалить хук жизненного цикла
     */
    removeHook(hookName, callback) {
        const hooks = this.hooks.get(hookName) || [];
        const index = hooks.indexOf(callback);
        if (index > -1) {
            hooks.splice(index, 1);
            this.hooks.set(hookName, hooks);
        }
    }
    /**
     * Добавить middleware
     */
    addMiddleware(operation, middleware) {
        const middlewares = this.middleware.get(operation) || [];
        middlewares.push(middleware);
        this.middleware.set(operation, middlewares);
    }
    /**
     * Добавить middleware с конфигурацией
     */
    addAdvancedMiddleware(operation, config) {
        this.advancedMiddleware.addMiddleware(operation, config);
    }
    /**
     * Удалить middleware
     */
    removeMiddleware(operation, middleware) {
        const middlewares = this.middleware.get(operation) || [];
        const index = middlewares.indexOf(middleware);
        if (index > -1) {
            middlewares.splice(index, 1);
            this.middleware.set(operation, middlewares);
        }
    }
    /**
     * Выполнить хуки жизненного цикла
     */
    async executeHooks(hookName, ...args) {
        const hooks = this.hooks.get(hookName) || [];
        for (const hook of hooks) {
            try {
                const result = hook(...args);
                if (result instanceof Promise) {
                    await result;
                }
            }
            catch (error) {
                console.error(`Ошибка в хуке ${hookName}:`, error);
                this.emit('hook:error', { hookName, error, args });
            }
        }
    }
    /**
     * Выполнить middleware цепочку
     */
    async executeMiddleware(context, operation) {
        // Сначала пробуем расширенную middleware систему
        const advancedMiddlewares = this.advancedMiddleware.getMiddleware(context.operation);
        if (advancedMiddlewares.length > 0) {
            return await this.advancedMiddleware.executeMiddleware(context, operation);
        }
        // Fallback на старую систему middleware
        const middlewares = this.middleware.get(context.operation) || [];
        if (middlewares.length === 0) {
            return await operation();
        }
        let index = 0;
        const next = async () => {
            if (index >= middlewares.length) {
                return await operation();
            }
            const middleware = middlewares[index++];
            try {
                return await middleware(context, next);
            }
            catch (error) {
                console.error(`Ошибка в middleware ${context.operation}:`, error);
                this.emit('middleware:error', { context, error });
                throw error;
            }
        };
        return await next();
    }
    /**
     * Очистить все плагины (для тестирования)
     */
    clear() {
        // Удаляем все плагины
        for (const [name] of this.plugins) {
            try {
                this.uninstall(name);
            }
            catch (error) {
                console.error(`Ошибка при удалении плагина ${name}:`, error);
            }
        }
        // Очищаем хуки и middleware
        this.initializeHooks();
        this.initializeMiddleware();
        // Очищаем advanced middleware
        this.advancedMiddleware.clearAllMiddleware();
    }
    /**
     * Получить статистику плагинов
     */
    getStats() {
        const totalHooks = Array.from(this.hooks.values())
            .reduce((sum, hooks) => sum + hooks.length, 0);
        const totalMiddleware = Array.from(this.middleware.values())
            .reduce((sum, middlewares) => sum + middlewares.length, 0);
        return {
            totalPlugins: this.plugins.size,
            totalHooks,
            totalMiddleware,
            pluginsByType: {} // Можно расширить для категоризации плагинов
        };
    }
}
//# sourceMappingURL=PluginManager.js.map