/**
 * Интеграция State Machine в CSElement
 */
import { CSElement } from './CSElement';
// import { StateMachine } from '../state-machine/StateMachine';
import { StateMachineManagerImpl } from '../state-machine/StateMachineManager';
import { StateMachineEventType } from '../types/state-machine-interfaces';
/**
 * Расширение CSElement для поддержки State Machine
 */
export class StateMachineIntegration {
    /**
     * Получить глобальный менеджер State Machine
     */
    static get stateMachineManager() {
        return this._globalStateMachineManager;
    }
    /**
     * Создать State Machine для элемента
     */
    static createStateMachine(element, config) {
        return this._globalStateMachineManager.create(element, config);
    }
    /**
     * Получить State Machine элемента
     */
    static getStateMachine(element) {
        return this._globalStateMachineManager.get(element.id);
    }
    /**
     * Удалить State Machine элемента
     */
    static removeStateMachine(element) {
        return this._globalStateMachineManager.remove(element.id);
    }
    /**
     * Проверить, есть ли у элемента State Machine
     */
    static hasStateMachine(element) {
        return this._globalStateMachineManager.has(element.id);
    }
    /**
     * Получить все State Machine
     */
    static getAllStateMachines() {
        return this._globalStateMachineManager.getAll();
    }
    /**
     * Найти State Machine по состоянию
     */
    static findByState(state) {
        return this._globalStateMachineManager.findByState(state);
    }
    /**
     * Отправить событие всем State Machine в определенном состоянии
     */
    static async sendToState(state, event, payload) {
        return this._globalStateMachineManager.sendToState(state, event, payload);
    }
    /**
     * Получить статистику State Machine
     */
    static getStats() {
        return this._globalStateMachineManager.getStats();
    }
    /**
     * Очистить все State Machine
     */
    static clear() {
        this._globalStateMachineManager.clear();
    }
}
StateMachineIntegration._globalStateMachineManager = new StateMachineManagerImpl();
/**
 * Миксин для добавления методов State Machine в CSElement
 */
export function addStateMachineMethods(CSElementClass) {
    // Создание и управление State Machine
    CSElementClass.prototype.createStateMachine = function (config) {
        if (StateMachineIntegration.hasStateMachine(this)) {
            throw new Error(`State Machine для элемента "${this.id}" уже существует`);
        }
        return StateMachineIntegration.createStateMachine(this, config);
    };
    CSElementClass.prototype.getStateMachine = function () {
        return StateMachineIntegration.getStateMachine(this);
    };
    CSElementClass.prototype.removeStateMachine = function () {
        return StateMachineIntegration.removeStateMachine(this);
    };
    CSElementClass.prototype.hasStateMachine = function () {
        return StateMachineIntegration.hasStateMachine(this);
    };
    // Работа с состояниями
    CSElementClass.prototype.getCurrentState = function () {
        const sm = this.getStateMachine();
        return sm?.currentState;
    };
    CSElementClass.prototype.canTransition = function (event, toState) {
        const sm = this.getStateMachine();
        return sm?.canTransition(event, toState) || false;
    };
    CSElementClass.prototype.getPossibleTransitions = function () {
        const sm = this.getStateMachine();
        return sm?.getPossibleTransitions() || [];
    };
    CSElementClass.prototype.getStateHistory = function () {
        const sm = this.getStateMachine();
        return sm?.getHistory() || [];
    };
    // Отправка событий
    CSElementClass.prototype.sendStateMachineEvent = async function (event, payload) {
        const sm = this.getStateMachine();
        if (!sm) {
            console.warn(`Элемент "${this.id}" не имеет State Machine`);
            return undefined;
        }
        return await sm.send(event, payload);
    };
    // Подписки на события State Machine
    CSElementClass.prototype.onStateChanged = function (callback) {
        const sm = this.getStateMachine();
        return sm?.subscribe(StateMachineEventType.STATE_CHANGED, callback);
    };
    CSElementClass.prototype.onTransitionStarted = function (callback) {
        const sm = this.getStateMachine();
        return sm?.subscribe(StateMachineEventType.TRANSITION_STARTED, callback);
    };
    CSElementClass.prototype.onTransitionCompleted = function (callback) {
        const sm = this.getStateMachine();
        return sm?.subscribe(StateMachineEventType.TRANSITION_COMPLETED, callback);
    };
    CSElementClass.prototype.onTransitionFailed = function (callback) {
        const sm = this.getStateMachine();
        return sm?.subscribe(StateMachineEventType.TRANSITION_FAILED, callback);
    };
    CSElementClass.prototype.offStateMachineEvent = function (eventType, callbackId) {
        const sm = this.getStateMachine();
        return sm?.unsubscribe(eventType, callbackId) || false;
    };
    // Управление State Machine
    CSElementClass.prototype.resetStateMachine = async function () {
        const sm = this.getStateMachine();
        if (sm) {
            await sm.reset();
        }
    };
    CSElementClass.prototype.destroyStateMachine = function () {
        const sm = this.getStateMachine();
        if (sm) {
            sm.destroy();
            this.removeStateMachine();
        }
    };
    // Статические методы для класса CSElement
    CSElementClass.getStateMachineManager = () => {
        return StateMachineIntegration.stateMachineManager;
    };
    CSElementClass.findElementsByState = (state) => {
        const stateMachines = StateMachineIntegration.findByState(state);
        return stateMachines
            .map(sm => sm.context.element)
            .filter(el => el instanceof CSElement);
    };
    CSElementClass.sendEventToElementsInState = async (state, event, payload) => {
        return StateMachineIntegration.sendToState(state, event, payload);
    };
    CSElementClass.getStateMachineStats = () => {
        return StateMachineIntegration.getStats();
    };
    CSElementClass.clearAllStateMachines = () => {
        StateMachineIntegration.clear();
    };
}
// Автоматически добавляем методы в CSElement при импорте
addStateMachineMethods(CSElement);
// Перехватываем метод destroy для автоматической очистки State Machine
const originalDestroy = CSElement.prototype.destroy;
CSElement.prototype.destroy = function () {
    // Удаляем State Machine перед уничтожением элемента
    if (this.hasStateMachine()) {
        this.destroyStateMachine();
    }
    // Вызываем оригинальный метод destroy
    originalDestroy.call(this);
};
//# sourceMappingURL=StateMachineIntegration.js.map