"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const src_1 = require("../src");
async function basicExample() {
    console.log('=== Базовый пример использования CSElement ===\n');
    // 1. Создание простых элементов
    const root = new src_1.CSElement('root');
    const child1 = await root.addElement('child1');
    const child2 = await root.addElement('child2');
    console.log(`Создан корневой элемент: ${root.name}`);
    console.log(`Количество дочерних элементов: ${root.elementsCount()}`);
    // 2. Работа с данными
    await root.setData('type', 'container');
    await root.setData('version', '1.0');
    await child1.setData('type', 'item');
    await child1.setData('value', 42);
    console.log(`\nДанные корневого элемента:`, Object.fromEntries(root.data));
    console.log(`Данные child1:`, Object.fromEntries(child1.data));
    // 3. Доступ к элементам по имени и индексу
    const foundByName = root.getElement('child1');
    const foundByIndex = root.getElement(0);
    console.log(`\nПоиск по имени 'child1': ${foundByName?.name}`);
    console.log(`Поиск по индексу 0: ${foundByIndex?.name}`);
    // 4. События
    child1.on(src_1.ElementEvent.DataChanged, (data) => {
        console.log(`\nСобытие: Данные изменены в ${child1.name}:`, data);
    });
    await child1.setData('value', 100);
    // 5. Вложенные структуры
    const subChild = await child1.addElement('subChild');
    await subChild.setData('level', 3);
    console.log(`\nГлубина структуры:`);
    console.log(`- ${root.name}`);
    console.log(`  - ${child1.name} (элементов: ${child1.elementsCount()})`);
    console.log(`    - ${subChild.name}`);
    console.log(`  - ${child2.name}`);
}
async function builderExample() {
    console.log('\n\n=== Пример использования ElementBuilder ===\n');
    // Создание сложной структуры через builder
    const menu = await src_1.ElementBuilder.create('mainMenu')
        .data('type', 'menu')
        .data('visible', true)
        .child('file', { data: new Map([['label', 'Файл']]) })
        .child('edit', { data: new Map([['label', 'Правка']]) })
        .child('view', { data: new Map([['label', 'Вид']]) })
        .build();
    console.log(`Создано меню: ${menu.name}`);
    console.log(`Пункты меню:`);
    menu.getAllElements().forEach((item, index) => {
        console.log(`  ${index + 1}. ${item.name} - ${item.data.get('label')}`);
    });
    // Добавление подменю
    const fileMenu = menu.getElement('file');
    if (fileMenu) {
        await fileMenu.addElement('new', { data: new Map([['label', 'Новый'], ['shortcut', 'Ctrl+N']]) });
        await fileMenu.addElement('open', { data: new Map([['label', 'Открыть'], ['shortcut', 'Ctrl+O']]) });
        await fileMenu.addElement('save', { data: new Map([['label', 'Сохранить'], ['shortcut', 'Ctrl+S']]) });
        console.log(`\nПодменю "Файл":`);
        fileMenu.getAllElements().forEach((item) => {
            const shortcut = item.data.get('shortcut');
            console.log(`  - ${item.data.get('label')}${shortcut ? ` (${shortcut})` : ''}`);
        });
    }
}
async function spaceExample() {
    console.log('\n\n=== Пример использования ElementSpace ===\n');
    // Создание пространства элементов
    const space = new src_1.ElementSpace('myApp');
    // Создание элементов в пространстве
    const app = await space.createElement('app');
    const header = await space.createElement('header');
    const content = await space.createElement('content');
    const footer = await space.createElement('footer');
    // Построение структуры
    await app.addElement(header);
    await app.addElement(content);
    await app.addElement(footer);
    // Добавление компонентов в content
    const sidebar = await space.createElement('sidebar');
    const main = await space.createElement('main');
    await content.addElement(sidebar);
    await content.addElement(main);
    console.log(`Пространство: ${space.name}`);
    console.log(`Всего элементов: ${space.getAllElements().length}`);
    console.log(`Корневых элементов: ${space.getRootElements().length}`);
    // Обход структуры
    console.log('\nСтруктура приложения:');
    await space.traverseDepthFirst(async (element) => {
        const level = getElementLevel(element);
        const indent = '  '.repeat(level);
        console.log(`${indent}- ${element.name}`);
    });
    // Поиск элементов
    const containers = space.findElements(el => el.elementsCount() > 0);
    console.log(`\nКонтейнеры (элементы с дочерними):`);
    containers.forEach(el => console.log(`  - ${el.name} (дочерних: ${el.elementsCount()})`));
    // Экспорт структуры
    console.log('\nЭкспорт структуры в JSON:');
    console.log(JSON.stringify(space.toJSON(), null, 2));
}
async function threadSafetyExample() {
    console.log('\n\n=== Пример потокобезопасности ===\n');
    const element = new src_1.CSElement('sharedResource');
    await element.setData('counter', 0);
    // Симуляция параллельных операций
    const operations = [];
    // 10 параллельных инкрементов
    for (let i = 0; i < 10; i++) {
        operations.push((async () => {
            const current = element.getData('counter');
            // Симуляция задержки
            await new Promise(resolve => setTimeout(resolve, Math.random() * 10));
            await element.setData('counter', current + 1);
        })());
    }
    await Promise.all(operations);
    console.log(`Значение счетчика после 10 параллельных операций: ${element.getData('counter')}`);
    console.log('(Благодаря AsyncLock все операции выполнены последовательно)');
    // Использование блокировок напрямую
    console.log('\nИспользование блокировок:');
    await element.enterLock();
    console.log(`Элемент заблокирован: ${element.isLocked()}`);
    // Критическая секция
    const value = element.getData('counter');
    await element.setData('counter', value * 2);
    await element.leaveLock();
    console.log(`Элемент разблокирован: ${element.isLocked()}`);
    console.log(`Финальное значение: ${element.getData('counter')}`);
}
function getElementLevel(element, level = 0) {
    if (!element.mainOwner)
        return level;
    return getElementLevel(element.mainOwner, level + 1);
}
// Запуск примеров
async function runExamples() {
    try {
        await basicExample();
        await builderExample();
        await spaceExample();
        await threadSafetyExample();
    }
    catch (error) {
        console.error('Ошибка при выполнении примеров:', error);
    }
}
// Запускаем примеры если файл выполняется напрямую
if (require.main === module) {
    runExamples();
}
