"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CSElement = void 0;
const eventemitter3_1 = require("eventemitter3");
const interfaces_1 = require("../types/interfaces");
const helpers_1 = require("../utils/helpers");
const AsyncLock_1 = require("../utils/AsyncLock");
/**
 * Основная реализация CSElement
 * Потокобезопасный класс для работы с графовыми структурами
 */
class CSElement extends eventemitter3_1.EventEmitter {
    constructor(name = '', options = {}) {
        super();
        this._id = (0, helpers_1.generateId)();
        this._name = name;
        this._index = options.index ?? 0;
        this._data = options.data || new Map();
        this._elements = new Map();
        this._elementsByIndex = [];
        this._owners = new Map();
        this._mainOwner = null;
        this._lock = new AsyncLock_1.AsyncLock();
        this._lastElement = null;
    }
    // Геттеры для readonly свойств
    get id() {
        return this._id;
    }
    get name() {
        return this._name;
    }
    get index() {
        return this._index;
    }
    get data() {
        return new Map(this._data);
    }
    get mainOwner() {
        return this._mainOwner;
    }
    get lastElement() {
        return this._lastElement;
    }
    /**
     * Получить владельца по имени или индексу
     */
    getOwner(nameOrIndex) {
        if (typeof nameOrIndex === 'string') {
            return this._owners.get(nameOrIndex) || null;
        }
        else {
            const owners = Array.from(this._owners.values());
            return owners[nameOrIndex] || null;
        }
    }
    /**
     * Асинхронно получить владельца
     */
    async getOwnerAsync(nameOrIndex) {
        return this._lock.withLock(async () => {
            return this.getOwner(nameOrIndex);
        });
    }
    /**
     * Получить элемент по имени или индексу
     */
    getElement(nameOrIndex) {
        if (typeof nameOrIndex === 'string') {
            return this._elements.get(nameOrIndex) || null;
        }
        else {
            return this._elementsByIndex[nameOrIndex] || null;
        }
    }
    /**
     * Асинхронно получить элемент
     */
    async getElementAsync(nameOrIndex) {
        return this._lock.withLock(async () => {
            return this.getElement(nameOrIndex);
        });
    }
    /**
     * Получить относительный индекс для владельца
     */
    getRelativeIndex(owner) {
        if (!(owner instanceof CSElement)) {
            return -1;
        }
        const ownerElement = owner;
        const elements = Array.from(ownerElement._elements.values());
        return elements.findIndex(el => el._id === this._id);
    }
    /**
     * Количество дочерних элементов
     */
    elementsCount() {
        return this._elements.size;
    }
    /**
     * Количество владельцев
     */
    ownersCount() {
        return this._owners.size;
    }
    /**
     * Получить все дочерние элементы
     */
    getAllElements() {
        return Array.from(this._elements.values());
    }
    /**
     * Получить всех владельцев
     */
    getAllOwners() {
        return Array.from(this._owners.values());
    }
    /**
     * Добавить новый элемент
     */
    async addElement(value, options = {}) {
        return this._lock.withLock(async () => {
            const element = value instanceof CSElement ? value : new CSElement(value, options);
            // Устанавливаем имя если не задано
            if (!element._name && options.name) {
                element._name = options.name;
            }
            // Добавляем в коллекции
            if (element._name) {
                this._elements.set(element._name, element);
            }
            // Устанавливаем индекс
            element._index = options.index ?? this._elementsByIndex.length;
            // Добавляем в массив по индексу
            if (element._index >= this._elementsByIndex.length) {
                this._elementsByIndex.push(element);
            }
            else {
                this._elementsByIndex.splice(element._index, 0, element);
                // Обновляем индексы последующих элементов
                for (let i = element._index + 1; i < this._elementsByIndex.length; i++) {
                    this._elementsByIndex[i]._index = i;
                }
            }
            // Устанавливаем владельца
            element._owners.set(this._id, this);
            if (!element._mainOwner) {
                element._mainOwner = this;
            }
            // Обновляем последний элемент
            this._lastElement = element;
            // Генерируем событие
            this.emit(interfaces_1.ElementEvent.ElementAdded, element);
            return element;
        });
    }
    /**
     * Удалить элемент
     */
    async removeElement(nameOrIndex) {
        return this._lock.withLock(async () => {
            let element = null;
            if (nameOrIndex instanceof CSElement) {
                element = nameOrIndex;
            }
            else if (typeof nameOrIndex === 'string' || typeof nameOrIndex === 'number') {
                element = this.getElement(nameOrIndex);
            }
            else {
                // nameOrIndex is ICSElement but not CSElement
                element = nameOrIndex;
            }
            if (!element) {
                return false;
            }
            // Удаляем из коллекций
            if (element._name) {
                this._elements.delete(element._name);
            }
            // Удаляем из массива по индексу
            const index = this._elementsByIndex.indexOf(element);
            if (index !== -1) {
                this._elementsByIndex.splice(index, 1);
                // Обновляем индексы последующих элементов
                for (let i = index; i < this._elementsByIndex.length; i++) {
                    this._elementsByIndex[i]._index = i;
                }
            }
            // Удаляем владельца у элемента
            element._owners.delete(this._id);
            if (element._mainOwner === this) {
                // Если это был главный владелец, назначаем нового
                const newOwner = element._owners.values().next().value;
                element._mainOwner = newOwner || null;
            }
            // Обновляем последний элемент
            if (this._lastElement === element) {
                this._lastElement = this._elementsByIndex[this._elementsByIndex.length - 1] || null;
            }
            // Генерируем событие
            this.emit(interfaces_1.ElementEvent.ElementRemoved, element);
            return true;
        });
    }
    /**
     * Установить данные
     */
    async setData(key, value) {
        return this._lock.withLock(async () => {
            const oldValue = this._data.get(key);
            this._data.set(key, value);
            this.emit(interfaces_1.ElementEvent.DataChanged, { key, oldValue, newValue: value });
        });
    }
    /**
     * Получить данные
     */
    getData(key) {
        return this._data.get(key);
    }
    /**
     * Удалить данные
     */
    async deleteData(key) {
        return this._lock.withLock(async () => {
            const hadKey = this._data.has(key);
            if (hadKey) {
                const oldValue = this._data.get(key);
                this._data.delete(key);
                this.emit(interfaces_1.ElementEvent.DataChanged, { key, oldValue, newValue: undefined });
            }
            return hadKey;
        });
    }
    /**
     * Войти в блокировку
     */
    async enterLock() {
        await this._lock.acquire();
        this.emit(interfaces_1.ElementEvent.Locked);
    }
    /**
     * Выйти из блокировки
     */
    async leaveLock() {
        this._lock.release();
        this.emit(interfaces_1.ElementEvent.Unlocked);
    }
    /**
     * Проверить заблокирован ли элемент
     */
    isLocked() {
        return this._lock.isLocked();
    }
    /**
     * Добавить себя как дочерний элемент к другому элементу
     */
    async addTo(owner, options = {}) {
        return owner.addElement(this, options);
    }
    /**
     * Удалить себя из владельца
     */
    async removeFrom(owner) {
        return owner.removeElement(this);
    }
}
exports.CSElement = CSElement;
