"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ElementSpace = void 0;
const eventemitter3_1 = require("eventemitter3");
const CSElement_1 = require("./CSElement");
const AsyncLock_1 = require("../utils/AsyncLock");
/**
 * Пространство элементов - единое место для управления всеми элементами
 */
class ElementSpace extends eventemitter3_1.EventEmitter {
    constructor(name = 'default') {
        super();
        this._elements = new Map();
        this._rootElements = new Set();
        this._lock = new AsyncLock_1.AsyncLock();
        this._name = name;
    }
    /**
     * Получить имя пространства
     */
    get name() {
        return this._name;
    }
    /**
     * Создать новый элемент в пространстве
     */
    async createElement(name, options) {
        return this._lock.withLock(async () => {
            const element = new CSElement_1.CSElement(name, options);
            this._elements.set(element.id, element);
            // Если у элемента нет владельца, добавляем в корневые
            if (!element.mainOwner) {
                this._rootElements.add(element);
            }
            // Подписываемся на события элемента
            this._subscribeToElement(element);
            this.emit('element:created', element);
            return element;
        });
    }
    /**
     * Получить элемент по ID
     */
    getElementById(id) {
        return this._elements.get(id) || null;
    }
    /**
     * Найти элементы по имени
     */
    getElementsByName(name) {
        const result = [];
        this._elements.forEach(element => {
            if (element.name === name) {
                result.push(element);
            }
        });
        return result;
    }
    /**
     * Получить все корневые элементы (без владельцев)
     */
    getRootElements() {
        return Array.from(this._rootElements);
    }
    /**
     * Получить все элементы
     */
    getAllElements() {
        return Array.from(this._elements.values());
    }
    /**
     * Удалить элемент из пространства
     */
    async removeElement(element) {
        return this._lock.withLock(async () => {
            if (!this._elements.has(element.id)) {
                return false;
            }
            // Удаляем из всех владельцев
            const owners = element.getAllOwners();
            for (const owner of owners) {
                if (owner instanceof CSElement_1.CSElement) {
                    await owner.removeElement(element);
                }
            }
            // Удаляем все дочерние элементы
            const children = element.getAllElements();
            for (const child of children) {
                if (child instanceof CSElement_1.CSElement) {
                    await this.removeElement(child);
                }
            }
            // Удаляем из коллекций
            this._elements.delete(element.id);
            this._rootElements.delete(element);
            // Отписываемся от событий
            this._unsubscribeFromElement(element);
            this.emit('element:removed', element);
            return true;
        });
    }
    /**
     * Найти элементы по условию
     */
    findElements(predicate) {
        const result = [];
        this._elements.forEach(element => {
            if (predicate(element)) {
                result.push(element);
            }
        });
        return result;
    }
    /**
     * Найти первый элемент по условию
     */
    findElement(predicate) {
        const elements = Array.from(this._elements.values());
        for (const element of elements) {
            if (predicate(element)) {
                return element;
            }
        }
        return null;
    }
    /**
     * Обход всех элементов в глубину
     */
    async traverseDepthFirst(callback, startFrom) {
        const visited = new Set();
        const traverse = async (element) => {
            if (visited.has(element.id)) {
                return;
            }
            visited.add(element.id);
            await callback(element);
            const children = element.getAllElements();
            for (const child of children) {
                if (child instanceof CSElement_1.CSElement) {
                    await traverse(child);
                }
            }
        };
        if (startFrom) {
            await traverse(startFrom);
        }
        else {
            const roots = Array.from(this._rootElements);
            for (const root of roots) {
                await traverse(root);
            }
        }
    }
    /**
     * Обход всех элементов в ширину
     */
    async traverseBreadthFirst(callback, startFrom) {
        const visited = new Set();
        const queue = [];
        if (startFrom) {
            queue.push(startFrom);
        }
        else {
            queue.push(...Array.from(this._rootElements));
        }
        while (queue.length > 0) {
            const element = queue.shift();
            if (visited.has(element.id)) {
                continue;
            }
            visited.add(element.id);
            await callback(element);
            const children = element.getAllElements();
            for (const child of children) {
                if (child instanceof CSElement_1.CSElement && !visited.has(child.id)) {
                    queue.push(child);
                }
            }
        }
    }
    /**
     * Очистить пространство
     */
    async clear() {
        return this._lock.withLock(async () => {
            // Удаляем все элементы
            const allElements = Array.from(this._elements.values());
            for (const element of allElements) {
                await this.removeElement(element);
            }
            this._elements.clear();
            this._rootElements.clear();
            this.emit('space:cleared');
        });
    }
    /**
     * Подписаться на события элемента
     */
    _subscribeToElement(element) {
        // Перенаправляем события элемента в пространство
        element.on('element:added', (child) => {
            if (child instanceof CSElement_1.CSElement) {
                this._elements.set(child.id, child);
                this._subscribeToElement(child);
            }
            this.emit('element:added', { parent: element, child });
        });
        element.on('element:removed', (child) => {
            this.emit('element:removed', { parent: element, child });
        });
        element.on('data:changed', (data) => {
            this.emit('element:dataChanged', { element, ...data });
        });
    }
    /**
     * Отписаться от событий элемента
     */
    _unsubscribeFromElement(element) {
        element.removeAllListeners();
    }
    /**
     * Экспорт структуры в JSON
     */
    toJSON() {
        const exportElement = (element) => {
            return {
                id: element.id,
                name: element.name,
                index: element.index,
                data: Object.fromEntries(element.data),
                elements: element.getAllElements().map(child => {
                    if (child instanceof CSElement_1.CSElement) {
                        return exportElement(child);
                    }
                    return null;
                }).filter(Boolean)
            };
        };
        return {
            name: this._name,
            rootElements: this._rootElements.size,
            totalElements: this._elements.size,
            structure: Array.from(this._rootElements).map(exportElement)
        };
    }
}
exports.ElementSpace = ElementSpace;
