/**
 * Реализация менеджера Live queries (реактивных запросов)
 */
import { EventEmitter } from 'eventemitter3';
import { CSElement } from '../core/CSElement';
import { generateId } from '../utils/helpers';
export class LiveQueryManagerImpl extends EventEmitter {
    constructor() {
        super();
        this.queries = new Map();
        this.subscriptions = new Map();
        this.debounceTimers = new Map();
        this.updateBatches = new Map();
        this.indexes = new Map();
        this.stats = this.createEmptyStats();
    }
    createLiveQuery(selector, options = {}, config = {}) {
        const id = generateId();
        const query = {
            id,
            selector,
            options,
            config: {
                autoStart: true,
                debounce: 100,
                cache: true,
                cacheTTL: 5000,
                deep: false,
                batch: true,
                ...config
            },
            results: [],
            active: false,
            lastUpdated: 0,
            updateCount: 0,
            watchedElements: new Set(),
            filters: [],
            transforms: [],
            sort: undefined,
            onResults: undefined,
            onError: undefined
        };
        this.queries.set(id, query);
        this.subscriptions.set(id, new Map());
        this.stats.totalQueries++;
        // Обновляем статистику селекторов
        this.updateSelectorStats(selector);
        // Автоматический запуск если включен
        if (query.config.autoStart) {
            this.start(id);
        }
        this.emit('query-created', { queryId: id, query });
        this.emitEvent({
            type: 'query-created',
            queryId: id,
            data: { selector, options, config },
            timestamp: Date.now()
        });
        return query;
    }
    start(queryId) {
        const query = this.queries.get(queryId);
        if (!query || query.active) {
            return;
        }
        query.active = true;
        this.stats.activeQueries++;
        // Выполняем первоначальный запрос
        this.executeQuery(queryId);
        // Настраиваем наблюдение за изменениями
        this.setupQueryWatching(query);
        this.emitEvent({
            type: 'query-started',
            queryId,
            timestamp: Date.now()
        });
    }
    stop(queryId) {
        const query = this.queries.get(queryId);
        if (!query || !query.active) {
            return;
        }
        query.active = false;
        this.stats.activeQueries--;
        // Останавливаем наблюдение
        this.stopQueryWatching(query);
        // Очищаем таймеры debounce
        const timer = this.debounceTimers.get(queryId);
        if (timer) {
            clearTimeout(timer);
            this.debounceTimers.delete(queryId);
        }
        this.emitEvent({
            type: 'query-stopped',
            queryId,
            timestamp: Date.now()
        });
    }
    getLiveQuery(queryId) {
        return this.queries.get(queryId);
    }
    getAllLiveQueries() {
        return Array.from(this.queries.values());
    }
    removeLiveQuery(queryId) {
        const query = this.queries.get(queryId);
        if (!query) {
            return false;
        }
        // Останавливаем запрос
        if (query.active) {
            this.stop(queryId);
        }
        // Удаляем все подписки
        this.subscriptions.delete(queryId);
        // Удаляем запрос
        this.queries.delete(queryId);
        this.stats.totalQueries--;
        this.emitEvent({
            type: 'query-removed',
            queryId,
            timestamp: Date.now()
        });
        return true;
    }
    updateQuery(queryId) {
        const query = this.queries.get(queryId);
        if (!query || !query.active) {
            return;
        }
        if (query.config.debounce && query.config.debounce > 0) {
            // Используем debounce
            const existingTimer = this.debounceTimers.get(queryId);
            if (existingTimer) {
                clearTimeout(existingTimer);
            }
            const timer = setTimeout(() => {
                this.executeQuery(queryId);
                this.debounceTimers.delete(queryId);
            }, query.config.debounce);
            this.debounceTimers.set(queryId, timer);
        }
        else {
            // Немедленное обновление
            this.executeQuery(queryId);
        }
    }
    updateAllQueries() {
        for (const [queryId, query] of this.queries) {
            if (query.active) {
                this.updateQuery(queryId);
            }
        }
    }
    addFilter(queryId, filter) {
        const query = this.queries.get(queryId);
        if (query) {
            query.filters.push(filter);
            this.updateQuery(queryId);
        }
    }
    removeFilter(queryId, filterIndex) {
        const query = this.queries.get(queryId);
        if (query && filterIndex >= 0 && filterIndex < query.filters.length) {
            query.filters.splice(filterIndex, 1);
            this.updateQuery(queryId);
        }
    }
    addTransform(queryId, transform) {
        const query = this.queries.get(queryId);
        if (query) {
            query.transforms.push(transform);
            this.updateQuery(queryId);
        }
    }
    setSort(queryId, sort) {
        const query = this.queries.get(queryId);
        if (query) {
            query.sort = sort;
            this.updateQuery(queryId);
        }
    }
    subscribe(queryId, callback) {
        const subscriptionId = generateId();
        const subscriptions = this.subscriptions.get(queryId);
        if (subscriptions) {
            const subscription = {
                id: subscriptionId,
                queryId,
                callback,
                active: true,
                createdAt: Date.now()
            };
            subscriptions.set(subscriptionId, subscription);
        }
        return subscriptionId;
    }
    unsubscribe(queryId, subscriptionId) {
        const subscriptions = this.subscriptions.get(queryId);
        if (subscriptions) {
            return subscriptions.delete(subscriptionId);
        }
        return false;
    }
    getStats() {
        // Обновляем актуальную статистику
        this.updateStats();
        return { ...this.stats };
    }
    clear() {
        // Останавливаем все активные запросы
        for (const [queryId, query] of this.queries) {
            if (query.active) {
                this.stop(queryId);
            }
        }
        // Очищаем все данные
        this.queries.clear();
        this.subscriptions.clear();
        this.debounceTimers.clear();
        this.updateBatches.clear();
        this.indexes.clear();
        // Сбрасываем статистику
        this.stats = this.createEmptyStats();
    }
    notifyElementChange(element, _changeType) {
        // Находим все запросы, которые могут быть затронуты этим изменением
        const affectedQueries = this.findAffectedQueries(element);
        for (const queryId of affectedQueries) {
            this.updateQuery(queryId);
        }
    }
    notifyDataChange(element, key, _newValue, _oldValue) {
        // Находим запросы, которые используют этот ключ данных
        const affectedQueries = this.findQueriesUsingDataKey(element, key);
        for (const queryId of affectedQueries) {
            this.updateQuery(queryId);
        }
    }
    // Приватные методы
    async executeQuery(queryId) {
        const query = this.queries.get(queryId);
        if (!query) {
            return;
        }
        const startTime = Date.now();
        try {
            // Выполняем базовый запрос
            let results = await this.performQuery(query);
            // Применяем фильтры
            for (const filter of query.filters) {
                results = results.filter(filter);
            }
            // Применяем трансформации
            for (const transform of query.transforms) {
                results = results.map(transform);
            }
            // Применяем сортировку
            if (query.sort) {
                results.sort(query.sort);
            }
            // Применяем лимит
            if (query.config.limit && query.config.limit > 0) {
                results = results.slice(0, query.config.limit);
            }
            // Обновляем результаты
            const oldResults = query.results;
            query.results = results;
            query.lastUpdated = Date.now();
            query.updateCount++;
            // Обновляем статистику
            this.stats.totalUpdates++;
            const executionTime = Date.now() - startTime;
            this.updateExecutionTimeStats(executionTime);
            // Уведомляем подписчиков
            this.notifySubscribers(queryId, results, query);
            // Вызываем callback если установлен
            if (query.onResults) {
                query.onResults(results, query);
            }
            this.emitEvent({
                type: 'query-updated',
                queryId,
                data: {
                    results,
                    oldResults,
                    executionTime,
                    changeCount: this.calculateChangeCount(oldResults, results)
                },
                timestamp: Date.now()
            });
            this.emitEvent({
                type: 'results-changed',
                queryId,
                data: { results, executionTime },
                timestamp: Date.now()
            });
        }
        catch (error) {
            // Обработка ошибок
            if (query.onError) {
                query.onError(error, query);
            }
            this.emit('query-error', { queryId, error });
            console.error(`Live query ${queryId} error:`, error);
        }
    }
    async performQuery(query) {
        try {
            // Используем стандартный движок для поиска
            const allElements = CSElement.getAllElements();
            const results = allElements.filter(element => {
                // Простая реализация селектора для тестирования
                if (query.selector === '*')
                    return true;
                if (query.selector.startsWith('[') && query.selector.endsWith(']')) {
                    const attr = query.selector.slice(1, -1);
                    if (attr.includes('=')) {
                        const [key, value] = attr.split('=');
                        return element.getData(key) == value;
                    }
                    else {
                        return element.hasData(attr);
                    }
                }
                return element.name === query.selector;
            });
            return results;
        }
        catch (error) {
            console.warn('Error executing query:', error);
            return [];
        }
    }
    setupQueryWatching(query) {
        // Настраиваем наблюдение за элементами через систему реактивности
        // Это будет интегрироваться с ReactivityManager
        // Для демонстрации создаем базовое наблюдение
        // В реальной реализации это будет использовать computed свойства
        const computedResults = CSElement.computed(() => {
            // Здесь будет логика пересчета результатов запроса
            return this.performQuery(query);
        });
        // Наблюдаем за изменениями computed свойства
        CSElement.watch([computedResults.id], () => {
            this.updateQuery(query.id);
        });
    }
    stopQueryWatching(_query) {
        // Останавливаем наблюдение за элементами
        // Это будет реализовано при интеграции с ReactivityManager
    }
    findAffectedQueries(element) {
        const affected = [];
        for (const [queryId, query] of this.queries) {
            if (query.active && this.queryAffectedByElement(query, element)) {
                affected.push(queryId);
            }
        }
        return affected;
    }
    findQueriesUsingDataKey(_element, key) {
        const affected = [];
        for (const [queryId, query] of this.queries) {
            if (query.active && this.queryUsesDataKey(query, key)) {
                affected.push(queryId);
            }
        }
        return affected;
    }
    queryAffectedByElement(query, element) {
        // Проверяем, может ли изменение элемента повлиять на результаты запроса
        // Это упрощенная логика, в реальности будет более сложная
        return query.watchedElements.has(element.id) ||
            query.selector.includes(element.name) ||
            query.selector.includes('*');
    }
    queryUsesDataKey(query, key) {
        // Проверяем, использует ли запрос определенный ключ данных
        return query.selector.includes(`[${key}]`) ||
            query.selector.includes(`data-${key}`) ||
            query.selector.includes(`@${key}`);
    }
    notifySubscribers(queryId, results, query) {
        const subscriptions = this.subscriptions.get(queryId);
        if (!subscriptions) {
            return;
        }
        for (const subscription of subscriptions.values()) {
            if (subscription.active) {
                try {
                    subscription.callback(results, query);
                }
                catch (error) {
                    console.error(`Subscription callback error for query ${queryId}:`, error);
                }
            }
        }
    }
    calculateChangeCount(oldResults, newResults) {
        // Простой подсчет изменений
        if (oldResults.length !== newResults.length) {
            return Math.abs(oldResults.length - newResults.length);
        }
        let changes = 0;
        for (let i = 0; i < oldResults.length; i++) {
            if (oldResults[i] !== newResults[i]) {
                changes++;
            }
        }
        return changes;
    }
    updateSelectorStats(selector) {
        let selectorStats = this.stats.selectorStats.get(selector);
        if (!selectorStats) {
            selectorStats = {
                selector,
                queryCount: 0,
                averageResults: 0,
                lastUsed: Date.now(),
                totalExecutionTime: 0
            };
            this.stats.selectorStats.set(selector, selectorStats);
        }
        selectorStats.queryCount++;
        selectorStats.lastUsed = Date.now();
    }
    updateExecutionTimeStats(executionTime) {
        const currentAverage = this.stats.averageQueryTime;
        const totalQueries = this.stats.totalUpdates;
        this.stats.averageQueryTime =
            (currentAverage * (totalQueries - 1) + executionTime) / totalQueries;
    }
    updateStats() {
        this.stats.watchedElements = 0;
        this.stats.memoryUsage = this.calculateMemoryUsage();
        for (const query of this.queries.values()) {
            this.stats.watchedElements += query.watchedElements.size;
        }
    }
    calculateMemoryUsage() {
        // Упрощенный расчет использования памяти
        let usage = 0;
        for (const query of this.queries.values()) {
            usage += JSON.stringify(query.results).length;
            usage += query.selector.length;
            usage += query.watchedElements.size * 50; // Примерный размер ID элемента
        }
        return usage;
    }
    createEmptyStats() {
        return {
            totalQueries: 0,
            activeQueries: 0,
            totalUpdates: 0,
            averageQueryTime: 0,
            memoryUsage: 0,
            watchedElements: 0,
            selectorStats: new Map()
        };
    }
    emitEvent(event) {
        this.emit('live-query-event', event);
    }
}
// Builder для удобного создания Live queries
export class LiveQueryBuilderImpl {
    constructor(manager) {
        this._selector = '';
        this._options = {};
        this._config = {};
        this._filters = [];
        this._transforms = [];
        this.manager = manager;
    }
    selector(selector) {
        this._selector = selector;
        return this;
    }
    options(options) {
        this._options = { ...this._options, ...options };
        return this;
    }
    config(config) {
        this._config = { ...this._config, ...config };
        return this;
    }
    filter(filter) {
        this._filters.push(filter);
        return this;
    }
    transform(transform) {
        this._transforms.push(transform);
        return this;
    }
    sort(sort) {
        this._sort = sort;
        return this;
    }
    limit(limit) {
        this._config.limit = limit;
        return this;
    }
    debounce(ms) {
        this._config.debounce = ms;
        return this;
    }
    cache(ttl) {
        this._config.cache = true;
        if (ttl !== undefined) {
            this._config.cacheTTL = ttl;
        }
        return this;
    }
    subscribe(callback) {
        this._onResults = callback;
        return this;
    }
    onError(callback) {
        this._onError = callback;
        return this;
    }
    start() {
        const query = this.build();
        this.manager.start(query.id);
        return query;
    }
    build() {
        const query = this.manager.createLiveQuery(this._selector, this._options, { ...this._config, autoStart: false });
        // Применяем фильтры, трансформации и сортировку
        for (const filter of this._filters) {
            this.manager.addFilter(query.id, filter);
        }
        for (const transform of this._transforms) {
            this.manager.addTransform(query.id, transform);
        }
        if (this._sort) {
            this.manager.setSort(query.id, this._sort);
        }
        // Устанавливаем callbacks
        if (this._onResults) {
            query.onResults = this._onResults;
        }
        if (this._onError) {
            query.onError = this._onError;
        }
        return query;
    }
}
//# sourceMappingURL=LiveQueryManagerImpl.js.map