/**
 * Builder для создания миграций с fluent API
 */
export class MigrationBuilder {
    constructor() {
        this.migration = {
            dependencies: [],
            type: 'custom'
        };
    }
    /**
     * Устанавливает ID миграции
     */
    id(id) {
        this.migration.id = id;
        return this;
    }
    /**
     * Устанавливает версию миграции
     */
    version(version) {
        this.migration.version = version;
        return this;
    }
    /**
     * Устанавливает описание миграции
     */
    description(description) {
        this.migration.description = description;
        return this;
    }
    /**
     * Устанавливает тип миграции
     */
    type(type) {
        this.migration.type = type;
        return this;
    }
    /**
     * Добавляет зависимость от другой миграции
     */
    dependsOn(migrationId) {
        if (!this.migration.dependencies) {
            this.migration.dependencies = [];
        }
        this.migration.dependencies.push(migrationId);
        return this;
    }
    /**
     * Добавляет несколько зависимостей
     */
    dependsOnMany(migrationIds) {
        if (!this.migration.dependencies) {
            this.migration.dependencies = [];
        }
        this.migration.dependencies.push(...migrationIds);
        return this;
    }
    /**
     * Устанавливает функцию применения миграции
     */
    up(upFunction) {
        this.migration.up = upFunction;
        return this;
    }
    /**
     * Устанавливает функцию отката миграции
     */
    down(downFunction) {
        this.migration.down = downFunction;
        return this;
    }
    /**
     * Устанавливает функцию проверки возможности применения
     */
    canApply(canApplyFunction) {
        this.migration.canApply = canApplyFunction;
        return this;
    }
    /**
     * Устанавливает функцию проверки возможности отката
     */
    canRollback(canRollbackFunction) {
        this.migration.canRollback = canRollbackFunction;
        return this;
    }
    /**
     * Устанавливает функцию валидации
     */
    validate(validateFunction) {
        this.migration.validate = validateFunction;
        return this;
    }
    /**
     * Создает миграцию для изменения схемы
     */
    schemaChange(config) {
        this.migration.type = 'schema';
        this.migration.up = async (_context) => {
            const startTime = Date.now();
            let processedCount = 0;
            try {
                // Логика изменения схемы
                if (config.fieldChanges?.added) {
                    console.log(`Adding fields: ${config.fieldChanges.added.join(', ')}`);
                }
                if (config.fieldChanges?.removed) {
                    console.log(`Removing fields: ${config.fieldChanges.removed.join(', ')}`);
                }
                if (config.fieldChanges?.renamed) {
                    console.log(`Renaming fields: ${Object.entries(config.fieldChanges.renamed).map(([old, new_]) => `${old} -> ${new_}`).join(', ')}`);
                }
                processedCount = 1; // Одна схема обработана
                return {
                    success: true,
                    processedCount,
                    executionTime: Date.now() - startTime,
                    message: `Schema ${config.schemaName} updated from ${config.oldVersion} to ${config.newVersion}`
                };
            }
            catch (error) {
                return {
                    success: false,
                    error: error instanceof Error ? error.message : 'Unknown error',
                    executionTime: Date.now() - startTime
                };
            }
        };
        this.migration.down = async (_context) => {
            const startTime = Date.now();
            try {
                // Логика отката изменений схемы
                console.log(`Rolling back schema ${config.schemaName} from ${config.newVersion} to ${config.oldVersion}`);
                return {
                    success: true,
                    processedCount: 1,
                    executionTime: Date.now() - startTime,
                    message: `Schema ${config.schemaName} rolled back to ${config.oldVersion}`
                };
            }
            catch (error) {
                return {
                    success: false,
                    error: error instanceof Error ? error.message : 'Unknown error',
                    executionTime: Date.now() - startTime
                };
            }
        };
        return this;
    }
    /**
     * Создает миграцию для трансформации данных
     */
    dataTransform(config) {
        this.migration.type = 'data';
        this.migration.up = async (_context) => {
            const startTime = Date.now();
            let processedCount = 0;
            try {
                // Здесь была бы логика получения элементов и их трансформации
                // Для демонстрации используем заглушку
                const mockElements = []; // Получить элементы из контекста
                for (const element of mockElements) {
                    await config.transformer(element);
                    processedCount++;
                }
                return {
                    success: true,
                    processedCount,
                    executionTime: Date.now() - startTime,
                    message: `Transformed ${processedCount} elements`
                };
            }
            catch (error) {
                return {
                    success: false,
                    error: error instanceof Error ? error.message : 'Unknown error',
                    processedCount,
                    executionTime: Date.now() - startTime
                };
            }
        };
        this.migration.down = async (_context) => {
            const startTime = Date.now();
            let processedCount = 0;
            if (!config.rollbackTransformer) {
                return {
                    success: false,
                    error: 'No rollback transformer provided',
                    executionTime: Date.now() - startTime
                };
            }
            try {
                // Логика отката трансформации данных
                const mockElements = []; // Получить элементы из контекста
                for (const element of mockElements) {
                    await config.rollbackTransformer(element);
                    processedCount++;
                }
                return {
                    success: true,
                    processedCount,
                    executionTime: Date.now() - startTime,
                    message: `Rolled back ${processedCount} elements`
                };
            }
            catch (error) {
                return {
                    success: false,
                    error: error instanceof Error ? error.message : 'Unknown error',
                    processedCount,
                    executionTime: Date.now() - startTime
                };
            }
        };
        return this;
    }
    /**
     * Создает миграцию для выполнения SQL-подобных операций
     */
    sqlLike(config) {
        this.migration.type = 'custom';
        this.migration.up = async (_context) => {
            const startTime = Date.now();
            try {
                if (config.executor) {
                    await config.executor(config.up);
                }
                else {
                    console.log(`Executing UP query: ${config.up}`);
                }
                return {
                    success: true,
                    processedCount: 1,
                    executionTime: Date.now() - startTime,
                    message: 'SQL-like operation executed successfully'
                };
            }
            catch (error) {
                return {
                    success: false,
                    error: error instanceof Error ? error.message : 'Unknown error',
                    executionTime: Date.now() - startTime
                };
            }
        };
        this.migration.down = async (_context) => {
            const startTime = Date.now();
            try {
                if (config.executor) {
                    await config.executor(config.down);
                }
                else {
                    console.log(`Executing DOWN query: ${config.down}`);
                }
                return {
                    success: true,
                    processedCount: 1,
                    executionTime: Date.now() - startTime,
                    message: 'SQL-like rollback executed successfully'
                };
            }
            catch (error) {
                return {
                    success: false,
                    error: error instanceof Error ? error.message : 'Unknown error',
                    executionTime: Date.now() - startTime
                };
            }
        };
        return this;
    }
    /**
     * Создает миграцию с автоматическими проверками
     */
    withAutoChecks() {
        this.migration.canApply = async (context) => {
            // Автоматические проверки возможности применения
            return context.direction === 'up';
        };
        this.migration.canRollback = async (context) => {
            // Автоматические проверки возможности отката
            return context.direction === 'down';
        };
        this.migration.validate = async (_context) => {
            // Автоматическая валидация
            return true;
        };
        return this;
    }
    /**
     * Создает финальную миграцию
     */
    build() {
        if (!this.migration.id) {
            throw new Error('Migration ID is required');
        }
        if (!this.migration.version) {
            throw new Error('Migration version is required');
        }
        if (!this.migration.up) {
            throw new Error('Migration up function is required');
        }
        if (!this.migration.down) {
            throw new Error('Migration down function is required');
        }
        // Устанавливаем значения по умолчанию
        const migration = {
            id: this.migration.id,
            version: this.migration.version,
            description: this.migration.description || `Migration ${this.migration.id}`,
            type: this.migration.type || 'custom',
            dependencies: this.migration.dependencies || [],
            up: this.migration.up,
            down: this.migration.down,
            canApply: this.migration.canApply || (async () => true),
            canRollback: this.migration.canRollback || (async () => true),
            validate: this.migration.validate || (async () => true)
        };
        return migration;
    }
}
/**
 * Создает новый builder для миграции
 */
export function createMigration() {
    return new MigrationBuilder();
}
//# sourceMappingURL=MigrationBuilder.js.map