/**
 * Декораторы для упрощения создания миграций
 */
/**
 * Реестр миграций
 */
export class MigrationRegistry {
    static register(migration) {
        this.migrations.set(migration.id, migration);
    }
    static get(id) {
        return this.migrations.get(id);
    }
    static getAll() {
        return Array.from(this.migrations.values());
    }
    static clear() {
        this.migrations.clear();
    }
}
MigrationRegistry.migrations = new Map();
/**
 * Декоратор для класса миграции
 */
export function Migration(config) {
    return function (constructor) {
        const migration = {
            id: config.id,
            version: config.version,
            description: config.description,
            type: config.type || 'custom',
            dependencies: config.dependencies || [],
            async up(context) {
                const instance = new constructor();
                if (typeof instance.up === 'function') {
                    return await instance.up(context);
                }
                throw new Error(`Migration ${config.id} must implement up() method`);
            },
            async down(context) {
                const instance = new constructor();
                if (typeof instance.down === 'function') {
                    return await instance.down(context);
                }
                throw new Error(`Migration ${config.id} must implement down() method`);
            },
            async canApply(context) {
                const instance = new constructor();
                if (typeof instance.canApply === 'function') {
                    return await instance.canApply(context);
                }
                return true;
            },
            async canRollback(context) {
                const instance = new constructor();
                if (typeof instance.canRollback === 'function') {
                    return await instance.canRollback(context);
                }
                return true;
            },
            async validate(context) {
                const instance = new constructor();
                if (typeof instance.validate === 'function') {
                    return await instance.validate(context);
                }
                return true;
            }
        };
        // Автоматически регистрируем миграцию
        MigrationRegistry.register(migration);
        return constructor;
    };
}
/**
 * Декоратор для методов миграции
 */
export function Up(_target, _propertyKey, descriptor) {
    // Метод уже определен в классе, декоратор просто помечает его
    return descriptor;
}
/**
 * Декоратор для методов отката
 */
export function Down(_target, _propertyKey, descriptor) {
    // Метод уже определен в классе, декоратор просто помечает его
    return descriptor;
}
/**
 * Декоратор для проверки возможности применения
 */
export function CanApply(_target, _propertyKey, descriptor) {
    return descriptor;
}
/**
 * Декоратор для проверки возможности отката
 */
export function CanRollback(_target, _propertyKey, descriptor) {
    return descriptor;
}
/**
 * Декоратор для валидации
 */
export function Validate(_target, _propertyKey, descriptor) {
    return descriptor;
}
/**
 * Декоратор для зависимостей
 */
export function DependsOn(...migrationIds) {
    return function (target, _propertyKey) {
        if (!target.dependencies) {
            target.dependencies = [];
        }
        target.dependencies.push(...migrationIds);
    };
}
/**
 * Декоратор для типа миграции
 */
export function Type(type) {
    return function (target, _propertyKey) {
        target.migrationType = type;
    };
}
/**
 * Базовый класс для миграций
 */
export class BaseMigration {
    async canApply(_context) {
        return true;
    }
    async canRollback(_context) {
        return true;
    }
    async validate(_context) {
        return true;
    }
    createResult(success, options = {}) {
        return {
            success,
            ...options
        };
    }
    async measureTime(operation) {
        const startTime = Date.now();
        const result = await operation();
        const executionTime = Date.now() - startTime;
        return { result, executionTime };
    }
}
/**
 * Миграция схемы
 */
export class SchemaMigration extends BaseMigration {
    async up(context) {
        const { executionTime } = await this.measureTime(async () => {
            return this.updateSchema(context);
        });
        return this.createResult(true, {
            message: `Schema ${this.schemaName} updated from ${this.oldVersion} to ${this.newVersion}`,
            processedCount: 1,
            executionTime
        });
    }
    async down(context) {
        const { executionTime } = await this.measureTime(async () => {
            return this.rollbackSchema(context);
        });
        return this.createResult(true, {
            message: `Schema ${this.schemaName} rolled back from ${this.newVersion} to ${this.oldVersion}`,
            processedCount: 1,
            executionTime
        });
    }
}
/**
 * Миграция данных
 */
export class DataMigration extends BaseMigration {
    constructor() {
        super(...arguments);
        this.batchSize = 100;
    }
    async up(context) {
        const { result, executionTime } = await this.measureTime(async () => {
            return this.transformData(context);
        });
        return this.createResult(true, {
            message: `Transformed ${result.processedCount} elements`,
            processedCount: result.processedCount,
            executionTime
        });
    }
    async down(context) {
        const { result, executionTime } = await this.measureTime(async () => {
            return this.rollbackData(context);
        });
        return this.createResult(true, {
            message: `Rolled back ${result.processedCount} elements`,
            processedCount: result.processedCount,
            executionTime
        });
    }
}
/**
 * Утилиты для создания миграций
 */
export class MigrationUtils {
    /**
     * Создает простую миграцию
     */
    static simple(config) {
        return {
            id: config.id,
            version: config.version,
            description: config.description,
            type: config.type || 'custom',
            dependencies: config.dependencies || [],
            up: config.up,
            down: config.down,
            canApply: config.canApply || (async () => true),
            canRollback: config.canRollback || (async () => true),
            validate: config.validate || (async () => true)
        };
    }
    /**
     * Создает миграцию только для применения (без отката)
     */
    static upOnly(config) {
        return this.simple({
            ...config,
            down: async () => ({
                success: false,
                error: 'This migration cannot be rolled back'
            }),
            canRollback: async () => false
        });
    }
    /**
     * Создает миграцию-заглушку
     */
    static noop(config) {
        return this.simple({
            ...config,
            up: async () => ({
                success: true,
                message: 'No-op migration executed successfully',
                processedCount: 0,
                executionTime: 0
            }),
            down: async () => ({
                success: true,
                message: 'No-op migration rolled back successfully',
                processedCount: 0,
                executionTime: 0
            })
        });
    }
}
//# sourceMappingURL=MigrationDecorators.js.map