// Типы аналитики
export var EventType;
(function (EventType) {
    EventType["CREATE"] = "create";
    EventType["READ"] = "read";
    EventType["UPDATE"] = "update";
    EventType["DELETE"] = "delete";
    EventType["QUERY"] = "query";
    EventType["SERIALIZE"] = "serialize";
    EventType["CUSTOM"] = "custom";
})(EventType || (EventType = {}));
export var MetricType;
(function (MetricType) {
    MetricType["COUNTER"] = "counter";
    MetricType["GAUGE"] = "gauge";
    MetricType["HISTOGRAM"] = "histogram";
    MetricType["TIMER"] = "timer";
    MetricType["RATE"] = "rate";
})(MetricType || (MetricType = {}));
export var AggregationType;
(function (AggregationType) {
    AggregationType["SUM"] = "sum";
    AggregationType["AVERAGE"] = "average";
    AggregationType["MIN"] = "min";
    AggregationType["MAX"] = "max";
    AggregationType["COUNT"] = "count";
    AggregationType["PERCENTILE"] = "percentile";
})(AggregationType || (AggregationType = {}));
export var ReportFormat;
(function (ReportFormat) {
    ReportFormat["JSON"] = "json";
    ReportFormat["CSV"] = "csv";
    ReportFormat["HTML"] = "html";
    ReportFormat["PDF"] = "pdf";
    ReportFormat["EXCEL"] = "excel";
})(ReportFormat || (ReportFormat = {}));
export class AnalyticsPlugin {
    constructor(config = {}) {
        this.name = 'AnalyticsPlugin';
        this.version = '1.0.0';
        this.description = 'Плагин для сбора аналитики использования и отслеживания событий';
        this.events = [];
        this.metrics = [];
        this.reports = new Map();
        this.dashboards = new Map();
        this.userBehaviors = new Map();
        this.sessionStartTimes = new Map();
        this.config = {
            enabled: true,
            priority: 50,
            enableRealTimeTracking: true,
            enableUserBehaviorTracking: true,
            enablePerformanceTracking: true,
            enableCustomEvents: true,
            samplingRate: 1.0,
            maxEventsInMemory: 10000,
            retentionPeriod: 30,
            batchSize: 100,
            flushInterval: 60000, // 1 минута
            exportFormats: [ReportFormat.JSON, ReportFormat.CSV],
            anonymizeUserData: false,
            excludeEvents: [],
            includeMetadata: true,
            enableGeolocation: false,
            ...config
        };
        this.stats = {
            totalEvents: 0,
            eventsToday: 0,
            eventsThisWeek: 0,
            eventsThisMonth: 0,
            eventsByType: {},
            uniqueUsers: 0,
            uniqueSessions: 0,
            averageSessionDuration: 0,
            topElements: [],
            topOperations: [],
            errorRate: 0,
            memoryUsage: 0
        };
        this.performanceMetrics = {
            averageResponseTime: 0,
            throughput: 0,
            errorRate: 0,
            memoryUsage: 0,
            cpuUsage: 0,
            operationsPerSecond: {},
            slowestOperations: [],
            peakUsageTime: new Date()
        };
        // Инициализация счетчиков событий
        Object.values(EventType).forEach(type => {
            this.stats.eventsByType[type] = 0;
            this.performanceMetrics.operationsPerSecond[type] = 0;
        });
    }
    // Реализация интерфейса CSElementPlugin
    install(CSElementClass) {
        // Устанавливаем плагин в CSElement
        // Здесь можно добавить хуки и middleware для отслеживания событий
        console.log(`AnalyticsPlugin установлен для ${CSElementClass.name}`);
    }
    async initialize() {
        console.log(`Инициализация ${this.name} v${this.version}`);
        // Запуск периодической отправки данных
        if (this.config.flushInterval > 0) {
            this.flushTimer = setInterval(() => {
                this.flushEvents();
            }, this.config.flushInterval);
        }
        // Очистка старых данных
        setInterval(() => {
            this.cleanupOldData();
        }, 24 * 60 * 60 * 1000); // каждый день
        // Обновление статистики
        setInterval(() => {
            this.updateStats();
        }, 60000); // каждую минуту
    }
    async destroy() {
        if (this.flushTimer) {
            clearInterval(this.flushTimer);
        }
        // Финальная отправка данных
        await this.flushEvents();
        console.log(`${this.name} деинициализирован`);
    }
    getConfig() {
        return { ...this.config };
    }
    updateConfig(newConfig) {
        this.config = { ...this.config, ...newConfig };
    }
    // Lifecycle hooks
    async beforeCreate(element, context) {
        const startTime = Date.now();
        return this.withPerformanceTracking(async () => {
            await this.trackEvent({
                type: EventType.CREATE,
                elementId: element.id,
                success: true,
                metadata: {
                    elementType: element.constructor.name,
                    hasChildren: element.elementsCount() > 0,
                    dataKeys: Array.from(element.data.keys())
                }
            }, context);
            return { success: true };
        }, EventType.CREATE, startTime);
    }
    async afterCreate(element, context) {
        // Обновление пользовательского поведения
        if (this.config.enableUserBehaviorTracking) {
            this.updateUserBehavior(context, EventType.CREATE, element.id);
        }
        return { success: true };
    }
    async beforeRead(element, context) {
        const startTime = Date.now();
        return this.withPerformanceTracking(async () => {
            await this.trackEvent({
                type: EventType.READ,
                elementId: element.id,
                success: true,
                metadata: {
                    dataSize: JSON.stringify(Object.fromEntries(element.data)).length,
                    childrenCount: element.elementsCount()
                }
            }, context);
            return { success: true };
        }, EventType.READ, startTime);
    }
    async beforeUpdate(element, context) {
        const startTime = Date.now();
        return this.withPerformanceTracking(async () => {
            await this.trackEvent({
                type: EventType.UPDATE,
                elementId: element.id,
                success: true,
                metadata: {
                    previousDataSize: JSON.stringify(Object.fromEntries(element.data)).length,
                    operation: context.operation
                }
            }, context);
            return { success: true };
        }, EventType.UPDATE, startTime);
    }
    async beforeDelete(element, context) {
        const startTime = Date.now();
        return this.withPerformanceTracking(async () => {
            await this.trackEvent({
                type: EventType.DELETE,
                elementId: element.id,
                success: true,
                metadata: {
                    hadChildren: element.elementsCount() > 0,
                    dataKeys: Array.from(element.data.keys())
                }
            }, context);
            return { success: true };
        }, EventType.DELETE, startTime);
    }
    async onError(error, context) {
        await this.trackEvent({
            type: EventType.CUSTOM,
            success: false,
            error: error.message,
            metadata: {
                errorType: error.constructor.name,
                stack: error.stack,
                operation: context.operation
            },
            category: 'error'
        }, context);
        return { success: true };
    }
    // Методы отслеживания
    async trackEvent(eventData, context) {
        if (!this.config.enabled || !this.shouldSampleEvent()) {
            return;
        }
        if (eventData.type && this.config.excludeEvents.includes(eventData.type)) {
            return;
        }
        const event = {
            id: this.generateId(),
            type: eventData.type || EventType.CUSTOM,
            timestamp: new Date(),
            tags: [],
            metadata: {},
            success: true,
            ...eventData,
            userId: this.config.anonymizeUserData ? this.hashUserId(context.userId) : context.userId,
            sessionId: context.sessionId,
            userAgent: context.metadata?.userAgent,
            ipAddress: this.config.anonymizeUserData ? this.hashIpAddress(context.metadata?.ipAddress) : context.metadata?.ipAddress
        };
        this.events.push(event);
        this.stats.totalEvents++;
        this.stats.eventsByType[event.type]++;
        // Ограничение памяти
        if (this.events.length > this.config.maxEventsInMemory) {
            await this.flushEvents();
        }
        // Обновление метрик в реальном времени
        if (this.config.enableRealTimeTracking) {
            this.updateRealTimeMetrics(event);
        }
    }
    async trackCustomEvent(name, data, context) {
        if (!this.config.enableCustomEvents)
            return;
        await this.trackEvent({
            type: EventType.CUSTOM,
            success: true,
            metadata: { customEventName: name, ...data },
            category: 'custom',
            tags: [name]
        }, context);
    }
    recordMetric(name, value, type = MetricType.GAUGE, tags = {}) {
        const metric = {
            id: this.generateId(),
            name,
            type,
            value,
            timestamp: new Date(),
            tags
        };
        this.metrics.push(metric);
    }
    // Запросы и отчеты
    async query(query) {
        let filteredEvents = [...this.events];
        // Фильтрация по типам событий
        if (query.eventTypes && query.eventTypes.length > 0) {
            filteredEvents = filteredEvents.filter(event => query.eventTypes.includes(event.type));
        }
        // Фильтрация по дате
        if (query.dateRange) {
            filteredEvents = filteredEvents.filter(event => event.timestamp >= query.dateRange.start &&
                event.timestamp <= query.dateRange.end);
        }
        // Дополнительные фильтры
        if (query.filters) {
            filteredEvents = filteredEvents.filter(event => {
                return Object.entries(query.filters).every(([key, value]) => {
                    const eventValue = this.getNestedValue(event, key);
                    return eventValue === value;
                });
            });
        }
        // Сортировка
        if (query.sortBy) {
            filteredEvents.sort((a, b) => {
                const aValue = this.getNestedValue(a, query.sortBy);
                const bValue = this.getNestedValue(b, query.sortBy);
                if (query.sortOrder === 'desc') {
                    return bValue > aValue ? 1 : -1;
                }
                return aValue > bValue ? 1 : -1;
            });
        }
        // Пагинация
        if (query.offset || query.limit) {
            const start = query.offset || 0;
            const end = query.limit ? start + query.limit : undefined;
            filteredEvents = filteredEvents.slice(start, end);
        }
        return filteredEvents;
    }
    async generateReport(name, query, format = ReportFormat.JSON) {
        const events = await this.query(query);
        const summary = this.calculateSummary(events);
        const report = {
            id: this.generateId(),
            name,
            query,
            format,
            createdAt: new Date(),
            updatedAt: new Date(),
            data: events,
            summary,
            metadata: {
                totalEvents: events.length,
                generatedAt: new Date(),
                format
            }
        };
        this.reports.set(report.id, report);
        return report;
    }
    async exportReport(reportId, format) {
        const report = this.reports.get(reportId);
        if (!report) {
            throw new Error(`Отчет с ID ${reportId} не найден`);
        }
        const exportFormat = format || report.format;
        switch (exportFormat) {
            case ReportFormat.JSON:
                return JSON.stringify(report, null, 2);
            case ReportFormat.CSV:
                return this.convertToCSV(report.data);
            case ReportFormat.HTML:
                return this.convertToHTML(report);
            default:
                throw new Error(`Формат ${exportFormat} не поддерживается`);
        }
    }
    // Дашборды
    createDashboard(dashboardData) {
        const dashboard = {
            id: this.generateId(),
            createdAt: new Date(),
            updatedAt: new Date(),
            ...dashboardData
        };
        this.dashboards.set(dashboard.id, dashboard);
        return dashboard;
    }
    getDashboard(id) {
        return this.dashboards.get(id);
    }
    updateDashboard(id, updates) {
        const dashboard = this.dashboards.get(id);
        if (!dashboard)
            return false;
        Object.assign(dashboard, { ...updates, updatedAt: new Date() });
        return true;
    }
    deleteDashboard(id) {
        return this.dashboards.delete(id);
    }
    // Анализ поведения пользователей
    getUserBehavior(userId) {
        return this.userBehaviors.get(userId);
    }
    async analyzeUserBehavior(userId) {
        const behavior = this.userBehaviors.get(userId);
        if (!behavior)
            return {};
        const userEvents = this.events.filter(e => e.userId === userId);
        return {
            totalEvents: userEvents.length,
            sessionDuration: behavior.endTime ?
                behavior.endTime.getTime() - behavior.startTime.getTime() :
                Date.now() - behavior.startTime.getTime(),
            mostAccessedElements: this.getMostAccessedElements(userEvents),
            operationDistribution: behavior.operationsPerformed,
            averageTimePerElement: this.calculateAverageTimePerElement(behavior),
            errorRate: behavior.errorCount / userEvents.length,
            successRate: behavior.successRate,
            peakActivityTime: this.findPeakActivityTime(userEvents)
        };
    }
    // Метрики производительности
    getPerformanceMetrics() {
        return { ...this.performanceMetrics };
    }
    async generatePerformanceReport() {
        const recentEvents = this.events.filter(e => Date.now() - e.timestamp.getTime() < 24 * 60 * 60 * 1000 // последние 24 часа
        );
        return {
            totalEvents: recentEvents.length,
            averageResponseTime: this.performanceMetrics.averageResponseTime,
            throughput: this.performanceMetrics.throughput,
            errorRate: this.performanceMetrics.errorRate,
            operationsPerSecond: this.performanceMetrics.operationsPerSecond,
            slowestOperations: this.performanceMetrics.slowestOperations.slice(0, 10),
            memoryUsage: this.performanceMetrics.memoryUsage,
            peakUsageTime: this.performanceMetrics.peakUsageTime,
            eventDistribution: this.calculateEventDistribution(recentEvents)
        };
    }
    // Утилиты
    async withPerformanceTracking(operation, eventType, startTime) {
        try {
            const result = await operation();
            const duration = Date.now() - startTime;
            if (this.config.enablePerformanceTracking) {
                this.recordPerformanceMetric(eventType, duration, true);
            }
            return result;
        }
        catch (error) {
            const duration = Date.now() - startTime;
            if (this.config.enablePerformanceTracking) {
                this.recordPerformanceMetric(eventType, duration, false);
            }
            throw error;
        }
    }
    recordPerformanceMetric(eventType, duration, success) {
        // Обновление средних значений
        const currentAvg = this.performanceMetrics.averageResponseTime;
        this.performanceMetrics.averageResponseTime = (currentAvg + duration) / 2;
        // Обновление throughput
        this.performanceMetrics.throughput++;
        // Обновление error rate
        if (!success) {
            this.performanceMetrics.errorRate =
                (this.performanceMetrics.errorRate + 1) / this.performanceMetrics.throughput;
        }
        // Запись медленных операций
        if (duration > 1000) { // операции дольше 1 секунды
            this.performanceMetrics.slowestOperations.push({
                operation: eventType,
                duration,
                timestamp: new Date()
            });
            // Ограничение размера списка
            if (this.performanceMetrics.slowestOperations.length > 100) {
                this.performanceMetrics.slowestOperations =
                    this.performanceMetrics.slowestOperations.slice(-100);
            }
        }
    }
    updateUserBehavior(context, eventType, elementId) {
        const userId = context.userId;
        const sessionId = context.sessionId;
        if (!userId || !sessionId)
            return;
        let behavior = this.userBehaviors.get(userId);
        if (!behavior) {
            behavior = {
                userId,
                sessionId,
                startTime: new Date(),
                events: [],
                elementsAccessed: [],
                operationsPerformed: {},
                timeSpentByElement: {},
                errorCount: 0,
                successRate: 1
            };
            this.userBehaviors.set(userId, behavior);
            this.sessionStartTimes.set(sessionId, new Date());
        }
        // Обновление статистики операций
        behavior.operationsPerformed[eventType] = (behavior.operationsPerformed[eventType] || 0) + 1;
        // Добавление элемента в список доступа
        if (elementId && !behavior.elementsAccessed.includes(elementId)) {
            behavior.elementsAccessed.push(elementId);
        }
        // Обновление времени окончания сессии
        behavior.endTime = new Date();
    }
    shouldSampleEvent() {
        return Math.random() < this.config.samplingRate;
    }
    updateRealTimeMetrics(event) {
        // Обновление операций в секунду
        if (!this.performanceMetrics.operationsPerSecond[event.type]) {
            this.performanceMetrics.operationsPerSecond[event.type] = 0;
        }
        this.performanceMetrics.operationsPerSecond[event.type]++;
    }
    async flushEvents() {
        if (this.events.length === 0)
            return;
        // Здесь должна быть отправка данных во внешнее хранилище
        console.log(`Отправка ${this.events.length} событий в аналитическое хранилище`);
        // Очистка буфера
        this.events = [];
        this.stats.lastFlush = new Date();
    }
    cleanupOldData() {
        const cutoffDate = new Date();
        cutoffDate.setDate(cutoffDate.getDate() - this.config.retentionPeriod);
        // Очистка старых событий
        const initialEventCount = this.events.length;
        this.events = this.events.filter(event => event.timestamp > cutoffDate);
        // Очистка старых метрик
        this.metrics = this.metrics.filter(metric => metric.timestamp > cutoffDate);
        console.log(`Очищено ${initialEventCount - this.events.length} старых событий`);
    }
    updateStats() {
        const now = new Date();
        const today = new Date(now.getFullYear(), now.getMonth(), now.getDate());
        const thisWeek = new Date(today.getTime() - 7 * 24 * 60 * 60 * 1000);
        const thisMonth = new Date(today.getTime() - 30 * 24 * 60 * 60 * 1000);
        this.stats.eventsToday = this.events.filter(e => e.timestamp >= today).length;
        this.stats.eventsThisWeek = this.events.filter(e => e.timestamp >= thisWeek).length;
        this.stats.eventsThisMonth = this.events.filter(e => e.timestamp >= thisMonth).length;
        // Уникальные пользователи и сессии
        const uniqueUsers = new Set(this.events.map(e => e.userId).filter(Boolean));
        const uniqueSessions = new Set(this.events.map(e => e.sessionId).filter(Boolean));
        this.stats.uniqueUsers = uniqueUsers.size;
        this.stats.uniqueSessions = uniqueSessions.size;
        // Топ элементы и операции
        this.stats.topElements = this.calculateTopElements();
        this.stats.topOperations = this.calculateTopOperations();
        // Error rate
        const totalEvents = this.events.length;
        const errorEvents = this.events.filter(e => !e.success).length;
        this.stats.errorRate = totalEvents > 0 ? errorEvents / totalEvents : 0;
        // Использование памяти
        this.stats.memoryUsage = this.estimateMemoryUsage();
    }
    calculateSummary(events) {
        return {
            totalEvents: events.length,
            uniqueUsers: new Set(events.map(e => e.userId).filter(Boolean)).size,
            uniqueSessions: new Set(events.map(e => e.sessionId).filter(Boolean)).size,
            eventTypes: this.groupBy(events, 'type'),
            successRate: events.filter(e => e.success).length / events.length,
            averageDuration: events.filter(e => e.duration).reduce((sum, e) => sum + (e.duration || 0), 0) / events.filter(e => e.duration).length,
            timeRange: {
                start: Math.min(...events.map(e => e.timestamp.getTime())),
                end: Math.max(...events.map(e => e.timestamp.getTime()))
            }
        };
    }
    calculateTopElements() {
        const elementCounts = new Map();
        this.events.forEach(event => {
            if (event.elementId) {
                elementCounts.set(event.elementId, (elementCounts.get(event.elementId) || 0) + 1);
            }
        });
        return Array.from(elementCounts.entries())
            .map(([elementId, accessCount]) => ({ elementId, accessCount }))
            .sort((a, b) => b.accessCount - a.accessCount)
            .slice(0, 10);
    }
    calculateTopOperations() {
        return Object.entries(this.stats.eventsByType)
            .map(([operation, count]) => ({ operation: operation, count }))
            .sort((a, b) => b.count - a.count)
            .slice(0, 10);
    }
    getMostAccessedElements(events) {
        const counts = new Map();
        events.forEach(event => {
            if (event.elementId) {
                counts.set(event.elementId, (counts.get(event.elementId) || 0) + 1);
            }
        });
        return Array.from(counts.entries())
            .map(([elementId, count]) => ({ elementId, count }))
            .sort((a, b) => b.count - a.count);
    }
    calculateAverageTimePerElement(behavior) {
        const result = {};
        Object.entries(behavior.timeSpentByElement).forEach(([elementId, totalTime]) => {
            const accessCount = behavior.elementsAccessed.filter(id => id === elementId).length;
            result[elementId] = totalTime / accessCount;
        });
        return result;
    }
    findPeakActivityTime(events) {
        if (events.length === 0)
            return null;
        const hourCounts = new Map();
        events.forEach(event => {
            const hour = event.timestamp.getHours();
            hourCounts.set(hour, (hourCounts.get(hour) || 0) + 1);
        });
        const peakHour = Array.from(hourCounts.entries())
            .sort((a, b) => b[1] - a[1])[0];
        if (!peakHour)
            return null;
        const peakDate = new Date();
        peakDate.setHours(peakHour[0], 0, 0, 0);
        return peakDate;
    }
    calculateEventDistribution(events) {
        const distribution = {};
        events.forEach(event => {
            const hour = event.timestamp.getHours();
            const key = `${hour}:00`;
            distribution[key] = (distribution[key] || 0) + 1;
        });
        return distribution;
    }
    convertToCSV(data) {
        if (data.length === 0)
            return '';
        const headers = Object.keys(data[0]);
        const csvRows = [headers.join(',')];
        data.forEach(row => {
            const values = headers.map(header => {
                const value = row[header];
                return typeof value === 'string' ? `"${value}"` : value;
            });
            csvRows.push(values.join(','));
        });
        return csvRows.join('\n');
    }
    convertToHTML(report) {
        return `
            <!DOCTYPE html>
            <html>
            <head>
                <title>Отчет: ${report.name}</title>
                <style>
                    body { font-family: Arial, sans-serif; margin: 20px; }
                    table { border-collapse: collapse; width: 100%; }
                    th, td { border: 1px solid #ddd; padding: 8px; text-align: left; }
                    th { background-color: #f2f2f2; }
                    .summary { background-color: #f9f9f9; padding: 15px; margin: 20px 0; }
                </style>
            </head>
            <body>
                <h1>Отчет: ${report.name}</h1>
                <div class="summary">
                    <h2>Сводка</h2>
                    <pre>${JSON.stringify(report.summary, null, 2)}</pre>
                </div>
                <h2>Данные</h2>
                <p>Сгенерировано: ${report.createdAt.toLocaleString()}</p>
                <p>Всего событий: ${report.data.length}</p>
            </body>
            </html>
        `;
    }
    getNestedValue(obj, path) {
        return path.split('.').reduce((current, key) => current?.[key], obj);
    }
    groupBy(array, key) {
        return array.reduce((groups, item) => {
            const value = this.getNestedValue(item, key);
            groups[value] = (groups[value] || 0) + 1;
            return groups;
        }, {});
    }
    estimateMemoryUsage() {
        // Примерная оценка использования памяти в байтах
        const eventSize = JSON.stringify(this.events[0] || {}).length;
        const metricSize = JSON.stringify(this.metrics[0] || {}).length;
        return (this.events.length * eventSize) + (this.metrics.length * metricSize);
    }
    hashUserId(userId) {
        if (!userId)
            return undefined;
        // Простое хэширование для анонимизации
        return 'user_' + userId.split('').reduce((hash, char) => ((hash << 5) - hash) + char.charCodeAt(0), 0).toString(36);
    }
    hashIpAddress(ip) {
        if (!ip)
            return undefined;
        // Маскирование IP адреса
        const parts = ip.split('.');
        if (parts.length === 4) {
            return `${parts[0]}.${parts[1]}.xxx.xxx`;
        }
        return 'masked_ip';
    }
    generateId() {
        return Date.now().toString(36) + Math.random().toString(36).substr(2);
    }
    // Публичные методы для получения данных
    getStats() {
        return { ...this.stats };
    }
    getAllEvents() {
        return [...this.events];
    }
    getAllMetrics() {
        return [...this.metrics];
    }
    getAllReports() {
        return Array.from(this.reports.values());
    }
    getAllDashboards() {
        return Array.from(this.dashboards.values());
    }
    getAllUserBehaviors() {
        return Array.from(this.userBehaviors.values());
    }
}
//# sourceMappingURL=AnalyticsPlugin.js.map