// Типы резервного копирования
export var BackupType;
(function (BackupType) {
    BackupType["FULL"] = "full";
    BackupType["INCREMENTAL"] = "incremental";
    BackupType["DIFFERENTIAL"] = "differential";
    BackupType["SNAPSHOT"] = "snapshot";
})(BackupType || (BackupType = {}));
export var BackupStatus;
(function (BackupStatus) {
    BackupStatus["PENDING"] = "pending";
    BackupStatus["IN_PROGRESS"] = "in_progress";
    BackupStatus["COMPLETED"] = "completed";
    BackupStatus["FAILED"] = "failed";
    BackupStatus["CANCELLED"] = "cancelled";
})(BackupStatus || (BackupStatus = {}));
export var CompressionType;
(function (CompressionType) {
    CompressionType["NONE"] = "none";
    CompressionType["GZIP"] = "gzip";
    CompressionType["BROTLI"] = "brotli";
    CompressionType["LZ4"] = "lz4";
})(CompressionType || (CompressionType = {}));
export var StorageLocation;
(function (StorageLocation) {
    StorageLocation["LOCAL"] = "local";
    StorageLocation["CLOUD"] = "cloud";
    StorageLocation["NETWORK"] = "network";
    StorageLocation["DATABASE"] = "database";
})(StorageLocation || (StorageLocation = {}));
export var RestoreMode;
(function (RestoreMode) {
    RestoreMode["FULL_RESTORE"] = "full_restore";
    RestoreMode["SELECTIVE_RESTORE"] = "selective_restore";
    RestoreMode["POINT_IN_TIME"] = "point_in_time";
    RestoreMode["MERGE"] = "merge";
})(RestoreMode || (RestoreMode = {}));
export class BackupPlugin {
    install(CSElementClass) {
        console.log(`BackupPlugin установлен для ${CSElementClass.name}`);
    }
    constructor(config = {}) {
        this.name = 'BackupPlugin';
        this.version = '1.0.0';
        this.description = 'Плагин для резервного копирования и восстановления данных';
        this.backups = new Map();
        this.schedules = new Map();
        this.activeJobs = new Map();
        this.config = {
            enabled: true,
            priority: 200,
            enableAutoBackup: true,
            defaultBackupType: BackupType.INCREMENTAL,
            defaultCompression: CompressionType.GZIP,
            defaultStorageLocation: StorageLocation.LOCAL,
            maxBackupSize: 1024 * 1024 * 1024, // 1GB
            maxConcurrentJobs: 3,
            retentionPolicy: {
                maxAge: 90,
                maxCount: 50,
                keepDaily: 7,
                keepWeekly: 4,
                keepMonthly: 12,
                keepYearly: 5
            },
            encryptBackups: false,
            verifyIntegrity: true,
            includePluginData: true,
            backupPath: './backups',
            tempPath: './temp',
            chunkSize: 1024 * 1024, // 1MB
            ...config
        };
        this.stats = {
            totalBackups: 0,
            successfulBackups: 0,
            failedBackups: 0,
            totalSize: 0,
            compressedSize: 0,
            compressionRatio: 0,
            averageBackupTime: 0,
            activeJobs: 0,
            scheduledJobs: 0
        };
    }
    async initialize() {
        console.log(`Инициализация ${this.name} v${this.version}`);
        // Создание директорий
        await this.ensureDirectories();
        // Загрузка существующих бэкапов
        await this.loadExistingBackups();
        // Запуск планировщика
        if (this.config.enableAutoBackup) {
            this.startScheduler();
        }
        // Очистка старых бэкапов
        setInterval(() => {
            this.cleanupOldBackups();
        }, 24 * 60 * 60 * 1000); // каждый день
    }
    async destroy() {
        if (this.schedulerTimer) {
            clearInterval(this.schedulerTimer);
        }
        // Завершение активных задач
        for (const job of this.activeJobs.values()) {
            if (job.status === BackupStatus.IN_PROGRESS) {
                job.status = BackupStatus.CANCELLED;
            }
        }
        console.log(`${this.name} деинициализирован`);
    }
    getConfig() {
        return { ...this.config };
    }
    updateConfig(newConfig) {
        this.config = { ...this.config, ...newConfig };
    }
    // Lifecycle hooks
    async afterCreate(element, _context) {
        if (this.config.enableAutoBackup && this.shouldTriggerIncrementalBackup()) {
            await this.createIncrementalBackup(`Auto backup after create ${element.id}`);
        }
        return { success: true };
    }
    async afterUpdate(element, _context) {
        if (this.config.enableAutoBackup && this.shouldTriggerIncrementalBackup()) {
            await this.createIncrementalBackup(`Auto backup after update ${element.id}`);
        }
        return { success: true };
    }
    async afterDelete(element, _context) {
        if (this.config.enableAutoBackup) {
            await this.createIncrementalBackup(`Auto backup after delete ${element.id}`);
        }
        return { success: true };
    }
    // Основные методы резервного копирования
    async createBackup(name, type = this.config.defaultBackupType, options = {}) {
        const jobId = this.generateId();
        const job = {
            id: jobId,
            type,
            status: BackupStatus.PENDING,
            startTime: new Date(),
            progress: 0,
            currentOperation: 'Инициализация',
            elementsProcessed: 0,
            totalElements: 0,
            errors: []
        };
        this.activeJobs.set(jobId, job);
        this.stats.activeJobs++;
        try {
            job.status = BackupStatus.IN_PROGRESS;
            job.currentOperation = 'Сбор данных';
            // Сбор данных для бэкапа
            const backupData = await this.collectBackupData(type, job);
            job.currentOperation = 'Сериализация';
            job.progress = 60;
            // Создание метаданных
            const metadata = {
                id: this.generateId(),
                name,
                type,
                status: BackupStatus.IN_PROGRESS,
                createdAt: new Date(),
                size: 0,
                compressedSize: 0,
                compressionType: this.config.defaultCompression,
                storageLocation: this.config.defaultStorageLocation,
                elementCount: backupData.elements.length,
                checksum: '',
                version: this.version,
                tags: [],
                dependencies: type === BackupType.INCREMENTAL ? this.getIncrementalDependencies() : [],
                isEncrypted: this.config.encryptBackups,
                retentionPolicy: this.config.retentionPolicy,
                ...options
            };
            backupData.metadata = metadata;
            job.currentOperation = 'Сжатие и сохранение';
            job.progress = 80;
            // Сохранение бэкапа
            await this.saveBackup(backupData, job);
            job.status = BackupStatus.COMPLETED;
            job.endTime = new Date();
            job.progress = 100;
            job.result = metadata;
            metadata.status = BackupStatus.COMPLETED;
            metadata.completedAt = new Date();
            this.backups.set(metadata.id, metadata);
            this.updateStats();
            console.log(`Бэкап ${name} успешно создан (ID: ${metadata.id})`);
            return metadata.id;
        }
        catch (error) {
            job.status = BackupStatus.FAILED;
            job.endTime = new Date();
            job.errors.push(error instanceof Error ? error.message : String(error));
            this.stats.failedBackups++;
            this.stats.lastFailedBackup = new Date();
            throw error;
        }
        finally {
            this.stats.activeJobs--;
            setTimeout(() => {
                this.activeJobs.delete(jobId);
            }, 60000); // удаляем через минуту
        }
    }
    async createFullBackup(name) {
        this.lastFullBackup = new Date();
        return this.createBackup(name, BackupType.FULL);
    }
    async createIncrementalBackup(name) {
        if (!this.lastFullBackup) {
            return this.createFullBackup(`Auto full backup - ${name}`);
        }
        return this.createBackup(name, BackupType.INCREMENTAL);
    }
    async createSnapshot(name) {
        return this.createBackup(name, BackupType.SNAPSHOT);
    }
    // Восстановление
    async restoreBackup(backupId, options) {
        const backup = this.backups.get(backupId);
        if (!backup) {
            throw new Error(`Бэкап с ID ${backupId} не найден`);
        }
        const startTime = Date.now();
        const result = {
            success: false,
            restoredElements: [],
            skippedElements: [],
            conflicts: [],
            errors: [],
            duration: 0,
            timestamp: new Date()
        };
        try {
            console.log(`Начало восстановления бэкапа ${backup.name}`);
            // Загрузка данных бэкапа
            const backupData = await this.loadBackup(backupId);
            if (options.dryRun) {
                console.log('Режим проверки - изменения не будут применены');
                return this.simulateRestore(backupData, options);
            }
            // Валидация целостности
            if (options.validateIntegrity) {
                const isValid = await this.validateBackupIntegrity(backupData);
                if (!isValid) {
                    throw new Error('Нарушена целостность данных бэкапа');
                }
            }
            // Восстановление элементов
            await this.restoreElements(backupData, options, result);
            // Восстановление отношений
            if (options.restoreRelationships) {
                await this.restoreRelationships(backupData, result);
            }
            // Восстановление плагинов
            if (options.restorePlugins) {
                await this.restorePlugins(backupData, result);
            }
            result.success = result.errors.length === 0;
            console.log(`Восстановление завершено успешно. Восстановлено ${result.restoredElements.length} элементов`);
        }
        catch (error) {
            result.errors.push(error instanceof Error ? error.message : String(error));
            result.success = false;
            console.error('Ошибка при восстановлении:', error);
        }
        finally {
            result.duration = Date.now() - startTime;
        }
        return result;
    }
    // Управление расписанием
    createSchedule(scheduleData) {
        const schedule = {
            id: this.generateId(),
            lastRun: undefined,
            nextRun: this.calculateNextRun(scheduleData.cron),
            failureCount: 0,
            ...scheduleData
        };
        this.schedules.set(schedule.id, schedule);
        this.stats.scheduledJobs++;
        console.log(`Создано расписание бэкапов: ${schedule.name}`);
        return schedule.id;
    }
    getSchedule(id) {
        return this.schedules.get(id);
    }
    updateSchedule(id, updates) {
        const schedule = this.schedules.get(id);
        if (!schedule)
            return false;
        Object.assign(schedule, updates);
        if (updates.cron) {
            schedule.nextRun = this.calculateNextRun(updates.cron);
        }
        return true;
    }
    deleteSchedule(id) {
        const deleted = this.schedules.delete(id);
        if (deleted) {
            this.stats.scheduledJobs--;
        }
        return deleted;
    }
    // Управление бэкапами
    getBackup(id) {
        return this.backups.get(id);
    }
    getAllBackups() {
        return Array.from(this.backups.values()).sort((a, b) => b.createdAt.getTime() - a.createdAt.getTime());
    }
    async deleteBackup(id) {
        const backup = this.backups.get(id);
        if (!backup)
            return false;
        try {
            // Удаление файла бэкапа
            await this.deleteBackupFile(backup);
            // Удаление из реестра
            this.backups.delete(id);
            this.updateStats();
            console.log(`Бэкап ${backup.name} удален`);
            return true;
        }
        catch (error) {
            console.error('Ошибка при удалении бэкапа:', error);
            return false;
        }
    }
    // Утилиты
    async verifyBackup(id) {
        try {
            const backupData = await this.loadBackup(id);
            return this.validateBackupIntegrity(backupData);
        }
        catch (error) {
            console.error('Ошибка при проверке бэкапа:', error);
            return false;
        }
    }
    async getBackupInfo(id) {
        const backup = this.backups.get(id);
        if (!backup)
            return null;
        try {
            const backupData = await this.loadBackup(id);
            return {
                metadata: backup,
                elementCount: backupData.elements.length,
                relationshipCount: backupData.relationships.length,
                pluginCount: backupData.plugins.length,
                dataSize: this.calculateDataSize(backupData),
                isValid: await this.validateBackupIntegrity(backupData)
            };
        }
        catch (error) {
            return {
                metadata: backup,
                error: error instanceof Error ? error.message : String(error)
            };
        }
    }
    // Приватные методы
    async collectBackupData(_type, job) {
        // Заглушка для сбора данных
        // В реальной реализации здесь должен быть доступ к CSElement registry
        const elements = [];
        const relationships = [];
        const plugins = [];
        job.totalElements = 100; // заглушка
        job.currentOperation = 'Сериализация элементов';
        // Симуляция сбора данных
        for (let i = 0; i < job.totalElements; i++) {
            elements.push({
                id: `element_${i}`,
                data: { value: i },
                metadata: {},
                childrenIds: [],
                createdAt: new Date(),
                updatedAt: new Date()
            });
            job.elementsProcessed = i + 1;
            job.progress = Math.floor((i / job.totalElements) * 50);
        }
        return {
            metadata: {}, // будет заполнено позже
            elements,
            relationships,
            globalData: {},
            plugins,
            timestamp: new Date(),
            schemaVersion: '1.0.0'
        };
    }
    async saveBackup(backupData, _job) {
        const serialized = JSON.stringify(backupData);
        const size = Buffer.byteLength(serialized, 'utf8');
        // Сжатие
        const compressed = await this.compress(serialized, this.config.defaultCompression);
        const compressedSize = compressed.length;
        // Вычисление контрольной суммы
        const checksum = this.calculateChecksum(compressed);
        // Обновление метаданных
        backupData.metadata.size = size;
        backupData.metadata.compressedSize = compressedSize;
        backupData.metadata.checksum = checksum;
        // Сохранение (заглушка)
        console.log(`Сохранение бэкапа размером ${size} байт (сжато: ${compressedSize} байт)`);
        this.stats.totalSize += size;
        this.stats.compressedSize += compressedSize;
        this.stats.successfulBackups++;
        this.stats.lastBackupTime = new Date();
    }
    async loadBackup(id) {
        // Заглушка для загрузки бэкапа
        const backup = this.backups.get(id);
        if (!backup) {
            throw new Error(`Бэкап ${id} не найден`);
        }
        // Симуляция загрузки с тестовыми данными
        return {
            metadata: backup,
            elements: [
                {
                    id: 'test-element-1',
                    data: { key: 'value' },
                    metadata: { test: true },
                    childrenIds: [],
                    createdAt: new Date(),
                    updatedAt: new Date()
                }
            ],
            relationships: [],
            globalData: {},
            plugins: [],
            timestamp: backup.createdAt,
            schemaVersion: '1.0.0'
        };
    }
    async restoreElements(backupData, options, result) {
        const elementsToRestore = options.targetElements ?
            backupData.elements.filter(e => options.targetElements.includes(e.id)) :
            backupData.elements;
        for (const element of elementsToRestore) {
            try {
                // Здесь должна быть логика восстановления элемента
                // Проверка конфликтов, применение стратегии разрешения и т.д.
                result.restoredElements.push(element.id);
            }
            catch (error) {
                result.errors.push(`Ошибка восстановления элемента ${element.id}: ${error}`);
                result.skippedElements.push(element.id);
            }
        }
    }
    async restoreRelationships(backupData, _result) {
        // Заглушка для восстановления отношений
        console.log(`Восстановление ${backupData.relationships.length} отношений`);
    }
    async restorePlugins(backupData, _result) {
        // Заглушка для восстановления плагинов
        console.log(`Восстановление ${backupData.plugins.length} плагинов`);
    }
    async simulateRestore(backupData, _options) {
        // Симуляция восстановления для режима проверки
        return {
            success: true,
            restoredElements: backupData.elements.map(e => e.id),
            skippedElements: [],
            conflicts: [],
            errors: [],
            duration: 0,
            timestamp: new Date()
        };
    }
    async validateBackupIntegrity(backupData) {
        try {
            // Простая проверка целостности - наличие основных полей
            return !!(backupData &&
                backupData.metadata &&
                backupData.elements &&
                Array.isArray(backupData.elements) &&
                backupData.timestamp);
        }
        catch (error) {
            console.error('Ошибка валидации целостности:', error);
            return false;
        }
    }
    startScheduler() {
        this.schedulerTimer = setInterval(() => {
            this.runScheduledBackups();
        }, 60000); // проверка каждую минуту
    }
    async runScheduledBackups() {
        const now = new Date();
        for (const schedule of this.schedules.values()) {
            if (!schedule.enabled || !schedule.nextRun)
                continue;
            if (now >= schedule.nextRun) {
                try {
                    if (this.stats.activeJobs >= this.config.maxConcurrentJobs) {
                        console.log('Достигнуто максимальное количество одновременных задач');
                        continue;
                    }
                    const backupName = `${schedule.name} - ${now.toISOString()}`;
                    await this.createBackup(backupName, schedule.type);
                    schedule.lastRun = now;
                    schedule.nextRun = this.calculateNextRun(schedule.cron);
                    schedule.failureCount = 0;
                }
                catch (error) {
                    schedule.failureCount++;
                    console.error(`Ошибка выполнения запланированного бэкапа ${schedule.name}:`, error);
                    if (schedule.failureCount >= schedule.maxRetries) {
                        schedule.enabled = false;
                        console.error(`Расписание ${schedule.name} отключено из-за превышения количества ошибок`);
                    }
                }
            }
        }
    }
    calculateNextRun(_cron) {
        // Упрощенная реализация парсинга cron
        // В реальной реализации следует использовать библиотеку типа node-cron
        const now = new Date();
        return new Date(now.getTime() + 24 * 60 * 60 * 1000); // +1 день
    }
    shouldTriggerIncrementalBackup() {
        if (!this.lastFullBackup)
            return false;
        const lastIncremental = Array.from(this.backups.values())
            .filter(b => b.type === BackupType.INCREMENTAL)
            .sort((a, b) => b.createdAt.getTime() - a.createdAt.getTime())[0];
        if (!lastIncremental)
            return true;
        // Создавать инкрементальный бэкап каждые 30 минут
        const timeSinceLastIncremental = Date.now() - lastIncremental.createdAt.getTime();
        return timeSinceLastIncremental > 30 * 60 * 1000;
    }
    getIncrementalDependencies() {
        const lastFull = Array.from(this.backups.values())
            .filter(b => b.type === BackupType.FULL)
            .sort((a, b) => b.createdAt.getTime() - a.createdAt.getTime())[0];
        if (!lastFull)
            return [];
        const incrementals = Array.from(this.backups.values())
            .filter(b => b.type === BackupType.INCREMENTAL &&
            b.createdAt > lastFull.createdAt)
            .sort((a, b) => a.createdAt.getTime() - b.createdAt.getTime());
        return [lastFull.id, ...incrementals.map(b => b.id)];
    }
    async cleanupOldBackups() {
        const policy = this.config.retentionPolicy;
        const now = new Date();
        const backups = Array.from(this.backups.values()).sort((a, b) => b.createdAt.getTime() - a.createdAt.getTime());
        // Удаление по возрасту
        const maxAge = now.getTime() - (policy.maxAge * 24 * 60 * 60 * 1000);
        const oldBackups = backups.filter(b => b.createdAt.getTime() < maxAge);
        // Удаление по количеству
        const excessBackups = backups.slice(policy.maxCount);
        const toDelete = [...oldBackups, ...excessBackups];
        for (const backup of toDelete) {
            await this.deleteBackup(backup.id);
        }
        if (toDelete.length > 0) {
            console.log(`Удалено ${toDelete.length} старых бэкапов`);
        }
    }
    async ensureDirectories() {
        // Заглушка для создания директорий
        console.log(`Создание директорий: ${this.config.backupPath}, ${this.config.tempPath}`);
    }
    async loadExistingBackups() {
        // Заглушка для загрузки существующих бэкапов
        console.log('Загрузка существующих бэкапов');
    }
    async deleteBackupFile(backup) {
        // Заглушка для удаления файла бэкапа
        console.log(`Удаление файла бэкапа: ${backup.id}`);
    }
    async compress(data, _type) {
        // Заглушка для сжатия
        return Buffer.from(data, 'utf8');
    }
    calculateChecksum(data) {
        // Простая контрольная сумма
        let hash = 0;
        for (let i = 0; i < data.length; i++) {
            hash = ((hash << 5) - hash) + data[i];
            hash = hash & hash; // Convert to 32bit integer
        }
        return hash.toString(16);
    }
    calculateDataSize(backupData) {
        return Buffer.byteLength(JSON.stringify(backupData), 'utf8');
    }
    updateStats() {
        this.stats.totalBackups = this.backups.size;
        if (this.stats.totalSize > 0 && this.stats.compressedSize > 0) {
            this.stats.compressionRatio = this.stats.compressedSize / this.stats.totalSize;
        }
        const backupTimes = Array.from(this.backups.values())
            .filter(b => b.completedAt)
            .map(b => b.completedAt.getTime() - b.createdAt.getTime());
        if (backupTimes.length > 0) {
            this.stats.averageBackupTime = backupTimes.reduce((sum, time) => sum + time, 0) / backupTimes.length;
        }
        const sortedBackups = Array.from(this.backups.values()).sort((a, b) => a.createdAt.getTime() - b.createdAt.getTime());
        if (sortedBackups.length > 0) {
            this.stats.oldestBackup = sortedBackups[0].createdAt;
            this.stats.newestBackup = sortedBackups[sortedBackups.length - 1].createdAt;
            this.stats.lastSuccessfulBackup = sortedBackups
                .filter(b => b.status === BackupStatus.COMPLETED)
                .pop()?.completedAt;
        }
    }
    generateId() {
        return Date.now().toString(36) + Math.random().toString(36).substr(2);
    }
    // Публичные методы для получения данных
    getStats() {
        return { ...this.stats };
    }
    getAllSchedules() {
        return Array.from(this.schedules.values());
    }
    getActiveJobs() {
        return Array.from(this.activeJobs.values());
    }
    getJob(id) {
        return this.activeJobs.get(id);
    }
}
//# sourceMappingURL=BackupPlugin.js.map