export var CacheStrategy;
(function (CacheStrategy) {
    CacheStrategy["LRU"] = "lru";
    CacheStrategy["LFU"] = "lfu";
    CacheStrategy["TTL"] = "ttl";
})(CacheStrategy || (CacheStrategy = {}));
export class CachePlugin {
    constructor(options) {
        this.name = 'CachePlugin';
        this.version = '1.0.0';
        this.description = 'Плагин для кэширования данных';
        this.cache = new Map();
        this.maxSize = 1000;
        this.strategy = CacheStrategy.LRU;
        this.stats = {
            hits: 0,
            misses: 0
        };
        if (options) {
            this.maxSize = options.maxSize || this.maxSize;
            this.strategy = options.strategy || this.strategy;
        }
    }
    install(CSElementClass) {
        console.log(`CachePlugin установлен для ${CSElementClass.name}`);
    }
    set(key, value, ttl) {
        if (this.cache.size >= this.maxSize) {
            this.evict();
        }
        this.cache.set(key, {
            value,
            timestamp: Date.now(),
            accessCount: 0,
            ttl
        });
    }
    get(key) {
        const entry = this.cache.get(key);
        if (!entry) {
            this.stats.misses++;
            return undefined;
        }
        // Проверяем TTL
        if (entry.ttl && Date.now() - entry.timestamp > entry.ttl) {
            this.cache.delete(key);
            this.stats.misses++;
            return undefined;
        }
        // Обновляем статистику доступа
        entry.accessCount++;
        entry.timestamp = Date.now();
        this.stats.hits++;
        return entry.value;
    }
    has(key) {
        const entry = this.cache.get(key);
        if (!entry) {
            return false;
        }
        // Проверяем TTL
        if (entry.ttl && Date.now() - entry.timestamp > entry.ttl) {
            this.cache.delete(key);
            return false;
        }
        return true;
    }
    delete(key) {
        return this.cache.delete(key);
    }
    clear() {
        this.cache.clear();
    }
    size() {
        return this.cache.size;
    }
    keys() {
        return Array.from(this.cache.keys());
    }
    cacheQueryResult(query, elementId, result) {
        const key = `query:${query}:${elementId}`;
        this.set(key, result);
    }
    getStats() {
        const totalRequests = this.stats.hits + this.stats.misses;
        const hitRate = totalRequests > 0 ? (this.stats.hits / totalRequests) * 100 : 0;
        return {
            size: this.cache.size,
            maxSize: this.maxSize,
            strategy: this.strategy,
            hits: this.stats.hits,
            misses: this.stats.misses,
            entries: this.cache.size,
            hitRate: Math.round(hitRate)
        };
    }
    getCachedQueryResult(query, elementId) {
        const key = `query:${query}:${elementId}`;
        return this.get(key);
    }
    cacheSerializationResult(elementId, options, result) {
        const key = `serialization:${elementId}:${JSON.stringify(options)}`;
        this.set(key, result);
    }
    getCachedSerializationResult(elementId, options) {
        const key = `serialization:${elementId}:${JSON.stringify(options)}`;
        return this.get(key);
    }
    resetStats() {
        this.stats.hits = 0;
        this.stats.misses = 0;
    }
    updateConfig(options) {
        if (options.maxSize !== undefined) {
            this.maxSize = options.maxSize;
        }
        if (options.strategy !== undefined) {
            this.strategy = options.strategy;
        }
    }
    getConfig() {
        return {
            maxSize: this.maxSize,
            strategy: this.strategy
        };
    }
    evict() {
        if (this.cache.size === 0)
            return;
        let keyToEvict;
        switch (this.strategy) {
            case CacheStrategy.LRU:
                keyToEvict = this.findLRUKey();
                break;
            case CacheStrategy.LFU:
                keyToEvict = this.findLFUKey();
                break;
            case CacheStrategy.TTL:
                keyToEvict = this.findExpiredKey() || this.findLRUKey();
                break;
            default:
                keyToEvict = this.cache.keys().next().value;
        }
        if (keyToEvict) {
            this.cache.delete(keyToEvict);
        }
    }
    findLRUKey() {
        let oldestKey;
        let oldestTime = Date.now();
        for (const [key, entry] of this.cache) {
            if (entry.timestamp < oldestTime) {
                oldestTime = entry.timestamp;
                oldestKey = key;
            }
        }
        return oldestKey;
    }
    findLFUKey() {
        let leastUsedKey;
        let leastCount = Infinity;
        for (const [key, entry] of this.cache) {
            if (entry.accessCount < leastCount) {
                leastCount = entry.accessCount;
                leastUsedKey = key;
            }
        }
        return leastUsedKey;
    }
    findExpiredKey() {
        const now = Date.now();
        for (const [key, entry] of this.cache) {
            if (entry.ttl && now - entry.timestamp > entry.ttl) {
                return key;
            }
        }
        return null;
    }
}
export function cachePlugin(options) {
    return new CachePlugin(options);
}
//# sourceMappingURL=CachePlugin.js.map