import { DevToolsManager } from '../devtools/DevToolsManager';
/**
 * Плагин DevTools для CSElement
 * Обеспечивает инспектирование и профилирование элементов
 */
export class DevToolsPlugin {
    constructor() {
        this.name = 'devtools';
        this.version = '1.0.0';
        this.description = 'Инструменты разработчика для инспектирования и профилирования CSElement';
        this.isInstalled = false;
        this.CSElementClass = null;
        /**
         * Хуки жизненного цикла
         */
        this.hooks = {
            afterCreate: (element) => {
                // Автоматически регистрируем новые элементы
                this.manager.registerElement(element);
            },
            beforeDestroy: (element) => {
                // Отменяем регистрацию при уничтожении элемента
                this.manager.unregisterElement(element);
            }
        };
        this.manager = new DevToolsManager();
    }
    /**
     * Установка плагина
     */
    install(CSElementClass) {
        if (this.isInstalled) {
            console.warn('🔧 Плагин DevTools уже установлен');
            return;
        }
        // Сохраняем ссылку на CSElementClass
        this.CSElementClass = CSElementClass;
        // Расширяем CSElement методами DevTools
        this.extendCSElement();
        // Регистрируем хуки жизненного цикла если передан CSElementClass
        if (CSElementClass) {
            const pluginManager = CSElementClass.plugins;
            if (this.hooks.afterCreate) {
                pluginManager.addHook('afterCreate', this.hooks.afterCreate);
            }
            if (this.hooks.beforeDestroy) {
                pluginManager.addHook('beforeDestroy', this.hooks.beforeDestroy);
            }
        }
        else {
            console.log('⚠️ CSElementClass не передан, хуки не регистрируются');
        }
        this.isInstalled = true;
        console.log('🔧 Плагин DevTools установлен');
    }
    /**
     * Удаление плагина
     */
    uninstall() {
        if (!this.isInstalled) {
            console.warn('🔧 Плагин DevTools не установлен');
            return;
        }
        // Удаляем методы из прототипа
        this.removeCSElementExtensions();
        // Удаляем хуки жизненного цикла если есть сохраненная ссылка на CSElementClass
        if (this.CSElementClass) {
            const pluginManager = this.CSElementClass.plugins;
            if (this.hooks.afterCreate) {
                pluginManager.removeHook('afterCreate', this.hooks.afterCreate);
            }
            if (this.hooks.beforeDestroy) {
                pluginManager.removeHook('beforeDestroy', this.hooks.beforeDestroy);
            }
        }
        // Очищаем ссылку на CSElementClass
        this.CSElementClass = null;
        this.isInstalled = false;
        console.log('🔧 Плагин DevTools удален');
    }
    /**
     * Получить менеджер DevTools
     */
    getManager() {
        return this.manager;
    }
    /**
     * Расширить CSElement методами DevTools
     */
    extendCSElement() {
        // Используем сохраненную ссылку на CSElementClass
        const CSElementClass = this.CSElementClass;
        if (!CSElementClass)
            return;
        // Метод инспектирования элемента
        CSElementClass.prototype.inspect = function () {
            return devToolsPlugin.manager.inspector.inspect(this);
        };
        // Метод создания снимка структуры
        CSElementClass.prototype.createSnapshot = function () {
            return devToolsPlugin.manager.inspector.createSnapshot(this);
        };
        // Метод сериализации
        CSElementClass.prototype.serialize = function (options) {
            return devToolsPlugin.manager.inspector.serialize(this, options);
        };
        // Метод поиска по пути
        CSElementClass.prototype.findByPath = function (path) {
            return devToolsPlugin.manager.inspector.findByPath(this, path);
        };
        // Метод получения пути элемента
        CSElementClass.prototype.getElementPath = function () {
            return devToolsPlugin.manager.inspector.getElementPath(this);
        };
        // Статические методы
        CSElementClass.getDevToolsManager = function () {
            return devToolsPlugin.manager;
        };
        CSElementClass.runDiagnostics = function () {
            return devToolsPlugin.manager.runDiagnostics();
        };
        CSElementClass.exportDevToolsData = function (options) {
            return devToolsPlugin.manager.exportData(options);
        };
        CSElementClass.importDevToolsData = function (data) {
            return devToolsPlugin.manager.importData(data);
        };
        // Методы профилирования
        CSElementClass.startProfiling = function (operationType, metadata) {
            return devToolsPlugin.manager.profiler.startMeasurement(operationType, metadata);
        };
        CSElementClass.endProfiling = function (measurementId) {
            return devToolsPlugin.manager.profiler.endMeasurement(measurementId);
        };
        CSElementClass.getProfilerReport = function (timeRange) {
            return devToolsPlugin.manager.profiler.getReport(timeRange);
        };
    }
    /**
     * Удалить расширения CSElement
     */
    removeCSElementExtensions() {
        // Используем сохраненную ссылку на CSElementClass
        const CSElementClass = this.CSElementClass;
        if (!CSElementClass)
            return;
        // Удаляем методы экземпляра
        delete CSElementClass.prototype.inspect;
        delete CSElementClass.prototype.createSnapshot;
        delete CSElementClass.prototype.serialize;
        delete CSElementClass.prototype.findByPath;
        delete CSElementClass.prototype.getElementPath;
        // Удаляем статические методы
        delete CSElementClass.getDevToolsManager;
        delete CSElementClass.runDiagnostics;
        delete CSElementClass.exportDevToolsData;
        delete CSElementClass.importDevToolsData;
        delete CSElementClass.startProfiling;
        delete CSElementClass.endProfiling;
        delete CSElementClass.getProfilerReport;
    }
}
// Экспортируем экземпляр плагина
export const devToolsPlugin = new DevToolsPlugin();
//# sourceMappingURL=DevToolsPlugin.js.map