/**
 * Пример плагина для логирования операций CSElement
 */
export class LoggingPlugin {
    constructor(options = {}) {
        this.name = 'logging';
        this.version = '1.0.0';
        this.description = 'Плагин для логирования операций CSElement';
        this.options = {
            logCreate: true,
            logData: true,
            logElements: true,
            logDestroy: true,
            logPerformance: true,
            prefix: '[CSElement]',
            level: 'basic',
            ...options
        };
    }
    install(CSElementClass, options) {
        const mergedOptions = { ...this.options, ...options };
        this.options = mergedOptions;
        // Получаем менеджер плагинов
        const pluginManager = CSElementClass.plugins;
        // Регистрируем хуки жизненного цикла
        if (this.options.logCreate) {
            pluginManager.addHook('afterCreate', this.logElementCreated.bind(this));
        }
        if (this.options.logData) {
            pluginManager.addHook('beforeSetData', this.logBeforeSetData.bind(this));
            pluginManager.addHook('afterSetData', this.logAfterSetData.bind(this));
            pluginManager.addHook('beforeDeleteData', this.logBeforeDeleteData.bind(this));
            pluginManager.addHook('afterDeleteData', this.logAfterDeleteData.bind(this));
        }
        if (this.options.logElements) {
            pluginManager.addHook('beforeAddElement', this.logBeforeAddElement.bind(this));
            pluginManager.addHook('afterAddElement', this.logAfterAddElement.bind(this));
            pluginManager.addHook('beforeRemoveElement', this.logBeforeRemoveElement.bind(this));
            pluginManager.addHook('afterRemoveElement', this.logAfterRemoveElement.bind(this));
        }
        if (this.options.logDestroy) {
            pluginManager.addHook('beforeDestroy', this.logBeforeDestroy.bind(this));
            pluginManager.addHook('afterDestroy', this.logAfterDestroy.bind(this));
        }
        // Регистрируем middleware для измерения производительности
        if (this.options.logPerformance) {
            pluginManager.addMiddleware('setData', this.performanceMiddleware.bind(this));
            pluginManager.addMiddleware('deleteData', this.performanceMiddleware.bind(this));
            pluginManager.addMiddleware('addElement', this.performanceMiddleware.bind(this));
            pluginManager.addMiddleware('removeElement', this.performanceMiddleware.bind(this));
        }
        this.log('Плагин логирования установлен', 'info');
    }
    uninstall() {
        this.log('Плагин логирования удален', 'info');
    }
    // Хуки жизненного цикла
    logElementCreated(element) {
        this.log(`Создан элемент: ${element.name} (ID: ${element.id})`, 'create');
    }
    logBeforeSetData(element, key, value) {
        if (this.options.level === 'verbose') {
            this.log(`[${element.name}] Установка данных: ${key} = ${JSON.stringify(value)}`, 'data');
        }
    }
    logAfterSetData(element, key, _value) {
        if (this.options.level !== 'basic') {
            this.log(`[${element.name}] Данные установлены: ${key}`, 'data');
        }
    }
    logBeforeDeleteData(element, key) {
        if (this.options.level === 'verbose') {
            this.log(`[${element.name}] Удаление данных: ${key}`, 'data');
        }
    }
    logAfterDeleteData(element, key) {
        if (this.options.level !== 'basic') {
            this.log(`[${element.name}] Данные удалены: ${key}`, 'data');
        }
    }
    logBeforeAddElement(parent, child) {
        this.log(`[${parent.name}] Добавление дочернего элемента: ${child.name}`, 'element');
    }
    logAfterAddElement(parent, child) {
        if (this.options.level !== 'basic') {
            this.log(`[${parent.name}] Дочерний элемент добавлен: ${child.name}`, 'element');
        }
    }
    logBeforeRemoveElement(parent, child) {
        this.log(`[${parent.name}] Удаление дочернего элемента: ${child.name}`, 'element');
    }
    logAfterRemoveElement(parent, child) {
        if (this.options.level !== 'basic') {
            this.log(`[${parent.name}] Дочерний элемент удален: ${child.name}`, 'element');
        }
    }
    logBeforeDestroy(element) {
        this.log(`Уничтожение элемента: ${element.name} (ID: ${element.id})`, 'destroy');
    }
    logAfterDestroy(element) {
        if (this.options.level === 'verbose') {
            this.log(`Элемент уничтожен: ${element.name}`, 'destroy');
        }
    }
    // Middleware для производительности
    async performanceMiddleware(context, next) {
        const startTime = Date.now();
        try {
            const result = await next();
            const duration = Date.now() - startTime;
            if (this.options.level === 'verbose' || duration > 100) {
                this.log(`[${context.element.name}] ${context.operation} выполнен за ${duration}мс`, 'performance');
            }
            return result;
        }
        catch (error) {
            const duration = Date.now() - startTime;
            this.log(`[${context.element.name}] ${context.operation} завершился с ошибкой за ${duration}мс: ${error}`, 'error');
            throw error;
        }
    }
    // Утилиты логирования
    log(message, type) {
        const timestamp = new Date().toISOString();
        const formattedMessage = `${this.options.prefix} [${type.toUpperCase()}] ${timestamp} - ${message}`;
        switch (type) {
            case 'error':
                console.error(formattedMessage);
                break;
            case 'performance':
                console.log(formattedMessage);
                break;
            default:
                console.log(formattedMessage);
        }
    }
    // Публичные методы для управления логированием
    setLevel(level) {
        this.options.level = level;
        this.log(`Уровень логирования изменен на: ${level}`, 'info');
    }
    setPrefix(prefix) {
        this.options.prefix = prefix;
        this.log(`Префикс логирования изменен на: ${prefix}`, 'info');
    }
    getStats() {
        const enabledFeatures = [];
        if (this.options.logCreate)
            enabledFeatures.push('create');
        if (this.options.logData)
            enabledFeatures.push('data');
        if (this.options.logElements)
            enabledFeatures.push('elements');
        if (this.options.logDestroy)
            enabledFeatures.push('destroy');
        if (this.options.logPerformance)
            enabledFeatures.push('performance');
        return {
            level: this.options.level,
            prefix: this.options.prefix,
            enabledFeatures
        };
    }
}
//# sourceMappingURL=LoggingPlugin.js.map