/**
 * Плагин сериализации для CSElement
 * Поддерживает различные форматы сериализации
 */
/**
 * Поддерживаемые форматы сериализации
 */
export var SerializationFormat;
(function (SerializationFormat) {
    SerializationFormat["JSON"] = "json";
    SerializationFormat["XML"] = "xml";
    SerializationFormat["YAML"] = "yaml";
    SerializationFormat["CSV"] = "csv";
    SerializationFormat["BINARY"] = "binary";
    SerializationFormat["CUSTOM"] = "custom";
})(SerializationFormat || (SerializationFormat = {}));
/**
 * Плагин сериализации
 */
export class SerializationPlugin {
    constructor(options) {
        this.name = 'SerializationPlugin';
        this.version = '1.0.0';
        this.description = 'Плагин для сериализации данных в разные форматы';
        this.config = {
            enabled: true,
            defaultFormat: SerializationFormat.JSON,
            enabledFormats: [
                SerializationFormat.JSON,
                SerializationFormat.XML,
                SerializationFormat.YAML,
                SerializationFormat.CSV
            ],
            enableCache: true,
            enableCompression: false,
            enableEncryption: false,
            maxCacheSize: 1000,
            compressionLevel: 6
        };
        this.serializers = new Map();
        this.cache = new Map();
        if (options?.config) {
            this.config = { ...this.config, ...options.config };
        }
        this.initializeSerializers();
    }
    install(CSElementClass) {
        console.log(`SerializationPlugin установлен для ${CSElementClass.name}`);
    }
    getLifecycleHooks() {
        return {};
    }
    /**
     * Сериализация элемента
     */
    async serialize(element, format) {
        if (!this.config.enabled) {
            return {
                success: false,
                data: '',
                format,
                size: 0,
                error: 'Сериализация отключена'
            };
        }
        const fullOptions = {
            format,
            includeMetadata: true,
            includeChildren: true,
            includeParent: false,
            pretty: false,
            compression: this.config.enableCompression,
            encryption: this.config.enableEncryption,
        };
        // Проверяем кэш
        const cacheKey = this.generateCacheKey(element, fullOptions);
        if (this.config.enableCache && this.cache.has(cacheKey)) {
            return this.cache.get(cacheKey);
        }
        try {
            const serializer = this.serializers.get(format);
            if (!serializer) {
                return {
                    success: false,
                    data: '',
                    format,
                    size: 0,
                    error: `Сериализатор для формата ${format} не найден`
                };
            }
            let data = await serializer.serialize(element, fullOptions);
            // Применяем сжатие если включено
            if (fullOptions.compression) {
                data = await this.compress(data);
            }
            // Применяем шифрование если включено
            if (fullOptions.encryption && this.config.encryptionKey) {
                data = await this.encrypt(data);
            }
            const result = {
                success: true,
                data,
                format,
                size: typeof data === 'string' ? data.length : data.byteLength,
                metadata: {
                    timestamp: Date.now(),
                    compressed: fullOptions.compression,
                    encrypted: fullOptions.encryption,
                    options: fullOptions
                }
            };
            // Кэшируем результат
            if (this.config.enableCache) {
                this.cacheResult(cacheKey, result);
            }
            return result;
        }
        catch (error) {
            return {
                success: false,
                data: '',
                format,
                size: 0,
                error: error instanceof Error ? error.message : String(error)
            };
        }
    }
    /**
     * Десериализация данных
     */
    async deserialize(data, format) {
        if (!this.config.enabled) {
            return {
                success: false,
                error: 'Сериализация отключена'
            };
        }
        const fullOptions = {
            format,
            includeMetadata: true,
            includeChildren: true,
            includeParent: false,
            compression: this.config.enableCompression,
            encryption: this.config.enableEncryption,
        };
        try {
            let processedData = data;
            // Расшифровываем если нужно
            if (fullOptions.encryption && this.config.encryptionKey) {
                const decrypted = await this.decrypt(processedData);
                processedData = typeof decrypted === 'string' ? decrypted : decrypted.toString();
            }
            // Распаковываем если нужно
            if (fullOptions.compression) {
                const decompressed = await this.decompress(processedData);
                processedData = typeof decompressed === 'string' ? decompressed : decompressed.toString();
            }
            const serializer = this.serializers.get(format);
            if (!serializer) {
                return {
                    success: false,
                    error: `Сериализатор для формата ${format} не найден`
                };
            }
            const element = await serializer.deserialize(processedData, fullOptions);
            return {
                success: true,
                element,
                metadata: {
                    timestamp: Date.now(),
                    format,
                    options: fullOptions
                }
            };
        }
        catch (error) {
            return {
                success: false,
                error: error instanceof Error ? error.message : String(error)
            };
        }
    }
    /**
     * Регистрация пользовательского сериализатора
     */
    registerSerializer(serializer) {
        this.serializers.set(serializer.format, serializer);
        if (!this.config.enabledFormats.includes(serializer.format)) {
            this.config.enabledFormats.push(serializer.format);
        }
    }
    /**
     * Удаление сериализатора
     */
    unregisterSerializer(format) {
        return this.serializers.delete(format);
    }
    /**
     * Инициализация встроенных сериализаторов
     */
    initializeSerializers() {
        // JSON сериализатор
        this.serializers.set(SerializationFormat.JSON, {
            format: SerializationFormat.JSON,
            async serialize(element, options) {
                const data = this.elementToObject(element, options);
                return options.pretty ? JSON.stringify(data, null, 2) : JSON.stringify(data);
            },
            async deserialize(data) {
                return JSON.parse(data.toString());
            },
            validate(data) {
                try {
                    JSON.parse(data.toString());
                    return true;
                }
                catch {
                    return false;
                }
            },
            elementToObject(element, options) {
                const result = {
                    id: element.id,
                    name: element.name,
                    data: Array.from(element.data.entries()).reduce((acc, [key, value]) => {
                        acc[key] = value;
                        return acc;
                    }, {})
                };
                if (options.includeMetadata) {
                    result.metadata = {
                        index: element.index,
                        elementsCount: element.elementsCount(),
                        ownersCount: element.ownersCount()
                    };
                }
                if (options.includeChildren) {
                    result.children = element.getAllElements().map((child) => this.elementToObject(child, options));
                }
                return result;
            }
        });
        // XML сериализатор
        this.serializers.set(SerializationFormat.XML, {
            format: SerializationFormat.XML,
            async serialize(element, options) {
                const data = this.elementToXml(element, options);
                return options.pretty ? this.formatXml(data) : data;
            },
            async deserialize(data) {
                return this.parseXml(data.toString());
            },
            elementToXml(element, options) {
                let xml = `<element id="${element.id}" name="${element.name}">`;
                // Данные элемента
                for (const [key, value] of element.data) {
                    xml += `<data key="${key}">${this.escapeXml(String(value))}</data>`;
                }
                // Дочерние элементы
                if (options.includeChildren) {
                    element.getAllElements().forEach((child) => {
                        xml += this.elementToXml(child, options);
                    });
                }
                xml += '</element>';
                return xml;
            },
            formatXml(xml) {
                // Простое форматирование XML
                return xml.replace(/></g, '>\n<');
            },
            parseXml(xml) {
                // Простой парсер XML (заглушка)
                const result = {};
                const elementMatch = xml.match(/<element[^>]*>/);
                if (elementMatch) {
                    const idMatch = elementMatch[0].match(/id="([^"]*)"/);
                    const nameMatch = elementMatch[0].match(/name="([^"]*)"/);
                    if (idMatch)
                        result.id = idMatch[1];
                    if (nameMatch)
                        result.name = nameMatch[1];
                }
                return result;
            },
            escapeXml(str) {
                return str.replace(/[<>&'"]/g, (c) => {
                    switch (c) {
                        case '<': return '&lt;';
                        case '>': return '&gt;';
                        case '&': return '&amp;';
                        case "'": return '&apos;';
                        case '"': return '&quot;';
                        default: return c;
                    }
                });
            }
        });
        // YAML сериализатор
        this.serializers.set(SerializationFormat.YAML, {
            format: SerializationFormat.YAML,
            async serialize(element, options) {
                const data = this.elementToObject(element, options);
                return this.objectToYaml(data, options.pretty ? 2 : 0);
            },
            async deserialize(data) {
                return this.parseYaml(data.toString());
            },
            elementToObject(element, options) {
                const result = {
                    id: element.id,
                    name: element.name,
                    data: Array.from(element.data.entries()).reduce((acc, [key, value]) => {
                        acc[key] = value;
                        return acc;
                    }, {})
                };
                if (options.includeMetadata) {
                    result.metadata = {
                        index: element.index,
                        elementsCount: element.elementsCount(),
                        ownersCount: element.ownersCount()
                    };
                }
                if (options.includeChildren) {
                    result.children = element.getAllElements().map((child) => this.elementToObject(child, options));
                }
                return result;
            },
            objectToYaml(obj, indent = 0) {
                const spaces = ' '.repeat(indent);
                let yaml = '';
                for (const [key, value] of Object.entries(obj)) {
                    yaml += `${spaces}${key}:`;
                    if (typeof value === 'object' && value !== null) {
                        yaml += '\n';
                        yaml += this.objectToYaml(value, indent + 2);
                    }
                    else {
                        yaml += ` ${value}\n`;
                    }
                }
                return yaml;
            },
            parseYaml(yaml) {
                // Простой парсер YAML (заглушка)
                const result = {};
                const lines = yaml.split('\n');
                for (const line of lines) {
                    const match = line.match(/^(\s*)([^:]+):\s*(.+)$/);
                    if (match) {
                        const [, , key, value] = match;
                        result[key] = value;
                    }
                }
                return result;
            }
        });
        // CSV сериализатор
        this.serializers.set(SerializationFormat.CSV, {
            format: SerializationFormat.CSV,
            async serialize(element, _options) {
                return this.elementToCsv(element);
            },
            async deserialize(data) {
                return this.parseCsv(data.toString());
            },
            elementToCsv(element) {
                const dataObj = Array.from(element.data.entries()).reduce((acc, [key, value]) => {
                    acc[key] = value;
                    return acc;
                }, {});
                const headers = ['id', 'name', 'data'];
                const values = [element.id, element.name, JSON.stringify(dataObj)];
                return `${headers.join(',')}\n${values.join(',')}`;
            },
            parseCsv(csv) {
                const lines = csv.split('\n');
                if (lines.length < 2)
                    return {};
                const headers = lines[0].split(',');
                const values = lines[1].split(',');
                const result = {};
                headers.forEach((header, index) => {
                    result[header] = values[index];
                });
                return result;
            }
        });
    }
    /**
     * Генерация ключа кэша
     */
    generateCacheKey(element, options) {
        const optionsString = JSON.stringify(options);
        return `${element.id}_${this.hashString(optionsString)}`;
    }
    /**
     * Простое хэширование строки
     */
    hashString(str) {
        let hash = 0;
        for (let i = 0; i < str.length; i++) {
            const char = str.charCodeAt(i);
            hash = ((hash << 5) - hash) + char;
            hash = hash & hash;
        }
        return hash.toString();
    }
    /**
     * Кэширование результата
     */
    cacheResult(key, result) {
        // Проверяем лимит кэша
        if (this.cache.size >= this.config.maxCacheSize) {
            // Удаляем самую старую запись
            const firstKey = this.cache.keys().next().value;
            if (firstKey) {
                this.cache.delete(firstKey);
            }
        }
        this.cache.set(key, result);
    }
    /**
     * Сжатие данных (заглушка)
     */
    async compress(data) {
        // В реальной реализации здесь должно быть сжатие (например, gzip)
        const input = typeof data === 'string' ? Buffer.from(data) : data;
        return input;
    }
    /**
     * Распаковка данных (заглушка)
     */
    async decompress(data) {
        // В реальной реализации здесь должна быть распаковка
        return data;
    }
    /**
     * Шифрование данных (заглушка)
     */
    async encrypt(data) {
        // В реальной реализации здесь должно быть шифрование
        const input = typeof data === 'string' ? Buffer.from(data) : data;
        return input;
    }
    /**
     * Расшифровка данных (заглушка)
     */
    async decrypt(data) {
        // В реальной реализации здесь должна быть расшифровка
        return data;
    }
    /**
     * Получение поддерживаемых форматов
     */
    getSupportedFormats() {
        return Array.from(this.serializers.keys());
    }
    /**
     * Проверка поддержки формата
     */
    supportsFormat(format) {
        return this.serializers.has(format);
    }
    /**
     * Валидация данных для формата
     */
    validate(data, format) {
        const serializer = this.serializers.get(format);
        if (!serializer || !serializer.validate) {
            return false;
        }
        return serializer.validate(data);
    }
    /**
     * Очистка кэша
     */
    clearCache() {
        this.cache.clear();
    }
    /**
     * Получение статистики кэша
     */
    getCacheStats() {
        return {
            size: this.cache.size,
            maxSize: this.config.maxCacheSize
        };
    }
    /**
     * Обновление конфигурации
     */
    updateConfig(config) {
        this.config = { ...this.config, ...config };
        if (!this.config.enableCache) {
            this.clearCache();
        }
    }
    /**
     * Получение конфигурации
     */
    getConfig() {
        return { ...this.config };
    }
}
/**
 * Фабричная функция для создания плагина сериализации
 */
export function serializationPlugin(config) {
    return new SerializationPlugin({ config });
}
//# sourceMappingURL=SerializationPlugin.js.map