/**
 * Плагин трансформации данных для CSElement
 * Обеспечивает преобразование данных при операциях
 */
/**
 * Типы трансформации
 */
export var TransformType;
(function (TransformType) {
    TransformType["INPUT"] = "input";
    TransformType["OUTPUT"] = "output";
    TransformType["STORAGE"] = "storage";
    TransformType["DISPLAY"] = "display";
    TransformType["JSON_STRINGIFY"] = "json_stringify";
    TransformType["JSON_PARSE"] = "json_parse";
    TransformType["BASE64_ENCODE"] = "base64_encode";
    TransformType["BASE64_DECODE"] = "base64_decode";
    TransformType["UPPERCASE"] = "uppercase";
    TransformType["LOWERCASE"] = "lowercase";
    TransformType["CUSTOM"] = "custom";
})(TransformType || (TransformType = {}));
/**
 * Направление трансформации
 */
export var TransformDirection;
(function (TransformDirection) {
    TransformDirection["ENCODE"] = "encode";
    TransformDirection["DECODE"] = "decode";
    TransformDirection["BOTH"] = "both";
})(TransformDirection || (TransformDirection = {}));
/**
 * Плагин трансформации
 */
export class TransformPlugin {
    constructor(options) {
        this.name = 'TransformPlugin';
        this.version = '1.0.0';
        this.description = 'Плагин для трансформации данных';
        this.config = {
            enabled: true,
            enabledTypes: [TransformType.INPUT, TransformType.OUTPUT, TransformType.STORAGE],
            rules: [],
            maxTransformTime: 5000, // 5 секунд
            enableCache: true,
            enableStats: true
        };
        this.transformCache = new Map();
        this.stats = {
            totalTransforms: 0,
            successfulTransforms: 0,
            failedTransforms: 0,
            averageTime: 0,
            ruleStats: new Map()
        };
        if (options?.config) {
            this.config = { ...this.config, ...options.config };
        }
        // Добавляем встроенные трансформации
        this.addBuiltInTransforms();
    }
    install(CSElementClass) {
        console.log(`TransformPlugin установлен для ${CSElementClass.name}`);
    }
    getLifecycleHooks() {
        return {
            beforeSetData: async (element, key, value) => {
                if (this.config.enabled && this.config.enabledTypes.includes(TransformType.INPUT)) {
                    const context = {
                        element,
                        key,
                        type: TransformType.INPUT,
                        direction: TransformDirection.ENCODE,
                        metadata: {},
                        originalValue: value
                    };
                    const result = await this.applyTransforms(value, context);
                    if (result.success && result.value !== value) {
                        // Заменяем значение на трансформированное
                        // Это требует модификации оригинального значения
                        Object.assign(arguments[2], result.value);
                    }
                }
            }
        };
    }
    /**
     * Применение трансформаций
     */
    async applyTransforms(value, context) {
        if (!this.config.enabled || !this.config.enabledTypes.includes(context.type)) {
            return {
                success: true,
                value,
                appliedRules: []
            };
        }
        const startTime = Date.now();
        let currentValue = value;
        const appliedRules = [];
        let error;
        try {
            // Получаем применимые правила
            const applicableRules = this.getApplicableRules(context);
            // Применяем правила по приоритету
            for (const rule of applicableRules) {
                if (!rule.enabled) {
                    continue;
                }
                try {
                    const ruleStartTime = Date.now();
                    // Проверяем условие
                    if (rule.condition && !rule.condition(context)) {
                        continue;
                    }
                    // Применяем трансформацию с таймаутом
                    const transformedValue = await this.applyTransformWithTimeout(rule.transform, currentValue, context);
                    currentValue = transformedValue;
                    appliedRules.push(rule.name);
                    // Обновляем статистику правила
                    if (this.config.enableStats) {
                        this.updateRuleStats(rule.name, Date.now() - ruleStartTime, true);
                    }
                }
                catch (ruleError) {
                    if (this.config.enableStats) {
                        this.updateRuleStats(rule.name, Date.now() - startTime, false);
                    }
                    error = `Ошибка в правиле ${rule.name}: ${ruleError instanceof Error ? ruleError.message : String(ruleError)}`;
                    break;
                }
            }
            // Обновляем общую статистику
            if (this.config.enableStats) {
                this.updateStats(Date.now() - startTime, !error);
            }
            return {
                success: !error,
                value: currentValue,
                error,
                appliedRules
            };
        }
        catch (transformError) {
            if (this.config.enableStats) {
                this.updateStats(Date.now() - startTime, false);
            }
            return {
                success: false,
                value,
                error: transformError instanceof Error ? transformError.message : String(transformError),
                appliedRules
            };
        }
    }
    /**
     * Получение применимых правил
     */
    getApplicableRules(context) {
        return this.config.rules
            .filter(rule => rule.type === context.type &&
            rule.direction === context.direction &&
            this.matchesKeyPattern(context.key, rule.keyPattern) &&
            this.matchesValueType(context.originalValue, rule.valueType))
            .sort((a, b) => (b.priority || 0) - (a.priority || 0));
    }
    /**
     * Проверка соответствия паттерну ключа
     */
    matchesKeyPattern(key, pattern) {
        if (!pattern) {
            return true;
        }
        if (typeof pattern === 'string') {
            return pattern === '*' || key === pattern || key.includes(pattern);
        }
        if (pattern instanceof RegExp) {
            return pattern.test(key);
        }
        return false;
    }
    /**
     * Проверка соответствия типу значения
     */
    matchesValueType(value, type) {
        if (!type) {
            return true;
        }
        const actualType = typeof value;
        return actualType === type || type === '*';
    }
    /**
     * Применение трансформации с таймаутом
     */
    async applyTransformWithTimeout(transform, value, context) {
        return new Promise((resolve, reject) => {
            const timeout = setTimeout(() => {
                reject(new Error(`Трансформация превысила лимит времени ${this.config.maxTransformTime}мс`));
            }, this.config.maxTransformTime);
            Promise.resolve(transform(value, context))
                .then(result => {
                clearTimeout(timeout);
                resolve(result);
            })
                .catch(error => {
                clearTimeout(timeout);
                reject(error);
            });
        });
    }
    /**
     * Добавление правила трансформации
     */
    addRule(rule) {
        // Проверяем уникальность имени
        const existingIndex = this.config.rules.findIndex(r => r.name === rule.name);
        if (existingIndex >= 0) {
            this.config.rules[existingIndex] = rule;
        }
        else {
            this.config.rules.push(rule);
        }
    }
    /**
     * Удаление правила трансформации
     */
    removeRule(name) {
        const index = this.config.rules.findIndex(r => r.name === name);
        if (index >= 0) {
            this.config.rules.splice(index, 1);
            return true;
        }
        return false;
    }
    /**
     * Получение правила по имени
     */
    getRule(name) {
        return this.config.rules.find(r => r.name === name);
    }
    /**
     * Получение всех правил
     */
    getRules() {
        return [...this.config.rules];
    }
    /**
     * Включение/отключение правила
     */
    toggleRule(name, enabled) {
        const rule = this.config.rules.find(r => r.name === name);
        if (rule) {
            rule.enabled = enabled;
            return true;
        }
        return false;
    }
    /**
     * Трансформация значения напрямую
     */
    async transform(value, type, direction, element, key = 'direct') {
        const context = {
            element,
            key,
            type,
            direction,
            metadata: {},
            originalValue: value
        };
        return this.applyTransforms(value, context);
    }
    /**
     * Добавление встроенных трансформаций
     */
    addBuiltInTransforms() {
        // Трансформация строк
        this.addRule({
            name: 'string_trim',
            type: TransformType.INPUT,
            direction: TransformDirection.ENCODE,
            valueType: 'string',
            transform: (value) => value.trim(),
            priority: 100,
            enabled: true
        });
        this.addRule({
            name: 'string_lowercase',
            type: TransformType.INPUT,
            direction: TransformDirection.ENCODE,
            keyPattern: /.*_lower$/,
            valueType: 'string',
            transform: (value) => value.toLowerCase(),
            priority: 90,
            enabled: true
        });
        this.addRule({
            name: 'string_uppercase',
            type: TransformType.INPUT,
            direction: TransformDirection.ENCODE,
            keyPattern: /.*_upper$/,
            valueType: 'string',
            transform: (value) => value.toUpperCase(),
            priority: 90,
            enabled: true
        });
        // Трансформация чисел
        this.addRule({
            name: 'number_round',
            type: TransformType.INPUT,
            direction: TransformDirection.ENCODE,
            keyPattern: /.*_round$/,
            valueType: 'number',
            transform: (value) => Math.round(value),
            priority: 80,
            enabled: true
        });
        this.addRule({
            name: 'number_abs',
            type: TransformType.INPUT,
            direction: TransformDirection.ENCODE,
            keyPattern: /.*_abs$/,
            valueType: 'number',
            transform: (value) => Math.abs(value),
            priority: 80,
            enabled: true
        });
        // Трансформация дат
        this.addRule({
            name: 'date_iso_string',
            type: TransformType.OUTPUT,
            direction: TransformDirection.ENCODE,
            keyPattern: /.*_date$/,
            condition: (context) => context.originalValue instanceof Date,
            transform: (value) => value.toISOString(),
            priority: 70,
            enabled: true
        });
        this.addRule({
            name: 'date_parse',
            type: TransformType.INPUT,
            direction: TransformDirection.DECODE,
            keyPattern: /.*_date$/,
            valueType: 'string',
            transform: (value) => new Date(value),
            priority: 70,
            enabled: true
        });
        // JSON трансформации
        this.addRule({
            name: 'json_stringify',
            type: TransformType.STORAGE,
            direction: TransformDirection.ENCODE,
            keyPattern: /.*_json$/,
            condition: (context) => typeof context.originalValue === 'object',
            transform: (value) => JSON.stringify(value),
            priority: 60,
            enabled: true
        });
        this.addRule({
            name: 'json_parse',
            type: TransformType.STORAGE,
            direction: TransformDirection.DECODE,
            keyPattern: /.*_json$/,
            valueType: 'string',
            transform: (value) => {
                try {
                    return JSON.parse(value);
                }
                catch {
                    return value; // Возвращаем исходное значение если не удалось распарсить
                }
            },
            priority: 60,
            enabled: true
        });
        // Sanitization
        this.addRule({
            name: 'html_escape',
            type: TransformType.INPUT,
            direction: TransformDirection.ENCODE,
            keyPattern: /.*_html$/,
            valueType: 'string',
            transform: (value) => {
                return value
                    .replace(/&/g, '&amp;')
                    .replace(/</g, '&lt;')
                    .replace(/>/g, '&gt;')
                    .replace(/"/g, '&quot;')
                    .replace(/'/g, '&#39;');
            },
            priority: 50,
            enabled: true
        });
        // URL encoding
        this.addRule({
            name: 'url_encode',
            type: TransformType.OUTPUT,
            direction: TransformDirection.ENCODE,
            keyPattern: /.*_url$/,
            valueType: 'string',
            transform: (value) => encodeURIComponent(value),
            priority: 40,
            enabled: true
        });
        this.addRule({
            name: 'url_decode',
            type: TransformType.INPUT,
            direction: TransformDirection.DECODE,
            keyPattern: /.*_url$/,
            valueType: 'string',
            transform: (value) => decodeURIComponent(value),
            priority: 40,
            enabled: true
        });
    }
    /**
     * Обновление статистики
     */
    updateStats(duration, success) {
        this.stats.totalTransforms++;
        if (success) {
            this.stats.successfulTransforms++;
        }
        else {
            this.stats.failedTransforms++;
        }
        // Обновляем среднее время
        const totalTime = this.stats.averageTime * (this.stats.totalTransforms - 1) + duration;
        this.stats.averageTime = totalTime / this.stats.totalTransforms;
    }
    /**
     * Обновление статистики правила
     */
    updateRuleStats(ruleName, duration, success) {
        let ruleStats = this.stats.ruleStats.get(ruleName);
        if (!ruleStats) {
            ruleStats = {
                executions: 0,
                successes: 0,
                failures: 0,
                totalTime: 0,
                averageTime: 0
            };
            this.stats.ruleStats.set(ruleName, ruleStats);
        }
        ruleStats.executions++;
        ruleStats.totalTime += duration;
        ruleStats.averageTime = ruleStats.totalTime / ruleStats.executions;
        if (success) {
            ruleStats.successes++;
        }
        else {
            ruleStats.failures++;
        }
    }
    /**
     * Получение статистики
     */
    getStats() {
        return {
            ...this.stats,
            ruleStats: new Map(this.stats.ruleStats)
        };
    }
    /**
     * Сброс статистики
     */
    resetStats() {
        this.stats = {
            totalTransforms: 0,
            successfulTransforms: 0,
            failedTransforms: 0,
            averageTime: 0,
            ruleStats: new Map()
        };
    }
    /**
     * Очистка кэша
     */
    clearCache() {
        this.transformCache.clear();
    }
    /**
     * Обновление конфигурации
     */
    updateConfig(config) {
        this.config = { ...this.config, ...config };
        if (!this.config.enableCache) {
            this.clearCache();
        }
    }
    /**
     * Получение конфигурации
     */
    getConfig() {
        return { ...this.config };
    }
}
/**
 * Фабричная функция для создания плагина трансформации
 */
export function transformPlugin(config) {
    return new TransformPlugin({ config });
}
//# sourceMappingURL=TransformPlugin.js.map