/**
 * Плагин визуализации для CSElement
 */
import { VisualizationManager } from '../visualization/VisualizationManager';
import { AsciiEngine } from '../visualization/engines/AsciiEngine';
import { SvgEngine } from '../visualization/engines/SvgEngine';
import { HtmlEngine } from '../visualization/engines/HtmlEngine';
export class VisualizationPlugin {
    constructor() {
        this.name = 'visualization';
        this.version = '1.0.0';
        this.description = 'Плагин для визуализации CSElement структур в различных форматах';
        this.dependencies = [];
        this.isInstalled = false;
        this.CSElementClass = null;
        /**
         * Хуки жизненного цикла
         */
        this.hooks = {
            afterCreate: (element) => {
                // Добавляем метаданные для визуализации
                if (!element.data.has('_visualization')) {
                    // Используем прямой доступ к приватному полю _data
                    element._data.set('_visualization', {
                        created: Date.now(),
                        rendered: false
                    });
                }
            },
            afterAddElement: (element, _child) => {
                // Обновляем метаданные при добавлении дочерних элементов
                const vizData = element.data.get('_visualization') || {};
                vizData.lastModified = Date.now();
                vizData.rendered = false; // Требуется перерендеринг
                // Используем прямой доступ к приватному полю _data
                element._data.set('_visualization', vizData);
            },
            afterRemoveElement: (element, _child) => {
                // Обновляем метаданные при удалении дочерних элементов
                const vizData = element.data.get('_visualization') || {};
                vizData.lastModified = Date.now();
                vizData.rendered = false; // Требуется перерендеринг
                // Используем прямой доступ к приватному полю _data
                element._data.set('_visualization', vizData);
            }
        };
        this.manager = new VisualizationManager();
    }
    /**
     * Установка плагина
     */
    install(CSElementClass) {
        if (this.isInstalled) {
            console.warn('🎨 Плагин визуализации уже установлен');
            return;
        }
        // Сохраняем ссылку на CSElementClass
        this.CSElementClass = CSElementClass;
        // Регистрируем движки визуализации
        this.registerEngines();
        // Добавляем методы к прототипу CSElement
        this.extendCSElement();
        // Настраиваем обработчики событий
        this.setupEventHandlers();
        // Регистрируем хуки жизненного цикла если передан CSElementClass
        if (CSElementClass) {
            const pluginManager = CSElementClass.plugins;
            if (this.hooks.afterCreate) {
                pluginManager.addHook('afterCreate', this.hooks.afterCreate);
            }
            if (this.hooks.afterAddElement) {
                pluginManager.addHook('afterAddElement', this.hooks.afterAddElement);
            }
            if (this.hooks.afterRemoveElement) {
                pluginManager.addHook('afterRemoveElement', this.hooks.afterRemoveElement);
            }
        }
        else {
            console.log('⚠️ CSElementClass не передан, хуки не регистрируются');
        }
        this.isInstalled = true;
        // console.log('🎨 Плагин визуализации установлен');
        // console.log(`📊 Поддерживаемые форматы: ${this.getSupportedFormats().join(', ')}`);
    }
    /**
     * Удаление плагина
     */
    uninstall() {
        if (!this.isInstalled) {
            return;
        }
        // Удаляем методы из прототипа
        this.removeCSElementExtensions();
        // Удаляем хуки жизненного цикла если есть сохраненная ссылка на CSElementClass
        if (this.CSElementClass) {
            const pluginManager = this.CSElementClass.plugins;
            if (this.hooks.afterCreate) {
                pluginManager.removeHook('afterCreate', this.hooks.afterCreate);
            }
            if (this.hooks.afterAddElement) {
                pluginManager.removeHook('afterAddElement', this.hooks.afterAddElement);
            }
            if (this.hooks.afterRemoveElement) {
                pluginManager.removeHook('afterRemoveElement', this.hooks.afterRemoveElement);
            }
        }
        // Очищаем менеджер
        this.manager.clear();
        // Очищаем ссылку на CSElementClass
        this.CSElementClass = null;
        this.isInstalled = false;
        // console.log('🎨 Плагин визуализации удален');
    }
    /**
     * Регистрация движков визуализации
     */
    registerEngines() {
        this.manager.registerEngine(new AsciiEngine());
        this.manager.registerEngine(new SvgEngine());
        this.manager.registerEngine(new HtmlEngine());
    }
    /**
     * Расширение CSElement новыми методами
     */
    extendCSElement() {
        // Используем сохраненную ссылку на CSElementClass
        const CSElementClass = this.CSElementClass;
        // Метод визуализации
        CSElementClass.prototype.visualize = async function (options = {}) {
            return await visualizationPlugin.manager.visualize(this, options);
        };
        // Метод быстрой ASCII визуализации
        CSElementClass.prototype.toAscii = async function (options = {}) {
            const result = await this.visualize({ ...options, format: 'ascii' });
            return result.content;
        };
        // Метод экспорта в SVG
        CSElementClass.prototype.toSvg = async function (options = {}) {
            const result = await this.visualize({ ...options, format: 'svg' });
            return result.content;
        };
        // Метод создания HTML визуализации
        CSElementClass.prototype.toHtml = async function (options = {}) {
            return await this.visualize({ ...options, format: 'html' });
        };
        // Метод для вывода в консоль
        CSElementClass.prototype.print = async function (options = {}) {
            const ascii = await this.toAscii(options);
            console.log(ascii);
        };
        // Статический метод для получения менеджера визуализации
        CSElementClass.getVisualizationManager = () => {
            return visualizationPlugin.manager;
        };
        // Статический метод для добавления пользовательских тем
        CSElementClass.addVisualizationTheme = (theme) => {
            visualizationPlugin.manager.addTheme(theme);
        };
    }
    /**
     * Удаление расширений CSElement
     */
    removeCSElementExtensions() {
        // Используем сохраненную ссылку на CSElementClass
        const CSElementClass = this.CSElementClass;
        delete CSElementClass.prototype.visualize;
        delete CSElementClass.prototype.toAscii;
        delete CSElementClass.prototype.toSvg;
        delete CSElementClass.prototype.toHtml;
        delete CSElementClass.prototype.print;
        delete CSElementClass.getVisualizationManager;
        delete CSElementClass.addVisualizationTheme;
    }
    /**
     * Настройка обработчиков событий
     */
    setupEventHandlers() {
        // Логирование событий визуализации
        this.manager.on('render:start', (_event) => {
            //console.log(`🎨 Начало рендеринга: ${event.options.format || 'ascii'}`);
        });
        this.manager.on('render:complete', (_event) => {
            //const { nodeCount, edgeCount, generationTime } = event.result!.metadata;
            //console.log(`✅ Рендеринг завершен: ${nodeCount} узлов, ${edgeCount} связей за ${generationTime}мс`);
        });
        this.manager.on('render:error', (event) => {
            console.error(`❌ Ошибка рендеринга:`, event.error);
        });
    }
    /**
     * Получить поддерживаемые форматы
     */
    getSupportedFormats() {
        return this.manager.getStats().supportedFormats;
    }
    /**
     * Получить статистику плагина
     */
    getStats() {
        return {
            ...this.manager.getStats(),
            isInstalled: this.isInstalled
        };
    }
}
// Глобальный экземпляр плагина
export const visualizationPlugin = new VisualizationPlugin();
//# sourceMappingURL=VisualizationPlugin.js.map