// React-подобные утилиты для работы с CSElement
import { CSElement } from '../core/CSElement';
import { useState, useEffect, useCallback, useRef } from 'react';
/**
 * React хук для управления одним CSElement
 */
export function useElement(options = {}) {
    const { name, data, parent } = options;
    const [element, setElement] = useState(null);
    const [loading, setLoading] = useState(false);
    const [error, setError] = useState(null);
    // Инициализация элемента
    useEffect(() => {
        const initializeElement = async () => {
            try {
                setLoading(true);
                setError(null);
                let newElement = null;
                if (parent && name) {
                    // Ищем существующий элемент в родителе
                    const existing = parent.getElement(name);
                    if (existing && existing instanceof CSElement) {
                        newElement = existing;
                    }
                    else {
                        // Создаем новый элемент в родителе
                        newElement = await parent.addElement(name);
                    }
                }
                else if (name) {
                    // Создаем независимый элемент
                    newElement = new CSElement(name);
                }
                else {
                    // Создаем элемент с автогенерируемым именем
                    const autoName = `element_${Date.now()}_${Math.random().toString(36).substr(2, 9)}`;
                    newElement = new CSElement(autoName);
                }
                // Устанавливаем данные если переданы
                if (data && newElement) {
                    for (const [key, value] of Object.entries(data)) {
                        await newElement.setData(key, value);
                    }
                }
                setElement(newElement);
            }
            catch (err) {
                setError(err);
            }
            finally {
                setLoading(false);
            }
        };
        initializeElement();
    }, [name, parent]); // Зависимости для пересоздания элемента
    const update = useCallback(async (newData) => {
        if (!element)
            return;
        try {
            setError(null);
            for (const [key, value] of Object.entries(newData)) {
                await element.setData(key, value);
            }
            // Принудительно обновляем состояние для ре-рендера
            setElement(prevElement => prevElement);
        }
        catch (err) {
            setError(err);
        }
    }, [element]);
    const remove = useCallback(async () => {
        if (!element)
            return;
        try {
            setError(null);
            // Если элемент имеет владельца, удаляем его из владельца
            if (element.mainOwner && element.mainOwner instanceof CSElement) {
                await element.mainOwner.removeElement(element);
            }
            // Уничтожаем элемент
            element.destroy();
            setElement(null);
        }
        catch (err) {
            setError(err);
        }
    }, [element]);
    const save = useCallback(async () => {
        if (!element)
            return;
        try {
            setError(null);
            // Используем встроенную систему персистентности CSElement
            await element.save();
        }
        catch (err) {
            setError(err);
        }
    }, [element]);
    return {
        element,
        loading,
        error,
        update,
        remove,
        save
    };
}
/**
 * React хук для выполнения запросов к CSElement
 */
export function useQuery(root, options = {}) {
    const { selector = '*', filter, sort, limit, offset = 0, autoRefresh = false, refreshInterval = 5000 } = options;
    const [elements, setElements] = useState([]);
    const [count, setCount] = useState(0);
    const [hasMore, setHasMore] = useState(false);
    const [loading, setLoading] = useState(false);
    const [error, setError] = useState(null);
    const currentOffset = useRef(offset);
    const intervalRef = useRef(null);
    const executeQuery = useCallback(() => {
        if (!root) {
            setElements([]);
            setCount(0);
            setHasMore(false);
            return;
        }
        try {
            setLoading(true);
            setError(null);
            // Получаем все дочерние элементы
            let results = [];
            if (selector === '*') {
                // Для селектора '*' получаем все дочерние элементы
                results = root.getAllElements().filter(el => el instanceof CSElement);
            }
            else {
                // Используем встроенную систему запросов CSElement
                results = root.query(selector);
            }
            // Применяем фильтр если есть
            if (filter) {
                results = results.filter(filter);
            }
            // Применяем сортировку если есть
            if (sort) {
                results = [...results].sort(sort);
            }
            const totalCount = results.length;
            setCount(totalCount);
            // Применяем пагинацию
            if (limit) {
                const start = currentOffset.current;
                const end = start + limit;
                const paginatedResults = results.slice(start, end);
                setElements(paginatedResults);
                setHasMore(end < totalCount);
            }
            else {
                setElements(results);
                setHasMore(false);
            }
        }
        catch (err) {
            setError(err);
            setElements([]);
            setCount(0);
            setHasMore(false);
        }
        finally {
            setLoading(false);
        }
    }, [root, selector, filter, sort, limit]);
    // Выполняем запрос при изменении зависимостей
    useEffect(() => {
        currentOffset.current = offset;
        executeQuery();
    }, [executeQuery, offset]);
    // Автообновление
    useEffect(() => {
        if (autoRefresh && refreshInterval > 0) {
            intervalRef.current = setInterval(executeQuery, refreshInterval);
            return () => {
                if (intervalRef.current) {
                    clearInterval(intervalRef.current);
                }
            };
        }
        return undefined;
    }, [autoRefresh, refreshInterval, executeQuery]);
    const refresh = useCallback(async () => {
        currentOffset.current = offset;
        executeQuery();
    }, [executeQuery, offset]);
    const loadMore = useCallback(async () => {
        if (!hasMore || !limit)
            return;
        currentOffset.current += limit;
        executeQuery();
    }, [hasMore, limit, executeQuery]);
    return {
        elements,
        count,
        hasMore,
        loading,
        error,
        refresh,
        loadMore
    };
}
/**
 * React хук для управления всей системой CSElement
 */
export function useCSElement(options = {}) {
    const { root: providedRoot, autoConnect = true, enableHistory = false, enableReactivity = false, enablePersistence = false } = options;
    const [root, setRoot] = useState(providedRoot || null);
    const [connected, setConnected] = useState(false);
    const [error, setError] = useState(null);
    const [stats, setStats] = useState({
        elementCount: 0,
        lastUpdate: null
    });
    // Инициализация системы
    useEffect(() => {
        const initializeSystem = async () => {
            try {
                setError(null);
                let systemRoot = root;
                if (!systemRoot && autoConnect) {
                    systemRoot = new CSElement('root');
                    setRoot(systemRoot);
                }
                if (systemRoot) {
                    setConnected(true);
                    // Инициализация плагинов
                    if (enableHistory) {
                        CSElement.configureHistory({ maxOperations: 100 });
                    }
                    if (enableReactivity) {
                        CSElement.configureReactivity({ debug: true });
                    }
                    if (enablePersistence) {
                        // Конфигурация персистентности
                    }
                    // Обновляем статистику
                    setStats({
                        elementCount: systemRoot.getAllElements().length,
                        lastUpdate: new Date()
                    });
                }
            }
            catch (err) {
                setError(err);
            }
        };
        initializeSystem();
    }, [autoConnect, enableHistory, enableReactivity, enablePersistence]);
    // Обновляем статистику при изменении root
    useEffect(() => {
        if (root) {
            const updateStats = () => {
                setStats({
                    elementCount: root.getAllElements().length,
                    lastUpdate: new Date()
                });
            };
            updateStats();
            // Подписываемся на события изменения
            const handleElementAdded = () => updateStats();
            const handleElementRemoved = () => updateStats();
            root.on('element:added', handleElementAdded);
            root.on('element:removed', handleElementRemoved);
            return () => {
                root.off('element:added', handleElementAdded);
                root.off('element:removed', handleElementRemoved);
            };
        }
        return undefined;
    }, [root]);
    const createElement = useCallback((name, data) => {
        if (!root)
            throw new Error('No root element available');
        const element = new CSElement(name);
        if (data) {
            // Асинхронно устанавливаем данные
            Object.entries(data).forEach(async ([key, value]) => {
                try {
                    await element.setData(key, value);
                }
                catch (err) {
                    console.error('Error setting data:', err);
                }
            });
        }
        return element;
    }, [root]);
    const query = useCallback((selector) => {
        if (!root)
            return [];
        if (selector === '*') {
            return root.getAllElements().filter(el => el instanceof CSElement);
        }
        return root.query(selector);
    }, [root]);
    const queryAdvanced = useCallback((selector) => {
        if (!root)
            return [];
        return root.queryAdvanced(selector);
    }, [root]);
    const queryCSS = useCallback((selector) => {
        if (!root)
            return [];
        return root.queryCSS(selector);
    }, [root]);
    const queryXPath = useCallback((selector) => {
        if (!root)
            return [];
        return root.queryXPath(selector);
    }, [root]);
    const exportData = useCallback(() => {
        if (!root)
            return null;
        return root.serialize({
            includeChildren: true,
            includeData: true,
            includeMetadata: true
        });
    }, [root]);
    const importData = useCallback(async (data) => {
        if (!data) {
            setError(new Error('Invalid data provided'));
            return;
        }
        try {
            setError(null);
            const importedElement = CSElement.deserialize(data);
            setRoot(importedElement);
            setConnected(true);
        }
        catch (err) {
            setError(err);
        }
    }, []);
    const connect = useCallback(async () => {
        try {
            setError(null);
            if (!root) {
                const newRoot = new CSElement('root');
                setRoot(newRoot);
            }
            setConnected(true);
        }
        catch (err) {
            setError(err);
        }
    }, [root]);
    const disconnect = useCallback(() => {
        setConnected(false);
    }, []);
    const clear = useCallback(async () => {
        if (!root)
            return;
        try {
            setError(null);
            // Получаем копию массива дочерних элементов
            const children = [...root.getAllElements()];
            // Удаляем все дочерние элементы
            for (const child of children) {
                if (child instanceof CSElement) {
                    await root.removeElement(child);
                }
            }
            // Обновляем статистику
            setStats({
                elementCount: 0,
                lastUpdate: new Date()
            });
        }
        catch (err) {
            setError(err);
        }
    }, [root]);
    return {
        root,
        connected,
        error,
        createElement,
        query,
        queryAdvanced,
        queryCSS,
        queryXPath,
        export: exportData,
        import: importData,
        connect,
        disconnect,
        clear,
        stats
    };
}
//# sourceMappingURL=hooks.js.map