/**
 * Реализация менеджера реактивности и computed свойств
 */
import { EventEmitter } from 'eventemitter3';
export class ReactivityManagerImpl extends EventEmitter {
    constructor(config = {}) {
        super();
        this.computedProperties = new Map();
        this.watchers = new Map();
        this.reactiveProperties = new Map();
        this.refs = new Map();
        this.currentlyComputing = new Set();
        this.notificationQueue = [];
        this.batchTimeout = null;
        this.statsInterval = null;
        this.stats = {
            computedCount: 0,
            watcherCount: 0,
            refCount: 0,
            notificationsPerSecond: 0,
            averageComputeTime: 0,
            memoryUsage: 0,
            totalComputeTime: 0,
            computeCallCount: 0
        };
        this.notificationHistory = [];
        this.currentDependencyTracker = null;
        this.config = {
            maxComputed: 1000,
            maxWatchers: 10000,
            defaultTTL: 60000, // 1 минута
            maxDepth: 10,
            batchNotifications: true,
            batchSize: 100,
            debug: false,
            ...config
        };
        // Обновляем статистику каждую секунду
        this.statsInterval = setInterval(() => this.updateStats(), 1000);
    }
    computed(compute, config = {}) {
        if (this.computedProperties.size >= this.config.maxComputed) {
            throw new Error(`Превышен лимит computed свойств: ${this.config.maxComputed}`);
        }
        const id = this.generateId('computed');
        const computedConfig = {
            lazy: true,
            cache: true,
            ttl: this.config.defaultTTL,
            maxDepth: this.config.maxDepth,
            debug: this.config.debug,
            ...config
        };
        const computedProperty = {
            id,
            compute,
            dependencies: new Set(),
            cachedValue: undefined,
            isValid: false,
            lastUpdated: 0,
            config: computedConfig
        };
        this.computedProperties.set(id, computedProperty);
        this.stats.computedCount++;
        // Если не ленивое, сразу вычисляем
        if (!computedConfig.lazy) {
            this.getComputedValue(id);
        }
        this.emitEvent('computed-created', {
            computedId: id,
            dependencies: Array.from(computedProperty.dependencies)
        });
        return computedProperty;
    }
    watch(path, callback, config = {}) {
        if (this.watchers.size >= this.config.maxWatchers) {
            throw new Error(`Превышен лимит наблюдателей: ${this.config.maxWatchers}`);
        }
        const id = this.generateId('watcher');
        const pathArray = Array.isArray(path) ? path : [path];
        const pathKey = pathArray.join('.');
        const watcherConfig = {
            immediate: false,
            deep: false,
            flush: 'async',
            debug: this.config.debug,
            ...config
        };
        const watcher = {
            id,
            path: pathArray,
            callback,
            config: watcherConfig,
            active: true
        };
        this.watchers.set(id, watcher);
        this.stats.watcherCount++;
        // Регистрируем наблюдатель для свойства
        if (!this.reactiveProperties.has(pathKey)) {
            this.reactiveProperties.set(pathKey, {
                path: pathArray,
                watchers: new Set(),
                computedDependents: new Set()
            });
        }
        this.reactiveProperties.get(pathKey).watchers.add(watcher);
        this.emitEvent('watcher-created', {
            watcherId: id,
            path: pathArray
        });
        // Немедленный вызов если настроен
        if (watcherConfig.immediate) {
            try {
                callback(undefined, undefined, pathArray);
            }
            catch (error) {
                console.error(`Ошибка в немедленном вызове наблюдателя ${id}:`, error);
            }
        }
        return watcher;
    }
    unwatch(watcherId) {
        const watcher = this.watchers.get(watcherId);
        if (!watcher) {
            return false;
        }
        const pathKey = watcher.path.join('.');
        const property = this.reactiveProperties.get(pathKey);
        if (property) {
            property.watchers.delete(watcher);
            if (property.watchers.size === 0 && property.computedDependents.size === 0) {
                this.reactiveProperties.delete(pathKey);
            }
        }
        this.watchers.delete(watcherId);
        this.stats.watcherCount--;
        this.emitEvent('watcher-removed', {
            watcherId
        });
        return true;
    }
    getComputedValue(computedId) {
        const computed = this.computedProperties.get(computedId);
        if (!computed) {
            return undefined;
        }
        // Проверяем валидность кэша
        if (computed.isValid && computed.config.cache) {
            if (computed.config.ttl && Date.now() - computed.lastUpdated > computed.config.ttl) {
                computed.isValid = false;
            }
            else {
                return computed.cachedValue;
            }
        }
        // Предотвращаем циклические зависимости
        if (this.currentlyComputing.has(computedId)) {
            throw new Error(`Обнаружена циклическая зависимость в computed свойстве ${computedId}`);
        }
        const startTime = Date.now();
        this.currentlyComputing.add(computedId);
        try {
            // Очищаем старые зависимости
            this.clearComputedDependencies(computedId);
            // Отслеживаем зависимости во время вычисления
            const oldDependencyTracker = this.currentDependencyTracker;
            this.currentDependencyTracker = computedId;
            const value = computed.compute();
            this.currentDependencyTracker = oldDependencyTracker;
            // Обновляем кэш
            computed.cachedValue = value;
            computed.isValid = true;
            computed.lastUpdated = Date.now();
            const computeTime = Date.now() - startTime;
            this.stats.totalComputeTime += computeTime;
            this.stats.computeCallCount++;
            this.emitEvent('computed-updated', {
                computedId,
                newValue: value,
                dependencies: Array.from(computed.dependencies)
            });
            return value;
        }
        catch (error) {
            console.error(`Ошибка при вычислении computed свойства ${computedId}:`, error);
            throw error;
        }
        finally {
            this.currentlyComputing.delete(computedId);
        }
    }
    invalidateComputed(computedId) {
        const computed = this.computedProperties.get(computedId);
        if (computed) {
            computed.isValid = false;
            this.emitEvent('computed-invalidated', {
                computedId
            });
        }
    }
    invalidateByPath(path) {
        const pathKey = path.join('.');
        const property = this.reactiveProperties.get(pathKey);
        if (property) {
            // Инвалидируем все зависимые computed свойства
            for (const computedId of property.computedDependents) {
                this.invalidateComputed(computedId);
            }
        }
    }
    notify(path, newValue, oldValue) {
        if (this.config.batchNotifications) {
            this.notificationQueue.push({ path, newValue, oldValue });
            this.scheduleBatchNotification();
        }
        else {
            this.processNotification(path, newValue, oldValue);
        }
    }
    scheduleBatchNotification() {
        if (this.batchTimeout) {
            return;
        }
        this.batchTimeout = setTimeout(() => {
            this.processBatchNotifications();
            this.batchTimeout = null;
        }, 1); // Увеличиваем задержку до 1 мс для более предсказуемого поведения в тестах
    }
    processBatchNotifications() {
        const notifications = this.notificationQueue.splice(0, this.config.batchSize);
        for (const { path, newValue, oldValue } of notifications) {
            this.processNotification(path, newValue, oldValue);
        }
        // Если остались уведомления, планируем следующий батч
        if (this.notificationQueue.length > 0) {
            this.scheduleBatchNotification();
        }
    }
    processNotification(path, newValue, oldValue) {
        const pathKey = path.join('.');
        // Ищем точное совпадение пути
        let property = this.reactiveProperties.get(pathKey);
        // Если не нашли точное совпадение, ищем по всем зарегистрированным путям
        if (!property) {
            for (const [, prop] of this.reactiveProperties) {
                // Проверяем если пути совпадают по содержимому
                if (JSON.stringify(prop.path) === JSON.stringify(path)) {
                    property = prop;
                    break;
                }
            }
        }
        if (!property) {
            // Если все еще не нашли, выводим отладочную информацию
            if (this.config.debug) {
                console.log(`Property not found for path: ${JSON.stringify(path)}`);
                console.log(`Available properties:`, Array.from(this.reactiveProperties.keys()));
            }
            return;
        }
        // Инвалидируем зависимые computed свойства
        this.invalidateByPath(path);
        // Уведомляем наблюдателей
        for (const watcher of property.watchers) {
            if (!watcher.active) {
                continue;
            }
            try {
                if (watcher.config.flush === 'sync') {
                    watcher.callback(newValue, oldValue, path);
                }
                else {
                    // Асинхронный вызов
                    setTimeout(() => {
                        if (watcher.active) {
                            watcher.callback(newValue, oldValue, path);
                        }
                    }, 1); // Увеличиваем задержку для более предсказуемого поведения в тестах
                }
                this.emitEvent('watcher-triggered', {
                    watcherId: watcher.id,
                    path,
                    newValue,
                    oldValue
                });
            }
            catch (error) {
                console.error(`Ошибка в наблюдателе ${watcher.id}:`, error);
            }
        }
        this.emitEvent('property-changed', {
            path,
            newValue,
            oldValue
        });
        // Обновляем статистику
        this.notificationHistory.push(Date.now());
    }
    ref(value) {
        const id = this.generateId('ref');
        const manager = this; // Сохраняем ссылку на менеджер
        const ref = {
            get value() {
                // Отслеживаем зависимость если мы внутри computed
                if (manager.currentDependencyTracker) {
                    manager.trackDependency(manager.currentDependencyTracker, [id]);
                }
                return value;
            },
            set value(newValue) {
                const oldValue = value;
                value = newValue;
                manager.notify([id], newValue, oldValue);
            },
            id,
            watchers: new Set()
        };
        this.refs.set(id, ref);
        this.stats.refCount++;
        return ref;
    }
    reactive(obj) {
        return this.createReactiveProxy(obj, []);
    }
    createReactiveProxy(obj, basePath) {
        return new Proxy(obj, {
            get: (target, prop, receiver) => {
                const path = [...basePath, String(prop)];
                // Отслеживаем зависимость если мы внутри computed
                if (this.currentDependencyTracker) {
                    this.trackDependency(this.currentDependencyTracker, path);
                }
                const value = Reflect.get(target, prop, receiver);
                // Если значение - объект, делаем его реактивным
                if (typeof value === 'object' && value !== null) {
                    return this.createReactiveProxy(value, path);
                }
                return value;
            },
            set: (target, prop, value, receiver) => {
                const path = [...basePath, String(prop)];
                const oldValue = Reflect.get(target, prop, receiver);
                const result = Reflect.set(target, prop, value, receiver);
                if (result && oldValue !== value) {
                    this.notify(path, value, oldValue);
                }
                return result;
            }
        });
    }
    trackDependency(computedId, path) {
        const computed = this.computedProperties.get(computedId);
        if (!computed) {
            return;
        }
        const pathKey = path.join('.');
        computed.dependencies.add(pathKey);
        // Регистрируем computed как зависимого от этого свойства
        if (!this.reactiveProperties.has(pathKey)) {
            this.reactiveProperties.set(pathKey, {
                path,
                watchers: new Set(),
                computedDependents: new Set()
            });
        }
        this.reactiveProperties.get(pathKey).computedDependents.add(computedId);
        this.emitEvent('dependency-added', {
            computedId,
            path
        });
    }
    clearComputedDependencies(computedId) {
        const computed = this.computedProperties.get(computedId);
        if (!computed) {
            return;
        }
        // Удаляем computed из списка зависимых для всех свойств
        for (const pathKey of computed.dependencies) {
            const property = this.reactiveProperties.get(pathKey);
            if (property) {
                property.computedDependents.delete(computedId);
                if (property.watchers.size === 0 && property.computedDependents.size === 0) {
                    this.reactiveProperties.delete(pathKey);
                }
            }
        }
        computed.dependencies.clear();
    }
    getDependencies(computedId) {
        const computed = this.computedProperties.get(computedId);
        return computed ? Array.from(computed.dependencies) : [];
    }
    getDependencyGraph() {
        const nodes = new Map();
        const edges = new Map();
        // Добавляем узлы для всех свойств
        for (const [pathKey, property] of this.reactiveProperties) {
            nodes.set(pathKey, {
                path: property.path,
                type: 'property',
                dependentCount: property.computedDependents.size + property.watchers.size,
                dependencyCount: 0
            });
        }
        // Добавляем узлы для computed свойств
        for (const [computedId, computed] of this.computedProperties) {
            nodes.set(computedId, {
                path: [computedId],
                type: 'computed',
                dependentCount: 0,
                dependencyCount: computed.dependencies.size
            });
            // Добавляем рёбра зависимостей
            edges.set(computedId, new Set(computed.dependencies));
        }
        // Добавляем узлы для refs
        for (const [refId] of this.refs) {
            nodes.set(refId, {
                path: [refId],
                type: 'ref',
                dependentCount: 0,
                dependencyCount: 0
            });
        }
        return { nodes, edges };
    }
    clear() {
        this.computedProperties.clear();
        this.watchers.clear();
        this.reactiveProperties.clear();
        this.refs.clear();
        this.currentlyComputing.clear();
        this.notificationQueue = [];
        if (this.batchTimeout) {
            clearTimeout(this.batchTimeout);
            this.batchTimeout = null;
        }
        if (this.statsInterval) {
            clearInterval(this.statsInterval);
            this.statsInterval = null;
        }
        this.stats = {
            computedCount: 0,
            watcherCount: 0,
            refCount: 0,
            notificationsPerSecond: 0,
            averageComputeTime: 0,
            memoryUsage: 0,
            totalComputeTime: 0,
            computeCallCount: 0
        };
    }
    getStats() {
        return {
            computedCount: this.stats.computedCount,
            watcherCount: this.stats.watcherCount,
            refCount: this.stats.refCount,
            notificationsPerSecond: this.stats.notificationsPerSecond,
            averageComputeTime: this.stats.averageComputeTime,
            memoryUsage: this.estimateMemoryUsage()
        };
    }
    updateStats() {
        // Обновляем количество уведомлений в секунду
        const now = Date.now();
        this.notificationHistory = this.notificationHistory.filter(time => now - time < 1000);
        this.stats.notificationsPerSecond = this.notificationHistory.length;
        // Обновляем среднее время вычисления
        if (this.stats.computeCallCount > 0) {
            this.stats.averageComputeTime = this.stats.totalComputeTime / this.stats.computeCallCount;
        }
    }
    estimateMemoryUsage() {
        let size = 0;
        // Приблизительный расчёт размера в байтах
        size += this.computedProperties.size * 1000; // ~1KB на computed
        size += this.watchers.size * 500; // ~500B на watcher
        size += this.reactiveProperties.size * 200; // ~200B на property
        size += this.refs.size * 100; // ~100B на ref
        return size;
    }
    generateId(prefix) {
        return `${prefix}_${Date.now()}_${Math.random().toString(36).substr(2, 9)}`;
    }
    emitEvent(event, data) {
        const eventData = {
            event,
            timestamp: Date.now(),
            ...data
        };
        this.emit(event, eventData);
        if (this.config.debug) {
            console.log(`[Reactivity] ${event}:`, eventData);
        }
    }
    // Методы для подписки на события
    onReactivityEvent(event, callback) {
        this.on(event, callback);
    }
    offReactivityEvent(event, callback) {
        this.off(event, callback);
    }
}
//# sourceMappingURL=ReactivityManagerImpl.js.map