/**
 * Менеджер State Machine для CSElement
 */
import { StateMachine } from './StateMachine';
/**
 * Реализация менеджера State Machine
 */
export class StateMachineManagerImpl {
    constructor() {
        this._stateMachines = new Map();
    }
    /**
     * Создать state machine для элемента
     */
    create(element, config) {
        // Проверяем, что у элемента еще нет state machine
        if (this._stateMachines.has(element.id)) {
            throw new Error(`State Machine для элемента "${element.id}" уже существует`);
        }
        // Создаем новый state machine
        const stateMachine = new StateMachine(element, config);
        // Сохраняем в реестре
        this._stateMachines.set(element.id, stateMachine);
        console.log(`🔄 State Machine "${config.name}" создан для элемента "${element.id}"`);
        return stateMachine;
    }
    /**
     * Получить state machine элемента
     */
    get(elementId) {
        return this._stateMachines.get(elementId);
    }
    /**
     * Проверить, есть ли state machine для элемента
     */
    has(elementId) {
        return this._stateMachines.has(elementId);
    }
    /**
     * Удалить state machine элемента
     */
    remove(elementId) {
        const stateMachine = this._stateMachines.get(elementId);
        if (stateMachine) {
            // Уничтожаем state machine
            stateMachine.destroy();
            // Удаляем из реестра
            this._stateMachines.delete(elementId);
            console.log(`🔄 State Machine для элемента "${elementId}" удален`);
            return true;
        }
        return false;
    }
    /**
     * Получить все state machines
     */
    getAll() {
        return new Map(this._stateMachines);
    }
    /**
     * Очистить все state machines
     */
    clear() {
        // Уничтожаем все state machines
        for (const stateMachine of this._stateMachines.values()) {
            stateMachine.destroy();
        }
        // Очищаем реестр
        this._stateMachines.clear();
        console.log('🔄 Все State Machines очищены');
    }
    /**
     * Получить статистику
     */
    getStats() {
        const stats = {
            total: this._stateMachines.size,
            byState: new Map(),
            byName: new Map()
        };
        for (const sm of this._stateMachines.values()) {
            // Статистика по состояниям
            const currentState = sm.currentState;
            stats.byState.set(currentState, (stats.byState.get(currentState) || 0) + 1);
            // Статистика по именам
            const name = sm.name;
            stats.byName.set(name, (stats.byName.get(name) || 0) + 1);
        }
        return stats;
    }
    /**
     * Найти state machines по критериям
     */
    findByState(stateName) {
        return Array.from(this._stateMachines.values())
            .filter(sm => sm.currentState === stateName);
    }
    /**
     * Найти state machines по имени
     */
    findByName(name) {
        return Array.from(this._stateMachines.values())
            .filter(sm => sm.name === name);
    }
    /**
     * Отправить событие всем state machines
     */
    async broadcastEvent(event, payload) {
        const results = new Map();
        for (const [elementId, sm] of this._stateMachines) {
            try {
                const result = await sm.send(event, payload);
                results.set(elementId, result);
            }
            catch (error) {
                results.set(elementId, { error: error instanceof Error ? error.message : String(error) });
            }
        }
        return results;
    }
    /**
     * Отправить событие state machines в определенном состоянии
     */
    async sendToState(stateName, event, payload) {
        const results = new Map();
        const targetMachines = this.findByState(stateName);
        for (const sm of targetMachines) {
            try {
                const result = await sm.send(event, payload);
                results.set(sm.context.element.id, result);
            }
            catch (error) {
                results.set(sm.context.element.id, {
                    success: false,
                    fromState: sm.currentState,
                    toState: sm.currentState,
                    event: { type: event, payload, timestamp: Date.now() },
                    executedActions: [],
                    error: error instanceof Error ? error.message : String(error),
                    duration: 0
                });
            }
        }
        return results;
    }
}
//# sourceMappingURL=StateMachineManager.js.map