/**
 * Менеджер блокировок для системы транзакций
 * Обеспечивает изоляцию и предотвращение конфликтов
 */
import { EventEmitter } from 'eventemitter3';
import { TransactionEventType } from '../types/transaction-interfaces';
/**
 * Реализация менеджера блокировок
 */
export class LockManagerImpl extends EventEmitter {
    constructor() {
        super();
        this.locks = new Map();
        this.waitingQueue = new Map();
        this.defaultTimeout = 30000; // 30 секунд
        this.deadlockCheckInterval = 5000; // 5 секунд
        this.startDeadlockDetection();
    }
    /**
     * Получение блокировки на элемент
     */
    async acquireLock(elementId, transactionId, lockType) {
        const existingLock = this.locks.get(elementId);
        // Если блокировки нет, создаем новую
        if (!existingLock) {
            return this.createLock(elementId, transactionId, lockType);
        }
        // Если блокировка принадлежит той же транзакции
        if (existingLock.transactionId === transactionId) {
            // Обновляем тип блокировки если нужно (read -> write)
            if (existingLock.lockType === 'read' && lockType === 'write') {
                existingLock.lockType = 'write';
                existingLock.timestamp = Date.now();
            }
            return true;
        }
        // Проверяем совместимость блокировок
        const compatible = this.areLocksCompatible(existingLock.lockType, lockType);
        if (compatible && lockType === 'read') {
            // Для read-блокировок можем создать разделяемую блокировку
            return this.createSharedLock(elementId, transactionId, lockType);
        }
        // Блокировки несовместимы - добавляем в очередь ожидания
        return this.waitForLock(elementId, transactionId, lockType);
    }
    /**
     * Освобождение блокировки
     */
    async releaseLock(elementId, transactionId) {
        const lock = this.locks.get(elementId);
        if (!lock || lock.transactionId !== transactionId) {
            return false;
        }
        this.locks.delete(elementId);
        this.emitEvent({
            type: TransactionEventType.LOCK_RELEASED,
            timestamp: Date.now(),
            transactionId,
            elementId
        });
        // Обрабатываем очередь ожидания
        await this.processWaitingQueue(elementId);
        return true;
    }
    /**
     * Освобождение всех блокировок транзакции
     */
    async releaseAllLocks(transactionId) {
        let releasedCount = 0;
        const locksToRelease = [];
        // Собираем все блокировки транзакции
        for (const [elementId, lock] of this.locks) {
            if (lock.transactionId === transactionId) {
                locksToRelease.push(elementId);
            }
        }
        // Освобождаем блокировки
        for (const elementId of locksToRelease) {
            if (await this.releaseLock(elementId, transactionId)) {
                releasedCount++;
            }
        }
        return releasedCount;
    }
    /**
     * Проверка блокировки элемента
     */
    isLocked(elementId) {
        return this.locks.has(elementId);
    }
    /**
     * Получение информации о блокировке
     */
    getLockInfo(elementId) {
        const lock = this.locks.get(elementId);
        if (!lock) {
            return { locked: false };
        }
        return {
            locked: true,
            transactionId: lock.transactionId,
            lockType: lock.lockType,
            timestamp: lock.timestamp
        };
    }
    /**
     * Обнаружение взаимоблокировок
     */
    detectDeadlocks() {
        const graph = this.buildDependencyGraph();
        const deadlocks = [];
        const visited = new Set();
        const recursionStack = new Set();
        // Поиск циклов в графе зависимостей
        for (const transactionId of Object.keys(graph)) {
            if (!visited.has(transactionId)) {
                const cycle = this.findCycle(transactionId, graph, visited, recursionStack, []);
                if (cycle.length > 0) {
                    deadlocks.push(cycle);
                }
            }
        }
        if (deadlocks.length > 0) {
            this.emitEvent({
                type: TransactionEventType.DEADLOCK_DETECTED,
                timestamp: Date.now(),
                transactionId: deadlocks[0][0],
                metadata: { deadlocks }
            });
        }
        return deadlocks;
    }
    /**
     * Принудительное снятие блокировок
     */
    async forceReleaseLocks(elementIds) {
        let releasedCount = 0;
        for (const elementId of elementIds) {
            const lock = this.locks.get(elementId);
            if (lock) {
                this.locks.delete(elementId);
                releasedCount++;
                this.emitEvent({
                    type: TransactionEventType.LOCK_RELEASED,
                    timestamp: Date.now(),
                    transactionId: lock.transactionId,
                    elementId,
                    metadata: { forced: true }
                });
                // Обрабатываем очередь ожидания
                await this.processWaitingQueue(elementId);
            }
        }
        return releasedCount;
    }
    /**
     * Создание новой блокировки
     */
    createLock(elementId, transactionId, lockType) {
        const lockInfo = {
            elementId,
            transactionId,
            lockType,
            timestamp: Date.now(),
            expiresAt: Date.now() + this.defaultTimeout
        };
        this.locks.set(elementId, lockInfo);
        this.emitEvent({
            type: TransactionEventType.LOCK_ACQUIRED,
            timestamp: Date.now(),
            transactionId,
            elementId,
            metadata: { lockType }
        });
        return true;
    }
    /**
     * Создание разделяемой блокировки (для read-операций)
     */
    createSharedLock(elementId, transactionId, lockType) {
        // В упрощенной реализации просто создаем обычную блокировку
        // В полной реализации здесь была бы поддержка множественных read-блокировок
        return this.createLock(elementId, transactionId, lockType);
    }
    /**
     * Ожидание освобождения блокировки
     */
    async waitForLock(elementId, transactionId, lockType) {
        return new Promise((resolve) => {
            if (!this.waitingQueue.has(elementId)) {
                this.waitingQueue.set(elementId, []);
            }
            const queue = this.waitingQueue.get(elementId);
            queue.push({
                transactionId,
                lockType,
                resolve,
                timestamp: Date.now()
            });
            // Устанавливаем таймаут
            setTimeout(() => {
                const index = queue.findIndex(item => item.transactionId === transactionId);
                if (index >= 0) {
                    queue.splice(index, 1);
                    resolve(false);
                }
            }, this.defaultTimeout);
        });
    }
    /**
     * Обработка очереди ожидания после освобождения блокировки
     */
    async processWaitingQueue(elementId) {
        const queue = this.waitingQueue.get(elementId);
        if (!queue || queue.length === 0) {
            return;
        }
        // Берем первый запрос из очереди
        const nextRequest = queue.shift();
        // Пытаемся создать блокировку
        const success = this.createLock(elementId, nextRequest.transactionId, nextRequest.lockType);
        nextRequest.resolve(success);
        // Если остались запросы и новая блокировка совместима, обрабатываем их
        if (success && nextRequest.lockType === 'read' && queue.length > 0) {
            const compatibleRequests = queue.filter(req => req.lockType === 'read');
            for (const request of compatibleRequests) {
                const index = queue.indexOf(request);
                if (index >= 0) {
                    queue.splice(index, 1);
                    const lockCreated = this.createSharedLock(elementId, request.transactionId, request.lockType);
                    request.resolve(lockCreated);
                }
            }
        }
    }
    /**
     * Проверка совместимости блокировок
     */
    areLocksCompatible(existingType, requestedType) {
        // read-read совместимы, остальные комбинации - нет
        return existingType === 'read' && requestedType === 'read';
    }
    /**
     * Построение графа зависимостей для обнаружения взаимоблокировок
     */
    buildDependencyGraph() {
        const graph = {};
        // Инициализируем граф
        for (const lock of this.locks.values()) {
            if (!graph[lock.transactionId]) {
                graph[lock.transactionId] = new Set();
            }
        }
        // Добавляем зависимости из очереди ожидания
        for (const [elementId, queue] of this.waitingQueue) {
            const currentLock = this.locks.get(elementId);
            if (currentLock) {
                for (const waitingRequest of queue) {
                    if (!graph[waitingRequest.transactionId]) {
                        graph[waitingRequest.transactionId] = new Set();
                    }
                    graph[waitingRequest.transactionId].add(currentLock.transactionId);
                }
            }
        }
        return graph;
    }
    /**
     * Поиск цикла в графе зависимостей
     */
    findCycle(transactionId, graph, visited, recursionStack, path) {
        visited.add(transactionId);
        recursionStack.add(transactionId);
        path.push(transactionId);
        const dependencies = graph[transactionId] || new Set();
        for (const dependencyId of dependencies) {
            if (!visited.has(dependencyId)) {
                const cycle = this.findCycle(dependencyId, graph, visited, recursionStack, path);
                if (cycle.length > 0) {
                    return cycle;
                }
            }
            else if (recursionStack.has(dependencyId)) {
                // Найден цикл
                const cycleStart = path.indexOf(dependencyId);
                return path.slice(cycleStart);
            }
        }
        recursionStack.delete(transactionId);
        path.pop();
        return [];
    }
    /**
     * Запуск периодической проверки взаимоблокировок
     */
    startDeadlockDetection() {
        this.deadlockCheckTimer = setInterval(() => {
            const deadlocks = this.detectDeadlocks();
            if (deadlocks.length > 0) {
                console.warn('Обнаружены взаимоблокировки:', deadlocks);
                // В реальной реализации здесь была бы логика разрешения взаимоблокировок
            }
        }, this.deadlockCheckInterval);
    }
    /**
     * Остановка проверки взаимоблокировок
     */
    stopDeadlockDetection() {
        if (this.deadlockCheckTimer) {
            clearInterval(this.deadlockCheckTimer);
            this.deadlockCheckTimer = undefined;
        }
    }
    /**
     * Очистка ресурсов
     */
    destroy() {
        this.stopDeadlockDetection();
        this.locks.clear();
        this.waitingQueue.clear();
        this.removeAllListeners();
    }
    /**
     * Получение статистики блокировок
     */
    getStats() {
        const locks = Array.from(this.locks.values());
        const readLocks = locks.filter(lock => lock.lockType === 'read').length;
        const writeLocks = locks.filter(lock => lock.lockType === 'write').length;
        let waitingRequests = 0;
        for (const queue of this.waitingQueue.values()) {
            waitingRequests += queue.length;
        }
        const oldestLock = locks.reduce((oldest, current) => {
            return !oldest || current.timestamp < oldest.timestamp ? current : oldest;
        }, undefined);
        return {
            totalLocks: locks.length,
            readLocks,
            writeLocks,
            waitingRequests,
            oldestLock
        };
    }
    /**
     * Генерация события транзакционной системы
     */
    emitEvent(eventData) {
        this.emit('transaction:event', eventData);
        this.emit(eventData.type, eventData);
    }
}
//# sourceMappingURL=LockManagerImpl.js.map