/**
 * Расширенный движок запросов для CSElement
 * Поддерживает CSS-селекторы, XPath, индексирование и оптимизацию запросов
 */
import { QueryEngine } from './QueryEngine';
/**
 * Типы селекторов
 */
export var SelectorType;
(function (SelectorType) {
    SelectorType["CSS"] = "css";
    SelectorType["XPATH"] = "xpath";
    SelectorType["OBJECT"] = "object";
})(SelectorType || (SelectorType = {}));
/**
 * Расширенный движок запросов
 */
export class AdvancedQueryEngine {
    /**
     * Выполняет поиск элементов по универсальному селектору
     */
    static query(root, selector) {
        const startTime = performance.now();
        const selectorString = this.getSelectorString(selector);
        // Проверяем кэш
        const cached = this.getCachedResult(selectorString);
        if (cached) {
            this.stats.cacheHits++;
            this.stats.totalQueries++; // Кэшированные запросы тоже считаются как запросы
            // Обновляем частоту селекторов для кэшированных запросов
            const currentCount = this.selectorFrequency.get(selectorString) || 0;
            this.selectorFrequency.set(selectorString, currentCount + 1);
            // Обновляем топ селекторов
            this.stats.mostFrequentSelectors = Array.from(this.selectorFrequency.entries())
                .map(([sel, count]) => ({ selector: sel, count }))
                .sort((a, b) => b.count - a.count)
                .slice(0, 10);
            return cached;
        }
        let result;
        if (typeof selector === 'string') {
            // Автоматически определяем тип селектора
            if (selector.startsWith('//') || selector.startsWith('/')) {
                result = this.queryXPath(root, selector);
            }
            else {
                result = this.queryCSS(root, selector);
            }
        }
        else {
            switch (selector.type) {
                case SelectorType.CSS:
                    result = this.queryCSS(root, selector.selector);
                    break;
                case SelectorType.XPATH:
                    result = this.queryXPath(root, selector.expression);
                    break;
                case SelectorType.OBJECT:
                    result = this.queryObject(root, selector.selector);
                    break;
                default:
                    throw new Error(`Неподдерживаемый тип селектора: ${selector.type}`);
            }
        }
        // Обновляем статистику
        const executionTime = performance.now() - startTime;
        this.updateStats(selectorString, executionTime);
        // Кэшируем результат
        this.cacheResult(selectorString, result);
        return result;
    }
    /**
     * Находит первый элемент по селектору
     */
    static queryOne(root, selector) {
        const results = this.query(root, selector);
        return results.length > 0 ? results[0] : null;
    }
    /**
     * Выполняет поиск по CSS-селектору
     */
    static queryCSS(root, selector) {
        const parsed = this.parseCSS(selector);
        return this.executeCSS(root, parsed);
    }
    /**
     * Выполняет поиск по XPath
     */
    static queryXPath(root, expression) {
        const context = this.createXPathContext(root);
        return this.evaluateXPath(context, expression);
    }
    /**
     * Выполняет поиск по объектному селектору
     */
    static queryObject(root, selector) {
        return QueryEngine.query(root, selector);
    }
    /**
     * Включить/выключить индексирование
     */
    static setIndexingEnabled(enabled) {
        // В данной реализации индексирование всегда включено
        // Этот метод оставлен для совместимости API
        console.log(`Индексирование ${enabled ? 'включено' : 'выключено'}`);
    }
    /**
     * Создает или обновляет индекс для элемента
     */
    static buildIndex(root) {
        const index = {
            byName: new Map(),
            byId: new Map(),
            byClass: new Map(),
            byAttribute: new Map(),
            byDepth: new Map(),
            all: new Set()
        };
        this.traverseAndIndex(root, index, 0);
        this.indices.set(root, index);
    }
    /**
     * Получает индекс для элемента
     */
    static getIndex(root) {
        let index = this.indices.get(root);
        if (!index) {
            this.buildIndex(root);
            index = this.indices.get(root);
        }
        return index;
    }
    /**
     * Очищает кэш запросов
     */
    static clearCache() {
        this.queryCache.clear();
    }
    /**
     * Получает статистику запросов
     */
    static getStats() {
        return { ...this.stats };
    }
    /**
     * Сбрасывает статистику
     */
    static resetStats() {
        this.stats = {
            totalQueries: 0,
            cacheHits: 0,
            averageExecutionTime: 0,
            slowestQuery: { selector: '', time: 0 },
            mostFrequentSelectors: []
        };
        this.selectorFrequency.clear();
    }
    /**
     * Приватные методы
     */
    static getSelectorString(selector) {
        if (typeof selector === 'string') {
            return selector;
        }
        switch (selector.type) {
            case SelectorType.CSS:
                return `css:${selector.selector}`;
            case SelectorType.XPATH:
                return `xpath:${selector.expression}`;
            case SelectorType.OBJECT:
                return `object:${JSON.stringify(selector.selector)}`;
            default:
                return 'unknown';
        }
    }
    static getCachedResult(selector) {
        const cached = this.queryCache.get(selector);
        if (cached && Date.now() - cached.timestamp < this.cacheTimeout) {
            return cached.result;
        }
        if (cached) {
            this.queryCache.delete(selector);
        }
        return null;
    }
    static cacheResult(selector, result) {
        this.queryCache.set(selector, {
            result: [...result],
            timestamp: Date.now()
        });
    }
    static updateStats(selector, executionTime) {
        this.stats.totalQueries++;
        // Обновляем среднее время выполнения
        this.stats.averageExecutionTime =
            (this.stats.averageExecutionTime * (this.stats.totalQueries - 1) + executionTime) / this.stats.totalQueries;
        // Обновляем самый медленный запрос
        if (executionTime > this.stats.slowestQuery.time) {
            this.stats.slowestQuery = { selector, time: executionTime };
        }
        // Обновляем частоту селекторов
        const currentCount = this.selectorFrequency.get(selector) || 0;
        this.selectorFrequency.set(selector, currentCount + 1);
        // Обновляем топ селекторов
        this.stats.mostFrequentSelectors = Array.from(this.selectorFrequency.entries())
            .map(([sel, count]) => ({ selector: sel, count }))
            .sort((a, b) => b.count - a.count)
            .slice(0, 10);
    }
    static parseCSS(selector) {
        // Улучшенный CSS парсер
        const result = {};
        // Убираем лишние пробелы и разбиваем по комбинаторам
        const trimmed = selector.trim();
        const combinatorMatch = trimmed.match(/^([^>+~\s]+)(?:\s*([>+~]|\s+)\s*(.+))?$/);
        if (!combinatorMatch) {
            return result;
        }
        const [, currentPart, combinator, restPart] = combinatorMatch;
        // Парсим текущую часть селектора
        this.parseSelectorPart(currentPart, result);
        // Обрабатываем комбинаторы
        if (combinator && restPart) {
            result.combinator = combinator.trim() === '' ? ' ' : combinator.trim();
            result.next = this.parseCSS(restPart);
        }
        return result;
    }
    static parseSelectorPart(part, result) {
        let remaining = part;
        // Парсим элемент, ID, классы, атрибуты и псевдо-классы
        while (remaining) {
            if (remaining.startsWith('#')) {
                // ID селектор
                const match = remaining.match(/^#([^.:\[]+)/);
                if (match) {
                    result.id = match[1];
                    remaining = remaining.substring(match[0].length);
                }
                else {
                    break;
                }
            }
            else if (remaining.startsWith('.')) {
                // Class селектор
                const match = remaining.match(/^\.([^.:\[#]+)/);
                if (match) {
                    if (!result.classes)
                        result.classes = [];
                    result.classes.push(match[1]);
                    remaining = remaining.substring(match[0].length);
                }
                else {
                    break;
                }
            }
            else if (remaining.startsWith('[')) {
                // Атрибутный селектор
                const match = remaining.match(/^\[([^=\]]+)(?:(=|~=|\|=|\^=|\$=|\*=)"?([^"\]]+)"?)?\]/);
                if (match) {
                    if (!result.attributes)
                        result.attributes = [];
                    result.attributes.push({
                        name: match[1],
                        operator: match[2],
                        value: match[3]
                    });
                    remaining = remaining.substring(match[0].length);
                }
                else {
                    break;
                }
            }
            else if (remaining.startsWith(':')) {
                // Псевдо-класс
                const match = remaining.match(/^:([^:(]+)(?:\(([^)]+)\))?/);
                if (match) {
                    if (!result.pseudoClasses)
                        result.pseudoClasses = [];
                    result.pseudoClasses.push({
                        name: match[1],
                        argument: match[2]
                    });
                    remaining = remaining.substring(match[0].length);
                }
                else {
                    break;
                }
            }
            else {
                // Имя элемента (должно быть в начале)
                const match = remaining.match(/^([a-zA-Z][a-zA-Z0-9-_]*)/);
                if (match && !result.element && !result.id && !result.classes && !result.attributes && !result.pseudoClasses) {
                    result.element = match[1];
                    remaining = remaining.substring(match[0].length);
                }
                else {
                    break;
                }
            }
        }
    }
    static executeCSS(root, parsed) {
        const results = [];
        if (parsed.combinator && parsed.next) {
            // Обрабатываем комбинаторы
            const intermediateResults = this.findElementsByParsedSelector(root, parsed);
            for (const element of intermediateResults) {
                switch (parsed.combinator) {
                    case '>': // Прямые дочерние элементы
                        const children = element.getAllElements();
                        for (const child of children) {
                            if (this.matchesParsedSelector(child, parsed.next)) {
                                results.push(child);
                            }
                        }
                        break;
                    case '+': // Следующий соседний элемент
                        // Реализация для следующего соседа
                        break;
                    case '~': // Все следующие соседние элементы
                        // Реализация для всех следующих соседей
                        break;
                    case ' ': // Потомки
                        const descendants = this.findElementsByParsedSelector(element, parsed.next);
                        results.push(...descendants);
                        break;
                }
            }
        }
        else {
            // Простой селектор без комбинаторов
            return this.findElementsByParsedSelector(root, parsed);
        }
        return results;
    }
    static findElementsByParsedSelector(root, parsed) {
        const results = [];
        const index = this.getIndex(root);
        // Оптимизация: используем индекс если возможно
        let candidates = null;
        if (parsed.id) {
            const element = index.byId.get(parsed.id);
            candidates = element ? new Set([element]) : new Set();
        }
        else if (parsed.element) {
            candidates = index.byName.get(parsed.element) || new Set();
        }
        else if (parsed.classes && parsed.classes.length > 0) {
            candidates = index.byClass.get(parsed.classes[0]) || new Set();
            for (let i = 1; i < parsed.classes.length; i++) {
                const classSet = index.byClass.get(parsed.classes[i]) || new Set();
                candidates = new Set([...candidates].filter(el => classSet.has(el)));
            }
        }
        // Если нет кандидатов из индекса, используем все элементы
        if (!candidates) {
            candidates = index.all;
        }
        // Проверяем каждого кандидата
        for (const element of candidates) {
            if (this.matchesParsedSelector(element, parsed)) {
                results.push(element);
            }
        }
        return results;
    }
    static findDescendantsByParsedSelector(root, parsed) {
        const results = [];
        // Рекурсивно ищем среди потомков, не включая сам root
        const searchInChildren = (element) => {
            const children = element.getAllElements();
            for (const child of children) {
                if (this.matchesParsedSelector(child, parsed)) {
                    results.push(child);
                }
                // Рекурсивно ищем в потомках
                searchInChildren(child);
            }
        };
        searchInChildren(root);
        return results;
    }
    static matchesParsedSelector(element, parsed) {
        // Проверяем имя элемента
        if (parsed.element && element.name !== parsed.element) {
            return false;
        }
        // Проверяем ID
        if (parsed.id && element.getData('id') !== parsed.id) {
            return false;
        }
        // Проверяем классы
        if (parsed.classes) {
            const elementClasses = element.getData('class');
            if (!elementClasses)
                return false;
            const classList = typeof elementClasses === 'string' ? elementClasses.split(' ') : [];
            for (const className of parsed.classes) {
                if (!classList.includes(className)) {
                    return false;
                }
            }
        }
        // Проверяем атрибуты
        if (parsed.attributes) {
            for (const attr of parsed.attributes) {
                const value = element.getData(attr.name);
                if (!value)
                    return false;
                if (attr.operator && attr.value) {
                    switch (attr.operator) {
                        case '=':
                            if (value !== attr.value)
                                return false;
                            break;
                        case '~=':
                            if (!value.toString().split(' ').includes(attr.value))
                                return false;
                            break;
                        case '|=':
                            if (!value.toString().startsWith(attr.value + '-') && value !== attr.value)
                                return false;
                            break;
                        case '^=':
                            if (!value.toString().startsWith(attr.value))
                                return false;
                            break;
                        case '$=':
                            if (!value.toString().endsWith(attr.value))
                                return false;
                            break;
                        case '*=':
                            if (!value.toString().includes(attr.value))
                                return false;
                            break;
                    }
                }
            }
        }
        // Проверяем псевдо-классы
        if (parsed.pseudoClasses) {
            for (const pseudo of parsed.pseudoClasses) {
                if (!this.matchesPseudoClass(element, pseudo)) {
                    return false;
                }
            }
        }
        return true;
    }
    static matchesPseudoClass(element, pseudo) {
        switch (pseudo.name) {
            case 'first-child':
                return element.index === 0;
            case 'last-child':
                const parent = element.mainOwner;
                return parent ? element.index === parent.elementsCount() - 1 : true;
            case 'nth-child':
                if (pseudo.argument) {
                    const n = parseInt(pseudo.argument);
                    return element.index === n - 1; // CSS использует 1-based индексы
                }
                return false;
            case 'empty':
                return element.elementsCount() === 0;
            case 'not':
                // Реализация :not() селектора
                if (pseudo.argument) {
                    const notParsed = this.parseCSS(pseudo.argument);
                    return !this.matchesParsedSelector(element, notParsed);
                }
                return false;
            case 'has':
                // Реализация :has() селектора
                if (pseudo.argument) {
                    const hasParsed = this.parseCSS(pseudo.argument);
                    // Ищем только среди потомков, не включая сам элемент
                    const descendants = this.findDescendantsByParsedSelector(element, hasParsed);
                    return descendants.length > 0;
                }
                return false;
            default:
                return false;
        }
    }
    static createXPathContext(root) {
        // Создаем контекст для XPath
        return {
            root,
            currentNode: root,
            nodeMap: new Map()
        };
    }
    static evaluateXPath(context, expression) {
        // Упрощенная реализация XPath
        const results = [];
        if (expression === '//*') {
            // Все элементы
            this.getAllDescendants(context.root, results);
        }
        else if (expression.startsWith('//')) {
            // Поиск по всем потомкам
            const elementName = expression.substring(2);
            this.findByNameInDescendants(context.root, elementName, results);
        }
        else if (expression.startsWith('/')) {
            // Поиск от корня
            const parts = expression.substring(1).split('/');
            let current = context.root;
            for (const part of parts) {
                if (part === '*')
                    continue;
                const child = current.getElement(part);
                if (child) {
                    current = child;
                }
                else {
                    return [];
                }
            }
            results.push(current);
        }
        return results;
    }
    static getAllDescendants(element, results) {
        results.push(element);
        const children = element.getAllElements();
        for (const child of children) {
            this.getAllDescendants(child, results);
        }
    }
    static findByNameInDescendants(element, name, results) {
        if (element.name === name) {
            results.push(element);
        }
        const children = element.getAllElements();
        for (const child of children) {
            this.findByNameInDescendants(child, name, results);
        }
    }
    static traverseAndIndex(element, index, depth) {
        // Добавляем в общий индекс
        index.all.add(element);
        // Индексируем по имени
        if (element.name) {
            if (!index.byName.has(element.name)) {
                index.byName.set(element.name, new Set());
            }
            index.byName.get(element.name).add(element);
        }
        // Индексируем по ID
        const id = element.getData('id');
        if (id) {
            index.byId.set(id, element);
        }
        // Индексируем по классам
        const classes = element.getData('class');
        if (classes) {
            const classList = typeof classes === 'string' ? classes.split(' ') : [classes];
            for (const className of classList) {
                if (!index.byClass.has(className)) {
                    index.byClass.set(className, new Set());
                }
                index.byClass.get(className).add(element);
            }
        }
        // Индексируем по атрибутам
        const data = element.data;
        for (const [key, value] of data) {
            if (!index.byAttribute.has(key)) {
                index.byAttribute.set(key, new Map());
            }
            const attrMap = index.byAttribute.get(key);
            if (!attrMap.has(value)) {
                attrMap.set(value, new Set());
            }
            attrMap.get(value).add(element);
        }
        // Индексируем по глубине
        if (!index.byDepth.has(depth)) {
            index.byDepth.set(depth, new Set());
        }
        index.byDepth.get(depth).add(element);
        // Рекурсивно обрабатываем дочерние элементы
        const children = element.getAllElements();
        for (const child of children) {
            this.traverseAndIndex(child, index, depth + 1);
        }
    }
}
AdvancedQueryEngine.indices = new WeakMap();
AdvancedQueryEngine.queryCache = new Map();
AdvancedQueryEngine.cacheTimeout = 5000; // 5 секунд
AdvancedQueryEngine.stats = {
    totalQueries: 0,
    cacheHits: 0,
    averageExecutionTime: 0,
    slowestQuery: { selector: '', time: 0 },
    mostFrequentSelectors: []
};
AdvancedQueryEngine.selectorFrequency = new Map();
//# sourceMappingURL=AdvancedQueryEngine.js.map