/**
 * Движок запросов для поиска элементов
 */
export class QueryEngine {
    /**
     * Выполняет поиск элементов по селектору
     */
    static query(root, selector) {
        if (typeof selector === 'string') {
            return this.parseAndQuery(root, selector);
        }
        return this.queryBySelector(root, selector);
    }
    /**
     * Находит первый элемент по селектору
     */
    static queryOne(root, selector) {
        const results = this.query(root, selector);
        return results.length > 0 ? results[0] : null;
    }
    /**
     * Парсит строковый селектор и выполняет поиск
     */
    static parseAndQuery(root, selectorString) {
        const selector = this.parseSelector(selectorString);
        return this.queryBySelector(root, selector);
    }
    /**
     * Парсит CSS-like селектор в объект QuerySelector
     */
    static parseSelector(selectorString) {
        const selector = {};
        // Парсим имя элемента (например: "button", ".myClass", "#myId")
        if (selectorString.startsWith('#')) {
            // ID селектор (используем как имя)
            selector.name = selectorString.substring(1);
        }
        else if (selectorString.startsWith('.')) {
            // Class селектор (ищем в данных элемента)
            const className = selectorString.substring(1);
            selector.hasData = ['class'];
            selector.custom = (element) => {
                const classes = element.getData('class');
                return typeof classes === 'string' && classes.split(' ').includes(className);
            };
        }
        else if (selectorString.includes('[')) {
            // Атрибутный селектор
            const match = selectorString.match(/^([^[]+)?\[([^=]+)(?:=([^\]]+))?\]$/);
            if (match) {
                const [, name, attr, value] = match;
                if (name)
                    selector.name = name;
                if (value) {
                    selector.custom = (element) => element.getData(attr) === value;
                }
                else {
                    selector.hasData = [attr];
                }
            }
        }
        else if (selectorString.includes(':nth-child(')) {
            // Псевдо-селектор nth-child
            const match = selectorString.match(/^([^:]+)?:nth-child\((\d+)\)$/);
            if (match) {
                const [, name, indexStr] = match;
                if (name)
                    selector.name = name;
                selector.index = parseInt(indexStr) - 1; // CSS использует 1-based индексы
            }
        }
        else {
            // Простой селектор по имени
            selector.name = selectorString;
        }
        return selector;
    }
    /**
     * Выполняет поиск по объекту селектора
     */
    static queryBySelector(root, selector) {
        const results = [];
        this.traverseAndMatch(root, selector, results);
        return results;
    }
    /**
     * Рекурсивно обходит дерево и собирает подходящие элементы
     */
    static traverseAndMatch(element, selector, results) {
        if (this.matchesSelector(element, selector)) {
            results.push(element);
        }
        // Рекурсивно проверяем дочерние элементы
        const children = element.getAllElements();
        for (const child of children) {
            this.traverseAndMatch(child, selector, results);
        }
    }
    /**
     * Проверяет, соответствует ли элемент селектору
     */
    static matchesSelector(element, selector) {
        // Проверяем имя
        if (selector.name !== undefined && element.name !== selector.name) {
            return false;
        }
        // Проверяем индекс
        if (selector.index !== undefined && element.index !== selector.index) {
            return false;
        }
        // Проверяем наличие данных
        if (selector.hasData) {
            for (const key of selector.hasData) {
                if (!element.getData(key)) {
                    return false;
                }
            }
        }
        // Проверяем глубину
        if (selector.depth !== undefined) {
            const depth = this.getElementDepth(element);
            if (typeof selector.depth === 'number') {
                if (depth !== selector.depth)
                    return false;
            }
            else {
                const { min, max } = selector.depth;
                if (min !== undefined && depth < min)
                    return false;
                if (max !== undefined && depth > max)
                    return false;
            }
        }
        // Проверяем кастомное условие
        if (selector.custom && !selector.custom(element)) {
            return false;
        }
        return true;
    }
    /**
     * Вычисляет глубину элемента в дереве
     */
    static getElementDepth(element) {
        let depth = 0;
        let current = element.mainOwner;
        while (current) {
            depth++;
            current = current.mainOwner;
        }
        return depth;
    }
    /**
     * Создает комплексный селектор для поиска элементов
     */
    static createSelector() {
        return new SelectorBuilder();
    }
}
/**
 * Builder для создания сложных селекторов
 */
export class SelectorBuilder {
    constructor() {
        this.selector = {};
    }
    /**
     * Устанавливает имя элемента
     */
    withName(name) {
        this.selector.name = name;
        return this;
    }
    /**
     * Устанавливает индекс элемента
     */
    withIndex(index) {
        this.selector.index = index;
        return this;
    }
    /**
     * Требует наличие определенных данных
     */
    withData(...keys) {
        this.selector.hasData = [...(this.selector.hasData || []), ...keys];
        return this;
    }
    /**
     * Устанавливает ограничение по глубине
     */
    withDepth(depth) {
        this.selector.depth = depth;
        return this;
    }
    /**
     * Добавляет кастомное условие
     */
    withCustom(predicate) {
        const existing = this.selector.custom;
        if (existing) {
            this.selector.custom = (element) => existing(element) && predicate(element);
        }
        else {
            this.selector.custom = predicate;
        }
        return this;
    }
    /**
     * Строит финальный селектор
     */
    build() {
        return { ...this.selector };
    }
}
/**
 * Предопределенные селекторы
 */
export const CommonSelectors = {
    /**
     * Все элементы с определенным именем
     */
    byName: (name) => ({ name }),
    /**
     * Все элементы с определенным индексом
     */
    byIndex: (index) => ({ index }),
    /**
     * Все листовые элементы (без детей)
     */
    leaves: () => ({
        custom: (element) => element.elementsCount() === 0
    }),
    /**
     * Все корневые элементы (без родителей)
     */
    roots: () => ({
        custom: (element) => element.mainOwner === null
    }),
    /**
     * Элементы с определенным количеством детей
     */
    withChildrenCount: (count) => ({
        custom: (element) => element.elementsCount() === count
    }),
    /**
     * Элементы на определенной глубине
     */
    atDepth: (depth) => ({ depth }),
    /**
     * Элементы с данными определенного типа
     */
    withDataType: (key, type) => ({
        custom: (element) => typeof element.getData(key) === type
    })
};
//# sourceMappingURL=QueryEngine.js.map