/**
 * Система валидации для CSElement
 */
export class ElementValidator {
    constructor() {
        this.rules = new Map();
    }
    /**
     * Добавляет правило валидации
     */
    addRule(elementType, rule) {
        if (!this.rules.has(elementType)) {
            this.rules.set(elementType, []);
        }
        this.rules.get(elementType).push(rule);
    }
    /**
     * Удаляет все правила для типа элемента
     */
    clearRules(elementType) {
        this.rules.delete(elementType);
    }
    /**
     * Валидирует элемент
     */
    validate(element, elementType = 'default') {
        const errors = [];
        const rules = this.rules.get(elementType) || [];
        for (const rule of rules) {
            const error = this.validateRule(element, rule);
            if (error) {
                errors.push(error);
            }
        }
        return {
            isValid: errors.length === 0,
            errors
        };
    }
    /**
     * Валидирует одно правило
     */
    validateRule(element, rule) {
        const value = this.getFieldValue(element, rule.field);
        switch (rule.type) {
            case 'required':
                if (value === undefined || value === null || value === '') {
                    return {
                        field: rule.field,
                        message: rule.message || `Поле ${rule.field} обязательно для заполнения`,
                        value
                    };
                }
                break;
            case 'type':
                if (value !== undefined && typeof value !== rule.value) {
                    return {
                        field: rule.field,
                        message: rule.message || `Поле ${rule.field} должно быть типа ${rule.value}`,
                        value
                    };
                }
                break;
            case 'range':
                if (typeof value === 'number' && rule.value) {
                    const { min, max } = rule.value;
                    if ((min !== undefined && value < min) || (max !== undefined && value > max)) {
                        return {
                            field: rule.field,
                            message: rule.message || `Поле ${rule.field} должно быть в диапазоне [${min}, ${max}]`,
                            value
                        };
                    }
                }
                break;
            case 'custom':
                if (rule.validator) {
                    const result = rule.validator(value, element);
                    if (typeof result === 'string') {
                        return {
                            field: rule.field,
                            message: result,
                            value
                        };
                    }
                    else if (!result) {
                        return {
                            field: rule.field,
                            message: rule.message || `Поле ${rule.field} не прошло валидацию`,
                            value
                        };
                    }
                }
                break;
        }
        return null;
    }
    /**
     * Получает значение поля элемента
     */
    getFieldValue(element, field) {
        switch (field) {
            case 'name':
                return element.name;
            case 'index':
                return element.index;
            case 'id':
                return element.id;
            default:
                // Проверяем в данных элемента
                return element.getData(field);
        }
    }
    /**
     * Создает стандартные правила валидации
     */
    static createStandardRules() {
        const validator = new ElementValidator();
        // Правила для базового элемента
        validator.addRule('default', {
            field: 'name',
            type: 'type',
            value: 'string',
            message: 'Имя элемента должно быть строкой'
        });
        validator.addRule('default', {
            field: 'index',
            type: 'type',
            value: 'number',
            message: 'Индекс элемента должен быть числом'
        });
        validator.addRule('default', {
            field: 'index',
            type: 'range',
            value: { min: 0 },
            message: 'Индекс элемента должен быть неотрицательным'
        });
        return validator;
    }
}
/**
 * Глобальный валидатор по умолчанию
 */
export const defaultValidator = ElementValidator.createStandardRules();
//# sourceMappingURL=Validator.js.map