/**
 * Менеджер Web Workers для CSElement
 * Управляет пулом воркеров и выполнением тяжелых операций
 */
import { EventEmitter } from 'events';
import { WorkerOperationType } from '../types/worker-interfaces';
export class WorkerManager extends EventEmitter {
    constructor(config = {}, poolConfig = {}) {
        super();
        this.workers = new Map();
        this.workerQueue = [];
        this.pendingOperations = new Map();
        this.operationCounter = 0;
        this.isInitialized = false;
        this.config = {
            maxConcurrentOperations: 4,
            timeoutMs: 30000,
            memoryLimitMB: 512,
            enableLogging: true,
            enableProfiling: false,
            enableCache: true,
            cacheSize: 100,
            ...config
        };
        this.poolConfig = {
            size: Math.max(1, navigator.hardwareConcurrency || 4),
            strategy: 'least-busy',
            healthCheck: true,
            recycleAfter: 1000,
            ...poolConfig
        };
    }
    /**
     * Инициализация пула воркеров
     */
    async initialize() {
        if (this.isInitialized) {
            return;
        }
        try {
            // Создаем пул воркеров
            const workerPromises = Array.from({ length: this.poolConfig.size }, () => this.createWorker());
            await Promise.all(workerPromises);
            this.isInitialized = true;
            if (this.config.enableLogging) {
                console.log(`WorkerManager initialized with ${this.poolConfig.size} workers`);
            }
            this.emit('initialized', { poolSize: this.poolConfig.size });
        }
        catch (error) {
            this.emit('error', { type: 'initialization', error });
            throw error;
        }
    }
    /**
     * Создание нового воркера
     */
    async createWorker(config) {
        const workerId = `worker-${Date.now()}-${Math.random().toString(36).substr(2, 9)}`;
        try {
            const worker = new CSElementWorker(workerId, config || this.config);
            await worker.initialize();
            this.workers.set(workerId, worker);
            this.workerQueue.push(worker);
            // Настройка обработчиков событий
            worker.on('message', (response) => {
                this.handleWorkerResponse(response);
            });
            worker.on('error', (error) => {
                this.handleWorkerError(workerId, error);
            });
            worker.on('terminated', () => {
                this.handleWorkerTermination(workerId);
            });
            if (this.config.enableLogging) {
                console.log(`Worker ${workerId} created`);
            }
            this.emit('worker:created', { workerId });
            return worker;
        }
        catch (error) {
            this.emit('worker:error', { workerId, error });
            throw error;
        }
    }
    /**
     * Завершение работы воркера
     */
    async terminateWorker(workerId) {
        const worker = this.workers.get(workerId);
        if (!worker) {
            throw new Error(`Worker ${workerId} not found`);
        }
        try {
            await worker.terminate();
            this.workers.delete(workerId);
            // Удаляем из очереди
            const index = this.workerQueue.findIndex(w => w.id === workerId);
            if (index !== -1) {
                this.workerQueue.splice(index, 1);
            }
            if (this.config.enableLogging) {
                console.log(`Worker ${workerId} terminated`);
            }
            this.emit('worker:terminated', { workerId });
        }
        catch (error) {
            this.emit('worker:error', { workerId, error });
            throw error;
        }
    }
    /**
     * Выполнение операции
     */
    async executeOperation(operation) {
        if (!this.isInitialized) {
            await this.initialize();
        }
        const worker = this.selectWorker();
        if (!worker) {
            throw new Error('No available workers');
        }
        return new Promise((resolve, reject) => {
            const timeout = setTimeout(() => {
                this.pendingOperations.delete(operation.id);
                reject(new Error(`Operation ${operation.id} timed out`));
            }, this.config.timeoutMs);
            this.pendingOperations.set(operation.id, {
                resolve,
                reject,
                timeout
            });
            worker.execute(operation).catch(error => {
                this.pendingOperations.delete(operation.id);
                clearTimeout(timeout);
                reject(error);
            });
        });
    }
    /**
     * Получение статуса воркера
     */
    getWorkerStatus(workerId) {
        if (workerId) {
            const worker = this.workers.get(workerId);
            if (!worker) {
                throw new Error(`Worker ${workerId} not found`);
            }
            return this.createWorkerStatus(worker);
        }
        // Возвращаем статус первого доступного воркера
        const worker = this.workerQueue[0];
        if (!worker) {
            throw new Error('No workers available');
        }
        return this.createWorkerStatus(worker);
    }
    /**
     * Получение статуса пула
     */
    getPoolStatus() {
        const totalWorkers = this.workers.size;
        const activeWorkers = Array.from(this.workers.values())
            .filter(w => w.status === 'busy').length;
        const totalOperations = Array.from(this.workers.values())
            .reduce((sum, w) => sum + w.operationsCount, 0);
        const averageExecutionTime = totalOperations > 0
            ? Array.from(this.workers.values())
                .reduce((sum, w) => sum + w.totalExecutionTime || 0, 0) / totalOperations
            : 0;
        const memoryUsage = Array.from(this.workers.values())
            .reduce((sum, w) => sum + w.memoryUsage, 0);
        return {
            totalWorkers,
            activeWorkers,
            queuedOperations: this.pendingOperations.size,
            totalOperations,
            averageExecutionTime,
            memoryUsage
        };
    }
    /**
     * Выбор воркера для выполнения операции
     */
    selectWorker() {
        const availableWorkers = this.workerQueue.filter(w => w.status === 'idle' || w.status === 'busy');
        if (availableWorkers.length === 0) {
            return null;
        }
        switch (this.poolConfig.strategy) {
            case 'round-robin':
                return this.selectRoundRobin(availableWorkers);
            case 'least-busy':
                return this.selectLeastBusy(availableWorkers);
            case 'random':
                return this.selectRandom(availableWorkers);
            default:
                return availableWorkers[0];
        }
    }
    selectRoundRobin(workers) {
        const index = this.operationCounter % workers.length;
        this.operationCounter++;
        return workers[index];
    }
    selectLeastBusy(workers) {
        return workers.reduce((least, current) => current.operationsCount < least.operationsCount ? current : least);
    }
    selectRandom(workers) {
        const index = Math.floor(Math.random() * workers.length);
        return workers[index];
    }
    /**
     * Обработка ответа от воркера
     */
    handleWorkerResponse(response) {
        const pending = this.pendingOperations.get(response.id);
        if (!pending) {
            return;
        }
        clearTimeout(pending.timeout);
        this.pendingOperations.delete(response.id);
        if (response.success) {
            pending.resolve(response.result);
        }
        else {
            pending.reject(new Error(response.error));
        }
        this.emit('operation:completed', response);
    }
    /**
     * Обработка ошибки воркера
     */
    handleWorkerError(workerId, error) {
        if (this.config.enableLogging) {
            console.error(`Worker ${workerId} error:`, error);
        }
        this.emit('worker:error', { workerId, error });
        // Если воркер критически поврежден, пересоздаем его
        if (error.code === 'CRITICAL_ERROR') {
            this.recreateWorker(workerId);
        }
    }
    /**
     * Обработка завершения воркера
     */
    handleWorkerTermination(workerId) {
        this.workers.delete(workerId);
        const index = this.workerQueue.findIndex(w => w.id === workerId);
        if (index !== -1) {
            this.workerQueue.splice(index, 1);
        }
        this.emit('worker:terminated', { workerId });
    }
    /**
     * Пересоздание воркера
     */
    async recreateWorker(workerId) {
        try {
            await this.terminateWorker(workerId);
            await this.createWorker();
        }
        catch (error) {
            this.emit('error', { type: 'recreation', workerId, error });
        }
    }
    /**
     * Создание объекта статуса воркера
     */
    createWorkerStatus(worker) {
        return {
            id: worker.id,
            status: worker.status,
            operationsCount: worker.operationsCount,
            averageExecutionTime: worker.averageExecutionTime || 0,
            memoryUsage: worker.memoryUsage,
            lastError: undefined
        };
    }
    /**
     * Завершение работы всех воркеров
     */
    async shutdown() {
        const terminationPromises = Array.from(this.workers.keys()).map(workerId => this.terminateWorker(workerId));
        await Promise.all(terminationPromises);
        this.isInitialized = false;
        this.emit('shutdown');
    }
    /**
     * Проверка здоровья воркеров
     */
    async healthCheck() {
        if (!this.poolConfig.healthCheck) {
            return {};
        }
        const healthPromises = Array.from(this.workers.entries()).map(async ([workerId, worker]) => {
            try {
                const healthMessage = {
                    id: `health-${Date.now()}`,
                    type: WorkerOperationType.CUSTOM_OPERATION,
                    timestamp: Date.now(),
                    payload: { operation: 'health-check' }
                };
                await worker.execute(healthMessage);
                return [workerId, true];
            }
            catch {
                return [workerId, false];
            }
        });
        const results = await Promise.all(healthPromises);
        return Object.fromEntries(results);
    }
}
/**
 * Реализация WorkerInstance
 */
class CSElementWorker extends EventEmitter {
    constructor(id, _config) {
        super();
        this.status = 'idle';
        this.operationsCount = 0;
        this.memoryUsage = 0;
        this.worker = null;
        this.totalExecutionTime = 0;
        this.id = id;
        this.createdAt = Date.now();
        this.lastActivity = Date.now();
    }
    async initialize() {
        try {
            // Создаем Web Worker
            const workerScript = this.generateWorkerScript();
            const blob = new Blob([workerScript], { type: 'application/javascript' });
            const workerUrl = URL.createObjectURL(blob);
            this.worker = new Worker(workerUrl);
            this.worker.onmessage = (event) => {
                this.handleMessage(event.data);
            };
            this.worker.onerror = (error) => {
                this.handleError({
                    code: 'WORKER_ERROR',
                    message: error.message,
                    stack: error.filename + ':' + error.lineno
                });
            };
            // Очищаем URL после создания воркера
            URL.revokeObjectURL(workerUrl);
            this.status = 'idle';
        }
        catch (error) {
            this.status = 'error';
            throw error;
        }
    }
    async execute(operation) {
        if (!this.worker || this.status === 'terminated') {
            throw new Error('Worker is not available');
        }
        this.status = 'busy';
        this.lastActivity = Date.now();
        return new Promise((resolve, reject) => {
            const startTime = Date.now();
            const messageHandler = (event) => {
                const response = event.data;
                if (response.id === operation.id) {
                    this.worker.removeEventListener('message', messageHandler);
                    const executionTime = Date.now() - startTime;
                    this.totalExecutionTime += executionTime;
                    this.operationsCount++;
                    this.status = 'idle';
                    if (response.success) {
                        resolve(response.result);
                    }
                    else {
                        reject(new Error(response.error));
                    }
                    this.emit('message', response);
                }
            };
            this.worker.addEventListener('message', messageHandler);
            this.worker.postMessage(operation);
        });
    }
    async terminate() {
        if (this.worker) {
            this.worker.terminate();
            this.worker = null;
        }
        this.status = 'terminated';
        this.emit('terminated');
    }
    handleMessage(_data) {
        this.lastActivity = Date.now();
        // Сообщения обрабатываются в execute()
    }
    handleError(error) {
        this.status = 'error';
        this.emit('error', error);
    }
    get averageExecutionTime() {
        return this.operationsCount > 0 ? this.totalExecutionTime / this.operationsCount : 0;
    }
    /**
     * Генерация кода для Web Worker
     */
    generateWorkerScript() {
        return `
      // Web Worker script for CSElement operations
      
      const operations = new Map();
      
      // Импорт необходимых функций (будет заменен на реальную реализацию)
      ${this.getWorkerOperations()}
      
      self.onmessage = async function(event) {
        const message = event.data;
        const startTime = Date.now();
        
        try {
          const result = await executeOperation(message);
          
          self.postMessage({
            id: message.id,
            success: true,
            result: result,
            timestamp: Date.now(),
            executionTime: Date.now() - startTime
          });
        } catch (error) {
          self.postMessage({
            id: message.id,
            success: false,
            error: error.message,
            timestamp: Date.now(),
            executionTime: Date.now() - startTime
          });
        }
      };
      
      async function executeOperation(message) {
        const { type, payload } = message;
        
        switch (type) {
          case 'serialize_element':
            return serializeElement(payload);
          case 'deserialize_element':
            return deserializeElement(payload);
          case 'query_elements':
            return queryElements(payload);
          case 'compute_diff':
            return computeDiff(payload);
          case 'three_way_merge':
            return threeWayMerge(payload);
          case 'analyze_structure':
            return analyzeStructure(payload);
          default:
            throw new Error('Unknown operation type: ' + type);
        }
      }
    `;
    }
    /**
     * Получение операций для воркера
     */
    getWorkerOperations() {
        return `
      function serializeElement(payload) {
        // Базовая сериализация
        return JSON.stringify(payload);
      }
      
      function deserializeElement(payload) {
        // Базовая десериализация
        return JSON.parse(payload.data);
      }
      
      function queryElements(payload) {
        // Заглушка для запросов
        return { elements: [], total: 0, hasMore: false, executionTime: 0 };
      }
      
      function computeDiff(payload) {
        // Заглушка для diff
        return { 
          changes: [], 
          summary: { additions: 0, deletions: 0, modifications: 0 }
        };
      }
      
      function threeWayMerge(payload) {
        // Заглушка для merge
        return {
          merged: payload.base,
          conflicts: [],
          autoResolved: 0,
          manualRequired: 0,
          success: true,
          metadata: {
            algorithm: 'basic',
            strategy: 'auto',
            executionTime: 0
          }
        };
      }
      
      function analyzeStructure(payload) {
        // Заглушка для анализа
        return {
          metrics: {},
          insights: [],
          recommendations: []
        };
      }
    `;
    }
}
//# sourceMappingURL=WorkerManager.js.map