/**
 * Система сериализации CSElement для Web Workers
 * Обеспечивает эффективную передачу данных между основным потоком и воркерами
 */
import { CSElement } from '../core/CSElement';
import { ICSElement } from '../types/interfaces';
import { SerializableElement, WorkerSerializationContext, SerializationResult, WorkerTransferData } from '../types/worker-interfaces';
/**
 * Менеджер сериализации для Web Workers
 */
export declare class WorkerSerializer {
    private static readonly MAX_DEPTH;
    private static readonly MAX_SIZE_MB;
    private context;
    private referenceMap;
    private circularRefs;
    constructor(context?: Partial<WorkerSerializationContext>);
    /**
     * Сериализация CSElement для передачи в Worker
     */
    serializeElement(element: ICSElement): SerializableElement;
    /**
     * Сериализация дерева элементов
     */
    serializeTree(rootElement: ICSElement): WorkerTransferData;
    /**
     * Десериализация элемента из Worker
     */
    deserializeElement(data: SerializableElement): CSElement;
    /**
     * Десериализация дерева элементов
     */
    deserializeTree(transferData: WorkerTransferData): CSElement;
    /**
     * Создание оптимизированного представления для передачи
     */
    createTransferableData(element: ICSElement, options?: {
        includeMetadata?: boolean;
        compressionLevel?: 'none' | 'fast' | 'best';
        format?: 'json' | 'binary' | 'compressed';
    }): SerializationResult;
    /**
     * Рекурсивная сериализация элемента
     */
    private serializeElementRecursive;
    /**
     * Сериализация значения с поддержкой пользовательских сериализаторов
     */
    private serializeValue;
    /**
     * Извлечение всех элементов из сериализованного дерева
     */
    private extractAllElements;
    /**
     * Конвертация SerializableElement в SerializedElement
     */
    private convertToSerializedElement;
    /**
     * Получение пути элемента
     */
    private getElementPath;
    /**
     * Подсчет количества элементов
     */
    private countElements;
    /**
     * Вычисление глубины дерева
     */
    private calculateDepth;
    /**
     * Оценка размера данных
     */
    private estimateSize;
    /**
     * Генерация ID сессии
     */
    private generateSessionId;
    /**
     * Сериализация в бинарный формат
     */
    private serializeToBinary;
    /**
     * Сериализация в MessagePack формат
     */
    private serializeToMessagePack;
    /**
     * Сжатие данных
     */
    private compressData;
}
/**
 * Утилитарные функции для работы с сериализацией
 */
export declare class SerializationUtils {
    /**
     * Проверка поддержки Transferable Objects
     */
    static supportsTransferableObjects(): boolean;
    /**
     * Создание Transferable Objects для оптимальной передачи
     */
    static createTransferableObjects(data: any): {
        data: any;
        transferable: Transferable[];
    };
    /**
     * Валидация сериализованных данных
     */
    static validateSerializedData(data: any): {
        valid: boolean;
        errors: string[];
    };
    /**
     * Оптимизация данных для передачи
     */
    static optimizeForTransfer(data: SerializableElement): SerializableElement;
}
//# sourceMappingURL=WorkerSerializer.d.ts.map