/**
 * Система сериализации CSElement для Web Workers
 * Обеспечивает эффективную передачу данных между основным потоком и воркерами
 */
import { CSElement } from '../core/CSElement';
/**
 * Менеджер сериализации для Web Workers
 */
export class WorkerSerializer {
    constructor(context = {}) {
        this.referenceMap = new Map();
        this.circularRefs = new Set();
        this.context = {
            preserveReferences: true,
            includeCircularRefs: false,
            maxDepth: WorkerSerializer.MAX_DEPTH,
            excludeFields: [],
            customSerializers: {},
            ...context
        };
    }
    /**
     * Сериализация CSElement для передачи в Worker
     */
    serializeElement(element) {
        this.referenceMap.clear();
        this.circularRefs.clear();
        return this.serializeElementRecursive(element, 0, new Set());
    }
    /**
     * Сериализация дерева элементов
     */
    serializeTree(rootElement) {
        try {
            const serializedRoot = this.serializeElement(rootElement);
            const allElements = this.extractAllElements(serializedRoot);
            const transferData = {
                elements: allElements,
                operations: [],
                context: {
                    rootId: rootElement.id,
                    timestamp: Date.now(),
                    sessionId: this.generateSessionId()
                }
            };
            // Проверяем размер данных
            const dataSize = this.estimateSize(transferData);
            if (dataSize > WorkerSerializer.MAX_SIZE_MB * 1024 * 1024) {
                throw new Error(`Serialized data too large: ${dataSize} bytes`);
            }
            return transferData;
        }
        catch (error) {
            throw new Error(`Serialization failed: ${error instanceof Error ? error.message : String(error)}`);
        }
    }
    /**
     * Десериализация элемента из Worker
     */
    deserializeElement(data) {
        try {
            // Конвертируем SerializableElement в формат, ожидаемый CSElement.deserialize
            const serializedData = this.convertToSerializedElement(data);
            return CSElement.deserialize(serializedData);
        }
        catch (error) {
            throw new Error(`Deserialization failed: ${error instanceof Error ? error.message : String(error)}`);
        }
    }
    /**
     * Десериализация дерева элементов
     */
    deserializeTree(transferData) {
        if (!transferData.context?.rootId) {
            throw new Error('Root element ID not found in transfer data');
        }
        const rootData = transferData.elements[transferData.context.rootId];
        if (!rootData) {
            throw new Error(`Root element ${transferData.context.rootId} not found`);
        }
        return this.deserializeElement(rootData);
    }
    /**
     * Создание оптимизированного представления для передачи
     */
    createTransferableData(element, options = {}) {
        const { compressionLevel = 'fast', format = 'json' } = options;
        const serialized = this.serializeElement(element);
        let data;
        let size;
        let compressionRatio;
        switch (format) {
            case 'json':
                data = JSON.stringify(serialized);
                size = new Blob([data]).size;
                break;
            case 'binary':
                data = this.serializeToBinary(serialized);
                size = data.byteLength;
                break;
            case 'compressed':
                data = this.serializeToMessagePack(serialized);
                size = data.byteLength;
                break;
            default:
                throw new Error(`Unsupported format: ${format}`);
        }
        // Применяем сжатие если требуется
        if (compressionLevel !== 'none') {
            const compressed = this.compressData(data, compressionLevel);
            compressionRatio = size / compressed.byteLength;
            data = compressed;
            size = compressed.byteLength;
        }
        const metadata = {
            elementCount: this.countElements(serialized),
            depth: this.calculateDepth(serialized),
            serializedAt: Date.now()
        };
        return {
            data,
            format,
            size,
            compressionRatio,
            metadata
        };
    }
    /**
     * Рекурсивная сериализация элемента
     */
    serializeElementRecursive(element, depth, visited) {
        if (depth > this.context.maxDepth) {
            throw new Error(`Maximum serialization depth exceeded: ${depth}`);
        }
        if (visited.has(element.id)) {
            if (this.context.includeCircularRefs) {
                this.circularRefs.add(element.id);
                return { id: element.id, name: element.name, data: {}, children: {} };
            }
            else {
                throw new Error(`Circular reference detected: ${element.id}`);
            }
        }
        visited.add(element.id);
        // Сериализуем данные элемента
        const data = {};
        for (const [key, value] of element.data.entries()) {
            if (!this.context.excludeFields.includes(key)) {
                data[key] = this.serializeValue(value);
            }
        }
        // Сериализуем дочерние элементы
        const children = {};
        for (const childElement of element.getAllElements()) {
            if (childElement instanceof CSElement) {
                children[childElement.id] = this.serializeElementRecursive(childElement, depth + 1, new Set(visited));
            }
        }
        const result = {
            id: element.id,
            name: element.name,
            data,
            children
        };
        // Добавляем метаданные если требуется
        if (this.context.preserveReferences) {
            result.metadata = {
                index: element.index,
                depth,
                path: this.getElementPath(element),
                createdAt: Date.now(),
                updatedAt: Date.now()
            };
        }
        return result;
    }
    /**
     * Сериализация значения с поддержкой пользовательских сериализаторов
     */
    serializeValue(value) {
        if (value === null || value === undefined) {
            return value;
        }
        // Проверяем пользовательские сериализаторы
        const valueType = typeof value;
        if (this.context.customSerializers[valueType]) {
            return this.context.customSerializers[valueType](value);
        }
        // Стандартная сериализация
        if (value instanceof Date) {
            return { __type: 'Date', value: value.toISOString() };
        }
        if (value instanceof RegExp) {
            return { __type: 'RegExp', source: value.source, flags: value.flags };
        }
        if (value instanceof Map) {
            return { __type: 'Map', entries: Array.from(value.entries()) };
        }
        if (value instanceof Set) {
            return { __type: 'Set', values: Array.from(value.values()) };
        }
        if (ArrayBuffer.isView(value)) {
            return { __type: 'TypedArray', constructor: value.constructor.name, data: Array.from(value) };
        }
        if (typeof value === 'function') {
            return { __type: 'Function', source: value.toString() };
        }
        return value;
    }
    /**
     * Извлечение всех элементов из сериализованного дерева
     */
    extractAllElements(root) {
        const elements = {};
        const queue = [root];
        while (queue.length > 0) {
            const current = queue.shift();
            elements[current.id] = current;
            Object.values(current.children).forEach(child => {
                queue.push(child);
            });
        }
        return elements;
    }
    /**
     * Конвертация SerializableElement в SerializedElement
     */
    convertToSerializedElement(data) {
        const result = {
            id: data.id,
            name: data.name,
            index: data.metadata?.index,
            data: data.data
        };
        // Конвертируем дочерние элементы
        if (Object.keys(data.children).length > 0) {
            result.children = Object.values(data.children).map(child => this.convertToSerializedElement(child));
        }
        return result;
    }
    /**
     * Получение пути элемента
     */
    getElementPath(element) {
        // Используем встроенный метод CSElement
        if (element instanceof CSElement) {
            return element.getPath();
        }
        // Fallback для других реализаций ICSElement
        return [element.name];
    }
    /**
     * Подсчет количества элементов
     */
    countElements(element) {
        let count = 1;
        Object.values(element.children).forEach(child => {
            count += this.countElements(child);
        });
        return count;
    }
    /**
     * Вычисление глубины дерева
     */
    calculateDepth(element) {
        if (Object.keys(element.children).length === 0) {
            return 1;
        }
        const childDepths = Object.values(element.children).map(child => this.calculateDepth(child));
        return 1 + Math.max(...childDepths);
    }
    /**
     * Оценка размера данных
     */
    estimateSize(data) {
        return new Blob([JSON.stringify(data)]).size;
    }
    /**
     * Генерация ID сессии
     */
    generateSessionId() {
        return `session-${Date.now()}-${Math.random().toString(36).substr(2, 9)}`;
    }
    /**
     * Сериализация в бинарный формат
     */
    serializeToBinary(data) {
        // Простая реализация - в реальном проекте можно использовать Protocol Buffers или MessagePack
        const jsonString = JSON.stringify(data);
        const encoder = new TextEncoder();
        return encoder.encode(jsonString).buffer;
    }
    /**
     * Сериализация в MessagePack формат
     */
    serializeToMessagePack(data) {
        // Заглушка - в реальном проекте нужна библиотека MessagePack
        return this.serializeToBinary(data);
    }
    /**
     * Сжатие данных
     */
    compressData(data, _level) {
        // Заглушка для сжатия - в реальном проекте можно использовать pako или другую библиотеку
        if (typeof data === 'string') {
            const encoder = new TextEncoder();
            return encoder.encode(data).buffer;
        }
        return data;
    }
}
WorkerSerializer.MAX_DEPTH = 50;
WorkerSerializer.MAX_SIZE_MB = 100;
/**
 * Утилитарные функции для работы с сериализацией
 */
export class SerializationUtils {
    /**
     * Проверка поддержки Transferable Objects
     */
    static supportsTransferableObjects() {
        try {
            // Проверяем поддержку Transferable Objects в Web Worker контексте
            if (typeof Worker === 'undefined') {
                return false;
            }
            const buffer = new ArrayBuffer(1);
            // В браузерном контексте postMessage может не быть доступен
            if (typeof postMessage === 'function') {
                // Используем правильный синтаксис для postMessage с transferable objects
                const channel = new MessageChannel();
                channel.port1.postMessage(buffer, [buffer]);
                return buffer.byteLength === 0;
            }
            return true; // Предполагаем поддержку если нет способа проверить
        }
        catch {
            return false;
        }
    }
    /**
     * Создание Transferable Objects для оптимальной передачи
     */
    static createTransferableObjects(data) {
        const transferable = [];
        // Поиск ArrayBuffer и других Transferable объектов
        const processValue = (value) => {
            if (value instanceof ArrayBuffer) {
                transferable.push(value);
                return value;
            }
            if (value instanceof MessagePort) {
                transferable.push(value);
                return value;
            }
            if (Array.isArray(value)) {
                return value.map(processValue);
            }
            if (value && typeof value === 'object') {
                const result = {};
                for (const [key, val] of Object.entries(value)) {
                    result[key] = processValue(val);
                }
                return result;
            }
            return value;
        };
        return {
            data: processValue(data),
            transferable
        };
    }
    /**
     * Валидация сериализованных данных
     */
    static validateSerializedData(data) {
        const errors = [];
        if (!data) {
            errors.push('Data is null or undefined');
            return { valid: false, errors };
        }
        if (typeof data !== 'object') {
            errors.push('Data must be an object');
            return { valid: false, errors };
        }
        // Проверяем обязательные поля
        if (!data.id) {
            errors.push('Missing required field: id');
        }
        if (!data.name) {
            errors.push('Missing required field: name');
        }
        if (!data.data || typeof data.data !== 'object') {
            errors.push('Invalid or missing data field');
        }
        if (!data.children || typeof data.children !== 'object') {
            errors.push('Invalid or missing children field');
        }
        return {
            valid: errors.length === 0,
            errors
        };
    }
    /**
     * Оптимизация данных для передачи
     */
    static optimizeForTransfer(data) {
        // Удаляем избыточные метаданные
        const optimized = { ...data };
        if (optimized.metadata) {
            // Оставляем только критически важные метаданные
            optimized.metadata = {
                index: optimized.metadata.index,
                depth: optimized.metadata.depth
            };
        }
        // Рекурсивно оптимизируем дочерние элементы
        const optimizedChildren = {};
        Object.entries(optimized.children).forEach(([key, child]) => {
            optimizedChildren[key] = SerializationUtils.optimizeForTransfer(child);
        });
        optimized.children = optimizedChildren;
        return optimized;
    }
}
//# sourceMappingURL=WorkerSerializer.js.map