# 💾 Персистентность и Адаптеры

CSElement предоставляет мощную систему персистентности с поддержкой различных хранилищ данных. Теперь с реальными реализациями IndexedDB и LocalForage адаптеров!

## 📋 Содержание

- [💾 Персистентность и Адаптеры](#-персистентность-и-адаптеры)
  - [📋 Содержание](#-содержание)
  - [🔧 Установка зависимостей](#-установка-зависимостей)
  - [🏗️ Архитектура системы](#️-архитектура-системы)
  - [📦 Встроенные адаптеры](#-встроенные-адаптеры)
    - [IndexedDB Адаптер](#indexeddb-адаптер)
    - [LocalForage Адаптер](#localforage-адаптер)
    - [Memory Storage Адаптер](#memory-storage-адаптер)
  - [⚙️ Настройка PersistenceManager](#️-настройка-persistencemanager)
  - [💾 Базовые операции](#-базовые-операции)
  - [🔄 Автоматическая синхронизация](#-автоматическая-синхронизация)
  - [🗜️ Сжатие данных](#️-сжатие-данных)
  - [📊 Батчинг операций](#-батчинг-операций)
  - [🔍 Мониторинг и отладка](#-мониторинг-и-отладка)
  - [🚀 Полный пример: Офлайн-приложение](#-полный-пример-офлайн-приложение)
  - [🛠️ Создание собственного адаптера](#️-создание-собственного-адаптера)
  - [🎯 Заключение](#-заключение)

## 🔧 Установка зависимостей

```bash
# Для IndexedDB поддержки
npm install idb

# Для LocalForage адаптера
npm install localforage

# Для сжатия данных
npm install pako

# Для дополнительных драйверов LocalForage
npm install localforage-driver-memory
```

## 🏗️ Архитектура системы

```typescript
import { 
  PersistenceManagerImpl, 
  IndexedDBAdapter, 
  LocalForageAdapter, 
  MemoryStorageAdapter 
} from 'cs-element';

// Основные интерфейсы
interface PersistenceAdapter {
  name: string;
  initialize(): Promise<void>;
  save(key: string, data: any): Promise<void>;
  load(key: string): Promise<any>;
  delete(key: string): Promise<void>;
  exists(key: string): Promise<boolean>;
  clear(): Promise<void>;
  keys(): Promise<string[]>;
}

interface PersistenceConfig {
  autoSave: boolean;
  saveInterval: number;
  batchSize: number;
  enableCompression: boolean;
  compressionThreshold: number;
  enableMetrics: boolean;
}
```

## 📦 Встроенные адаптеры

### IndexedDB Адаптер

Высокопроизводительное хранилище для браузеров:

```typescript
import { IndexedDBAdapter } from 'cs-element';

const indexedDBAdapter = new IndexedDBAdapter({
  dbName: 'MyAppDB',
  version: 1,
  stores: ['elements', 'metadata', 'relationships', 'cache'],
  upgradeHandler: (db, oldVersion, newVersion) => {
    console.log(`Обновление БД с версии ${oldVersion} до ${newVersion}`);
    
    // Создание индексов
    if (oldVersion < 1) {
      const elementsStore = db.createObjectStore('elements', { keyPath: 'id' });
      elementsStore.createIndex('type', 'type', { unique: false });
      elementsStore.createIndex('timestamp', 'timestamp', { unique: false });
    }
  }
});

// Инициализация
await indexedDBAdapter.initialize();

// Сохранение элемента
const element = new CSElement('user-123');
await element.setData('name', 'John Doe');
await element.setData('email', 'john@example.com');

await indexedDBAdapter.save('user-123', element.serialize());

// Загрузка элемента
const savedData = await indexedDBAdapter.load('user-123');
const restoredElement = CSElement.deserialize(savedData);

console.log('Восстановленный элемент:', await restoredElement.getData('name'));
```

### LocalForage Адаптер

Универсальный адаптер с поддержкой различных драйверов:

```typescript
import { LocalForageAdapter } from 'cs-element';

const localForageAdapter = new LocalForageAdapter({
  name: 'CSElementStorage',
  storeName: 'elements',
  driver: ['indexeddb', 'websql', 'localstorage'], // Приоритет драйверов
  version: 1.0,
  size: 4980736, // Размер в байтах для WebSQL
  description: 'CSElement persistent storage'
});

// Инициализация с проверкой поддержки
await localForageAdapter.initialize();

console.log('Используемый драйвер:', localForageAdapter.getCurrentDriver());

// Работа с большими объектами
const largeData = {
  elements: Array(1000).fill(null).map((_, i) => ({
    id: `element-${i}`,
    data: { value: Math.random() * 1000 },
    timestamp: Date.now()
  }))
};

await localForageAdapter.save('large-dataset', largeData);

// Асинхронная загрузка
const loadedData = await localForageAdapter.load('large-dataset');
console.log(`Загружено элементов: ${loadedData.elements.length}`);

// Получение всех ключей
const keys = await localForageAdapter.keys();
console.log('Сохраненные ключи:', keys);
```

### Memory Storage Адаптер

Быстрое хранилище в памяти для тестов и кэширования:

```typescript
import { MemoryStorageAdapter } from 'cs-element';

const memoryAdapter = new MemoryStorageAdapter('cache', {
  maxSize: 1000, // Максимальное количество элементов
  ttl: 300000, // Время жизни в миллисекундах (5 минут)
  enableMetrics: true
});

await memoryAdapter.initialize();

// Сохранение с TTL
await memoryAdapter.save('temp-data', { value: 'temporary' });

// Проверка существования
const exists = await memoryAdapter.exists('temp-data');
console.log('Данные существуют:', exists);

// Получение метрик
const metrics = memoryAdapter.getMetrics();
console.log('Метрики памяти:', {
  size: metrics.size,
  hits: metrics.hits,
  misses: metrics.misses,
  hitRate: metrics.hitRate
});
```

## ⚙️ Настройка PersistenceManager

```typescript
import { PersistenceManagerImpl } from 'cs-element';

// Создание менеджера с IndexedDB
const indexedDBAdapter = new IndexedDBAdapter({
  dbName: 'ProductionDB',
  version: 2,
  stores: ['elements', 'metadata', 'cache', 'backups']
});

const persistenceManager = new PersistenceManagerImpl(indexedDBAdapter, {
  autoSave: true,
  saveInterval: 10000, // Автосохранение каждые 10 секунд
  batchSize: 50, // Размер батча для операций
  enableCompression: true,
  compressionThreshold: 1024, // Сжимать данные больше 1KB
  enableMetrics: true
});

// Инициализация
await persistenceManager.initialize();

// Регистрация дополнительных адаптеров
const cacheAdapter = new MemoryStorageAdapter('cache');
persistenceManager.registerAdapter('cache', cacheAdapter);

// Настройка стратегий сохранения
persistenceManager.setStrategy('user-data', {
  adapter: 'default',
  compression: true,
  autoSave: true,
  backup: true
});

persistenceManager.setStrategy('cache-data', {
  adapter: 'cache',
  compression: false,
  autoSave: false,
  backup: false
});
```

## 💾 Базовые операции

```typescript
// Создание элементов
const user = new CSElement('user-456');
await user.setData('profile', {
  name: 'Alice Johnson',
  email: 'alice@example.com',
  preferences: {
    theme: 'dark',
    language: 'ru',
    notifications: true
  }
});

const project = new CSElement('project-789');
await project.setData('info', {
  name: 'Важный проект',
  status: 'active',
  deadline: '2024-12-31'
});

// Сохранение отдельных элементов
await persistenceManager.save('user-456', user);
await persistenceManager.save('project-789', project);

// Сохранение коллекции элементов
const elements = [user, project];
await persistenceManager.saveMany(elements);

// Загрузка элементов
const loadedUser = await persistenceManager.load('user-456');
console.log('Загруженный пользователь:', await loadedUser.getData('profile'));

// Загрузка нескольких элементов
const loadedElements = await persistenceManager.loadMany(['user-456', 'project-789']);
console.log(`Загружено элементов: ${loadedElements.length}`);

// Проверка существования
const userExists = await persistenceManager.exists('user-456');
console.log('Пользователь существует:', userExists);

// Удаление элементов
await persistenceManager.delete('user-456');

// Получение всех ключей
const allKeys = await persistenceManager.keys();
console.log('Все сохраненные ключи:', allKeys);
```

## 🔄 Автоматическая синхронизация

```typescript
// Настройка автоматической синхронизации
const syncManager = new SyncManager(persistenceManager, {
  syncInterval: 30000, // Синхронизация каждые 30 секунд
  conflictResolution: 'client-wins', // Стратегия разрешения конфликтов
  enableOfflineMode: true,
  maxRetries: 3
});

// Регистрация элементов для синхронизации
const syncableUser = new CSElement('sync-user-1');
await syncableUser.setData('name', 'Sync User');
await syncableUser.setData('lastModified', Date.now());

// Добавление в очередь синхронизации
syncManager.addToSyncQueue('sync-user-1', syncableUser);

// Обработка событий синхронизации
syncManager.on('sync:start', (elementId) => {
  console.log(`🔄 Начинается синхронизация ${elementId}`);
});

syncManager.on('sync:success', (elementId, data) => {
  console.log(`✅ Синхронизация ${elementId} успешна`);
});

syncManager.on('sync:conflict', (elementId, localData, remoteData) => {
  console.log(`⚠️ Конфликт синхронизации ${elementId}`);
  
  // Ручное разрешение конфликта
  const resolved = mergeConflictData(localData, remoteData);
  syncManager.resolveConflict(elementId, resolved);
});

syncManager.on('sync:error', (elementId, error) => {
  console.error(`❌ Ошибка синхронизации ${elementId}:`, error);
});

// Запуск синхронизации
await syncManager.start();

// Функция слияния данных при конфликте
function mergeConflictData(local: any, remote: any): any {
  return {
    ...remote,
    ...local,
    lastModified: Math.max(local.lastModified, remote.lastModified),
    conflictResolved: true,
    resolvedAt: Date.now()
  };
}
```

## 🗜️ Сжатие данных

```typescript
// Настройка сжатия для больших данных
const compressionConfig = {
  enableCompression: true,
  compressionThreshold: 2048, // Сжимать данные больше 2KB
  algorithm: 'gzip' as const,
  level: 6 // Уровень сжатия от 1 до 9
};

const compressedPersistence = new PersistenceManagerImpl(indexedDBAdapter, {
  ...compressionConfig,
  autoSave: true,
  saveInterval: 15000
});

// Создание больших данных
const largeElement = new CSElement('large-data-set');
const bigData = {
  items: Array(10000).fill(null).map((_, i) => ({
    id: i,
    name: `Item ${i}`,
    description: `This is a detailed description for item ${i}`.repeat(10),
    metadata: {
      created: new Date().toISOString(),
      tags: [`tag-${i % 100}`, `category-${i % 10}`],
      properties: Object.fromEntries(
        Array(20).fill(null).map((_, j) => [`prop${j}`, Math.random()])
      )
    }
  }))
};

await largeElement.setData('bigData', bigData);

console.log('Размер данных до сжатия:', JSON.stringify(bigData).length);

// Сохранение с автоматическим сжатием
await compressedPersistence.save('large-data-set', largeElement);

// Загрузка с автоматической декомпрессией
const loadedLargeElement = await compressedPersistence.load('large-data-set');
const decompressedData = await loadedLargeElement.getData('bigData');

console.log('Данные успешно декомпрессированы:', decompressedData.items.length);

// Получение статистики сжатия
const compressionStats = compressedPersistence.getCompressionStats();
console.log('Статистика сжатия:', {
  totalSaved: compressionStats.totalSaved,
  totalCompressed: compressionStats.totalCompressed,
  compressionRatio: compressionStats.compressionRatio,
  averageCompressionTime: compressionStats.averageCompressionTime
});
```

## 📊 Батчинг операций

```typescript
// Настройка батчинга для оптимизации производительности
const batchPersistence = new PersistenceManagerImpl(indexedDBAdapter, {
  batchSize: 100,
  batchTimeout: 5000, // Максимальное время ожидания батча
  enableBatching: true,
  autoSave: true
});

// Создание множества элементов
const elements = Array(500).fill(null).map((_, i) => {
  const element = new CSElement(`batch-element-${i}`);
  element.setData('index', i);
  element.setData('data', `Data for element ${i}`);
  element.setData('timestamp', Date.now());
  return element;
});

console.log('🚀 Начинаем батчевое сохранение...');

// Батчевое сохранение
const startTime = Date.now();
await batchPersistence.saveMany(elements);
const endTime = Date.now();

console.log(`✅ Сохранено ${elements.length} элементов за ${endTime - startTime}мс`);

// Батчевая загрузка
const keys = elements.map(e => e.id);
const loadStartTime = Date.now();
const loadedElements = await batchPersistence.loadMany(keys);
const loadEndTime = Date.now();

console.log(`✅ Загружено ${loadedElements.length} элементов за ${loadEndTime - loadStartTime}мс`);

// Получение статистики батчинга
const batchStats = batchPersistence.getBatchStats();
console.log('Статистика батчинга:', {
  totalBatches: batchStats.totalBatches,
  averageBatchSize: batchStats.averageBatchSize,
  averageBatchTime: batchStats.averageBatchTime,
  totalElementsProcessed: batchStats.totalElementsProcessed
});
```

## 🔍 Мониторинг и отладка

```typescript
// Создание системы мониторинга
class PersistenceMonitor {
  private persistence: PersistenceManagerImpl;
  private metrics: {
    operations: Array<{
      type: 'save' | 'load' | 'delete';
      key: string;
      duration: number;
      success: boolean;
      timestamp: number;
      size?: number;
    }>;
    errors: Array<{
      operation: string;
      error: string;
      timestamp: number;
    }>;
  };

  constructor(persistence: PersistenceManagerImpl) {
    this.persistence = persistence;
    this.metrics = { operations: [], errors: [] };
    this.setupEventListeners();
  }

  private setupEventListeners(): void {
    // Мониторинг операций сохранения
    this.persistence.on('save:start', (key) => {
      console.log(`💾 Начинается сохранение ${key}`);
    });

    this.persistence.on('save:success', (key, duration, size) => {
      console.log(`✅ Сохранение ${key} завершено за ${duration}мс (${size} байт)`);
      this.recordOperation('save', key, duration, true, size);
    });

    this.persistence.on('save:error', (key, error) => {
      console.error(`❌ Ошибка сохранения ${key}:`, error);
      this.recordOperation('save', key, 0, false);
      this.recordError('save', error);
    });

    // Мониторинг операций загрузки
    this.persistence.on('load:start', (key) => {
      console.log(`📖 Начинается загрузка ${key}`);
    });

    this.persistence.on('load:success', (key, duration, size) => {
      console.log(`✅ Загрузка ${key} завершена за ${duration}мс (${size} байт)`);
      this.recordOperation('load', key, duration, true, size);
    });

    this.persistence.on('load:error', (key, error) => {
      console.error(`❌ Ошибка загрузки ${key}:`, error);
      this.recordOperation('load', key, 0, false);
      this.recordError('load', error);
    });
  }

  private recordOperation(
    type: 'save' | 'load' | 'delete',
    key: string,
    duration: number,
    success: boolean,
    size?: number
  ): void {
    this.metrics.operations.push({
      type,
      key,
      duration,
      success,
      timestamp: Date.now(),
      size
    });

    // Ограничиваем размер истории
    if (this.metrics.operations.length > 1000) {
      this.metrics.operations = this.metrics.operations.slice(-1000);
    }
  }

  private recordError(operation: string, error: string): void {
    this.metrics.errors.push({
      operation,
      error,
      timestamp: Date.now()
    });

    if (this.metrics.errors.length > 100) {
      this.metrics.errors = this.metrics.errors.slice(-100);
    }
  }

  getPerformanceReport(): {
    summary: {
      totalOperations: number;
      successRate: number;
      averageOperationTime: number;
      totalDataSize: number;
      operationsPerSecond: number;
    };
    byType: Record<string, any>;
    recommendations: string[];
  } {
    const recentOps = this.metrics.operations
      .filter(op => Date.now() - op.timestamp < 300000); // Последние 5 минут

    const successful = recentOps.filter(op => op.success);
    const totalDuration = recentOps.reduce((sum, op) => sum + op.duration, 0);
    const totalSize = recentOps.reduce((sum, op) => sum + (op.size || 0), 0);

    const summary = {
      totalOperations: recentOps.length,
      successRate: recentOps.length > 0 ? successful.length / recentOps.length : 0,
      averageOperationTime: recentOps.length > 0 ? totalDuration / recentOps.length : 0,
      totalDataSize: totalSize,
      operationsPerSecond: recentOps.length / 300
    };

    const byType = ['save', 'load', 'delete'].reduce((acc, type) => {
      const typeOps = recentOps.filter(op => op.type === type);
      acc[type] = {
        count: typeOps.length,
        successRate: typeOps.length > 0 ? typeOps.filter(op => op.success).length / typeOps.length : 0,
        averageTime: typeOps.length > 0 ? typeOps.reduce((sum, op) => sum + op.duration, 0) / typeOps.length : 0
      };
      return acc;
    }, {} as Record<string, any>);

    const recommendations = [];
    if (summary.successRate < 0.95) {
      recommendations.push('Высокий процент ошибок, проверьте подключение к хранилищу');
    }
    if (summary.averageOperationTime > 1000) {
      recommendations.push('Медленные операции, рассмотрите оптимизацию или сжатие');
    }
    if (byType.save.averageTime > byType.load.averageTime * 2) {
      recommendations.push('Сохранение значительно медленнее загрузки, включите сжатие');
    }

    return { summary, byType, recommendations };
  }

  exportMetrics(): string {
    return JSON.stringify({
      timestamp: new Date().toISOString(),
      metrics: this.metrics,
      report: this.getPerformanceReport()
    }, null, 2);
  }
}

// Использование монитора
const monitor = new PersistenceMonitor(persistenceManager);

// Получение отчета через 5 минут
setTimeout(() => {
  const report = monitor.getPerformanceReport();
  console.log('📊 Отчет о производительности персистентности:', report);
}, 300000);
```

## 🚀 Полный пример: Офлайн-приложение

```typescript
import { 
  PersistenceManagerImpl, 
  IndexedDBAdapter, 
  LocalForageAdapter,
  MemoryStorageAdapter 
} from 'cs-element';

class OfflineApplication {
  private persistence: PersistenceManagerImpl;
  private cache: PersistenceManagerImpl;
  private syncQueue: Array<{
    id: string;
    operation: 'save' | 'delete';
    data?: any;
    timestamp: number;
  }> = [];
  private isOnline: boolean = navigator.onLine;

  constructor() {
    this.initializePersistence();
    this.setupNetworkMonitoring();
  }

  private async initializePersistence(): Promise<void> {
    // Основное хранилище
    const mainAdapter = new IndexedDBAdapter({
      dbName: 'OfflineAppDB',
      version: 1,
      stores: ['documents', 'users', 'settings', 'sync_queue']
    });

    this.persistence = new PersistenceManagerImpl(mainAdapter, {
      autoSave: true,
      saveInterval: 10000,
      enableCompression: true,
      batchSize: 50
    });

    // Кэш в памяти
    const cacheAdapter = new MemoryStorageAdapter('cache', {
      maxSize: 500,
      ttl: 600000 // 10 минут
    });

    this.cache = new PersistenceManagerImpl(cacheAdapter, {
      autoSave: false,
      enableCompression: false
    });

    await this.persistence.initialize();
    await this.cache.initialize();

    // Восстановление очереди синхронизации
    await this.restoreSyncQueue();
  }

  private setupNetworkMonitoring(): void {
    window.addEventListener('online', () => {
      console.log('🌐 Соединение восстановлено');
      this.isOnline = true;
      this.processSyncQueue();
    });

    window.addEventListener('offline', () => {
      console.log('📴 Соединение потеряно');
      this.isOnline = false;
    });
  }

  async createDocument(id: string, data: any): Promise<void> {
    const document = new CSElement(id);
    await document.setData('content', data);
    await document.setData('created', new Date().toISOString());
    await document.setData('modified', new Date().toISOString());
    await document.setData('synced', false);

    // Сохранение в основное хранилище
    await this.persistence.save(id, document);

    // Кэширование для быстрого доступа
    await this.cache.save(id, document);

    // Добавление в очередь синхронизации
    this.addToSyncQueue(id, 'save', document);

    console.log(`📄 Документ ${id} создан`);
  }

  async getDocument(id: string): Promise<CSElement | null> {
    // Сначала проверяем кэш
    try {
      const cached = await this.cache.load(id);
      if (cached) {
        console.log(`⚡ Документ ${id} загружен из кэша`);
        return cached;
      }
    } catch (error) {
      console.log(`Документ ${id} не найден в кэше`);
    }

    // Загружаем из основного хранилища
    try {
      const document = await this.persistence.load(id);
      
      // Кэшируем для следующего раза
      await this.cache.save(id, document);
      
      console.log(`💾 Документ ${id} загружен из хранилища`);
      return document;
    } catch (error) {
      console.error(`❌ Документ ${id} не найден:`, error);
      return null;
    }
  }

  async updateDocument(id: string, data: any): Promise<void> {
    const document = await this.getDocument(id);
    if (!document) {
      throw new Error(`Документ ${id} не найден`);
    }

    await document.setData('content', data);
    await document.setData('modified', new Date().toISOString());
    await document.setData('synced', false);

    // Обновляем в хранилище и кэше
    await this.persistence.save(id, document);
    await this.cache.save(id, document);

    // Добавляем в очередь синхронизации
    this.addToSyncQueue(id, 'save', document);

    console.log(`📝 Документ ${id} обновлен`);
  }

  async deleteDocument(id: string): Promise<void> {
    // Удаляем из хранилища и кэша
    await this.persistence.delete(id);
    await this.cache.delete(id);

    // Добавляем в очередь синхронизации
    this.addToSyncQueue(id, 'delete');

    console.log(`🗑️ Документ ${id} удален`);
  }

  async listDocuments(): Promise<string[]> {
    const keys = await this.persistence.keys();
    return keys.filter(key => key.startsWith('doc-'));
  }

  private addToSyncQueue(id: string, operation: 'save' | 'delete', data?: any): void {
    this.syncQueue.push({
      id,
      operation,
      data: data?.serialize(),
      timestamp: Date.now()
    });

    // Сохраняем очередь синхронизации
    this.persistence.save('sync_queue', this.syncQueue);

    // Если онлайн, пытаемся синхронизировать
    if (this.isOnline) {
      this.processSyncQueue();
    }
  }

  private async processSyncQueue(): Promise<void> {
    if (!this.isOnline || this.syncQueue.length === 0) {
      return;
    }

    console.log(`🔄 Обработка очереди синхронизации (${this.syncQueue.length} элементов)`);

    const processed = [];
    
    for (const item of this.syncQueue) {
      try {
        await this.syncItem(item);
        processed.push(item);
        console.log(`✅ Синхронизирован ${item.id}`);
      } catch (error) {
        console.error(`❌ Ошибка синхронизации ${item.id}:`, error);
        break; // Прерываем при первой ошибке
      }
    }

    // Удаляем обработанные элементы из очереди
    this.syncQueue = this.syncQueue.filter(item => !processed.includes(item));
    
    // Сохраняем обновленную очередь
    await this.persistence.save('sync_queue', this.syncQueue);

    console.log(`🎯 Обработано ${processed.length} элементов, осталось ${this.syncQueue.length}`);
  }

  private async syncItem(item: any): Promise<void> {
    // Имитация синхронизации с сервером
    const response = await fetch('/api/sync', {
      method: 'POST',
      headers: { 'Content-Type': 'application/json' },
      body: JSON.stringify(item)
    });

    if (!response.ok) {
      throw new Error(`Ошибка синхронизации: ${response.status}`);
    }

    // Помечаем элемент как синхронизированный
    if (item.operation === 'save' && item.data) {
      const element = CSElement.deserialize(item.data);
      await element.setData('synced', true);
      await this.persistence.save(item.id, element);
    }
  }

  private async restoreSyncQueue(): Promise<void> {
    try {
      const savedQueue = await this.persistence.load('sync_queue');
      if (savedQueue && Array.isArray(savedQueue)) {
        this.syncQueue = savedQueue;
        console.log(`🔄 Восстановлена очередь синхронизации (${this.syncQueue.length} элементов)`);
      }
    } catch (error) {
      console.log('Очередь синхронизации пуста');
    }
  }

  async getStatus(): Promise<{
    isOnline: boolean;
    syncQueueSize: number;
    cacheSize: number;
    totalDocuments: number;
  }> {
    const documents = await this.listDocuments();
    const cacheKeys = await this.cache.keys();
    
    return {
      isOnline: this.isOnline,
      syncQueueSize: this.syncQueue.length,
      cacheSize: cacheKeys.length,
      totalDocuments: documents.length
    };
  }

  async exportData(): Promise<Blob> {
    const documents = await this.listDocuments();
    const exportData = {
      timestamp: new Date().toISOString(),
      documents: await Promise.all(
        documents.map(async (id) => {
          const doc = await this.persistence.load(id);
          return { id, data: doc.serialize() };
        })
      ),
      syncQueue: this.syncQueue
    };

    return new Blob([JSON.stringify(exportData, null, 2)], {
      type: 'application/json'
    });
  }
}

// Использование приложения
async function main() {
  const app = new OfflineApplication();

  // Создание документов
  await app.createDocument('doc-1', { title: 'Документ 1', content: 'Содержимое документа 1' });
  await app.createDocument('doc-2', { title: 'Документ 2', content: 'Содержимое документа 2' });

  // Получение документа
  const doc1 = await app.getDocument('doc-1');
  console.log('Загруженный документ:', await doc1?.getData('content'));

  // Обновление документа
  await app.updateDocument('doc-1', { title: 'Обновленный документ 1', content: 'Новое содержимое' });

  // Получение статуса
  const status = await app.getStatus();
  console.log('Статус приложения:', status);

  // Экспорт данных
  const exportBlob = await app.exportData();
  console.log(`Экспорт готов, размер: ${exportBlob.size} байт`);
}

main().catch(console.error);
```

## 🛠️ Создание собственного адаптера

```typescript
import { PersistenceAdapter } from 'cs-element';

class CustomFirebaseAdapter implements PersistenceAdapter {
  public readonly name = 'firebase';
  private db: any; // Firebase Firestore instance
  private collection: string;

  constructor(config: {
    firebaseConfig: any;
    collection: string;
  }) {
    this.collection = config.collection;
    // Инициализация Firebase
    // this.db = firebase.firestore();
  }

  async initialize(): Promise<void> {
    // Инициализация соединения с Firebase
    console.log(`Инициализация Firebase адаптера для коллекции: ${this.collection}`);
  }

  async save(key: string, data: any): Promise<void> {
    try {
      await this.db.collection(this.collection).doc(key).set({
        data: JSON.stringify(data),
        timestamp: new Date(),
        version: 1
      });
    } catch (error) {
      throw new Error(`Ошибка сохранения в Firebase: ${error}`);
    }
  }

  async load(key: string): Promise<any> {
    try {
      const doc = await this.db.collection(this.collection).doc(key).get();
      if (!doc.exists) {
        throw new Error(`Документ ${key} не найден`);
      }
      return JSON.parse(doc.data().data);
    } catch (error) {
      throw new Error(`Ошибка загрузки из Firebase: ${error}`);
    }
  }

  async delete(key: string): Promise<void> {
    try {
      await this.db.collection(this.collection).doc(key).delete();
    } catch (error) {
      throw new Error(`Ошибка удаления из Firebase: ${error}`);
    }
  }

  async exists(key: string): Promise<boolean> {
    try {
      const doc = await this.db.collection(this.collection).doc(key).get();
      return doc.exists;
    } catch (error) {
      return false;
    }
  }

  async clear(): Promise<void> {
    const batch = this.db.batch();
    const snapshot = await this.db.collection(this.collection).get();
    
    snapshot.docs.forEach((doc: any) => {
      batch.delete(doc.ref);
    });
    
    await batch.commit();
  }

  async keys(): Promise<string[]> {
    const snapshot = await this.db.collection(this.collection).get();
    return snapshot.docs.map((doc: any) => doc.id);
  }
}

// Использование собственного адаптера
const firebaseAdapter = new CustomFirebaseAdapter({
  firebaseConfig: {
    // Конфигурация Firebase
  },
  collection: 'cs-elements'
});

const firebasePersistence = new PersistenceManagerImpl(firebaseAdapter, {
  autoSave: true,
  saveInterval: 20000,
  enableCompression: true
});

await firebasePersistence.initialize();
```

## 🎯 Заключение

Система персистентности CSElement теперь предоставляет:

- **🏪 Реальные адаптеры** для IndexedDB, LocalForage и Memory Storage
- **🔄 Автоматическую синхронизацию** с разрешением конфликтов
- **🗜️ Сжатие данных** для экономии места
- **📊 Батчинг операций** для оптимизации производительности
- **🔍 Мониторинг и отладку** с детальной аналитикой
- **📴 Офлайн-поддержку** с очередью синхронизации
- **🛠️ Расширяемость** для создания собственных адаптеров

Все заглушки заменены на реальные реализации, система готова для производственного использования в любых условиях!

**Следующие разделы:**
- [Система состояний](./11-state-machine.md)
- [Diff Engine](./12-diff-engine.md)
- [Blueprint система](./13-blueprint-system.md) 